package graphics2.listener;

import graphics2.*;
import graphics2.listener.generator.*;
import multimedia.generation.*;
import org.w3c.dom.*;
import utils.*;
import java.awt.event.*;
import java.io.IOException;
import java.lang.reflect.*;

/**
 * ScenariosGeneratorActionListener
 *
 * @author mladoue
 * @version 0.1, 17/01/03
 */
public class ScenariosGeneratorActionListener implements ActionListener {
	
    protected GestionTest gestionTest;
    protected GenericGeneration[] genericGenerator;
    protected String[] classGeneratorName;
    protected ModelGeneratorConstructor[] mal;
    protected int scenarioNumber = 0;
    private ConfigurationXMLFile cxf;
    protected NodeList nodesFlow;
    protected NodeList nlModelD;
    protected Element nFlowD;
    protected Element nCodecD;
	protected boolean generateStats;

	/**
	 * 
	 * @param gestionTest a reference to the main Panel of the graphical interface.
	 * @param generateStats is true if we want to generate statistical files instead of trace files
	 */
    public ScenariosGeneratorActionListener(GestionTest gestionTest, boolean generateStats) {
        this.gestionTest = gestionTest;
        this.generateStats = generateStats;
    }

    public void actionPerformed(ActionEvent e) {
        // on bloque les valueurs choisies
        gestionTest.freezeAll(true);

        // Get XML infos only once the menu is selected
        cxf = gestionTest.getClientPanel().getListScenario();
        scenarioNumber = cxf.numberOfScenario();

        genericGenerator = new GenericGeneration[scenarioNumber];
        classGeneratorName = new String[scenarioNumber];
        mal = new ModelGeneratorConstructor[scenarioNumber];

        // On recupere les donnees du doc XML
        Document document = cxf.getDocument();
        nodesFlow = document.getElementsByTagName("Flow");
        System.out.println("--> " + nodesFlow.getLength());	//DEBUG

        for (int i = 0; i < scenarioNumber; i++) {
            // Set for the current flow, its model and its codes (if available, ie not an aggregated flow)
            setXMLParameters(i);
			try {
				// TODO. Add support for aggregated.
				Class modelALDefinition = Class.forName("graphics2.listener.generator." +classGeneratorName[i] + "GeneratorConstructor");
				Class[] intArgClass = new Class[] {	GestionTest.class, Element.class };
				Object[] intArgs = new Object[] { gestionTest, nFlowD };

				Constructor intArgsConstructor;
				intArgsConstructor = modelALDefinition.getConstructor(intArgClass);

				mal[i] = (ModelGeneratorConstructor)ObjectConstructor.createObject(intArgsConstructor, intArgs);

				mal[i].constructModelGenerator();

				} 
				catch (IOException ioe) { ioe.printStackTrace();	}
               	
				catch (ClassNotFoundException cnfe) { System.out.println(cnfe); } 
               
				catch (NoSuchMethodException nsme) { System.out.println(nsme); }

				//System.out.println("ici");	//DEBUG
                genericGenerator[i] = mal[i].getGenerator();

                 try {
                 	if (generateStats) {
						genericGenerator[i].statisticsGeneration();
                 	}
                 	else {
                 		genericGenerator[i].scenarioGeneration();

	                    // Partie destiner a mettre en forme le fichier resultat
    	                //String rfnComplete = genericGenerator[i].getResultFilePath();
						String rfnComplete = genericGenerator[i].getResultFileName();
						System.out.println("SCENARIO NAME: " + rfnComplete ); //DEBUG
                	   	String tmp = "scenario.tmp";
                    	String[] rfn = rfnComplete.split("/", 30); //on suppose qu'on n'a pas plus de 30 "/" pour definir le path du chemin
	                    int rfnLength = rfn.length;
						// TODO verifier ce bout de code
                    	if (rfnLength != 1) {
                    		//MyNumberFormat.timeSizeFormatage(rfn[rfnLength-1],rfn[rfnLength-1]+".scf");
	                        MyNumberFormat.timeSizeFormatage(tmp, rfn[rfnLength - 1]);
    	                } 
        	            else {
            	        	//MyNumberFormat.timeSizeFormatage(rfnComplete, rfnComplete+".scf");
                	        MyNumberFormat.timeSizeFormatage(tmp, rfnComplete);
                    	}
                 	}
               	}
               	catch (IOException ioe) {
					ioe.printStackTrace();
           		}
        }

        // on debloque les valueurs choisies : pour permettre
        // par exemple de creer de nouveaux fichiers scenario
        gestionTest.freezeAll(false);
    }

    public String[] getClassGeneratorName() {
        return classGeneratorName;
    }

    public GestionTest getGestionTest() {
        return gestionTest;
    }

    /**
     * Permit to position classGeneratorName and nFlowD, nlModelD and nCodecD for each flow type
     * @param scenarioNb flow number in the XML file (it isn't the attribute number of Flow)
     */
    protected void setXMLParameters(int scenarioNb) {
        //TODO: Attention, ne supporte pas encore les modeles aggreges !!!!!!!!!!!!!!!!!!!!!!!!!
        // Ces derniers n'ont pas de codec mais une liste de submodel avec des codecs.
        // Positionne pour le flow i ses differentes caracteristiques
        nFlowD = (Element) nodesFlow.item(scenarioNb);

        // Get the model.
        NodeList nl = nFlowD.getChildNodes();

        for (int i = 0; i < nl.getLength(); i++) {
            if (((Element) nl.item(i)).getTagName() == "model") {
                classGeneratorName[scenarioNb] = (((Element) nl.item(i)).getAttribute("name"));
				nlModelD = nFlowD.getElementsByTagName( "model");
            }

            if (((Element) nl.item(i)).getTagName() == "codec") {
                nCodecD = ((Element) nl.item(i));
            }
        }
    }

    /**
     * @return int
     */
    public int getScenarioNumber() {
        return scenarioNumber;
    }

    /**
    * Returns list of flowsDescription
    * @return Element
    */
    public NodeList getFlowList() {
        return nodesFlow;
    }
}
