package graphics2.listener.generator;

import graphics2.*;
import java.io.IOException;

import multimedia.generation.*;
import multimedia.model.*;
import org.w3c.dom.Element;
import statistic.distribution.*;
import statistic.function.*;

/**
 * MGInfComplexGeneratorConstructor : Defined to create an MGInfComplex Model and a MGInfComplex Generator
 *
 * @author mladoue
 * @version 0.1, 05/02/03
 */

public class MGInfComplexGeneratorConstructor extends MGInfGeneratorConstructor {

	protected String movieName;
	
	   /**
     * MGInfComplexGeneratorConstructor
     * @see graphics.listener.ModelGeneratorConstructor#ModelGeneratorConstructor(GestionTest)
     */
    public MGInfComplexGeneratorConstructor(GestionTest gt, Element nlFlow) {
        super(gt, nlFlow);

        fileName = sfp.createScenarioFileName(cp);
        //fileName = "MGInfComplex_60";
        movieName = cp.getModelPanel().getInterneModelPanel().getModelName();
        unit = 0; // fixer pour l'instant en bytes uniquement
        
//		System.out.println("name function " + mf.getEquation() + " => " + autoCorr[0].getName()); // DEBUG
//		System.out.println("weight function " + ((MixedFunction)mf).getWeight()[0] + " " +mf.getName()+ " " + mf.getName());	 // DEBUG
//		System.out.println("parameter function " + ((MixedFunction)mf).getFunctionsParameter()[0] + " " + ((MixedFunction)mf).getFunctionsParameter()[1]);	 // DEBUG
//		System.out.println("name distributions " + ((MixedDistribution)distributionTest).getSpecificDistribution(0).getName() + "  " +((MixedDistribution)distributionSlot).getSpecificDistribution(0).getName() + " "  + ((MixedDistribution)distributionPacketSize).getSpecificDistribution(0).getName());	 // DEBUG
//		System.out.println("parameter distributions " + ((MixedDistribution)distributionTest).getSpecificDistribution(0).getParam(0) + " , " +((MixedDistribution)distributionSlot).getSpecificDistribution(0).getParam(0) + " , "  + ((MixedDistribution)distributionPacketSize).getSpecificDistribution(0).getParam(0));	 // DEBUG
//		 System.out.println("weight distributions " + ((MixedDistribution)distributionTest).getWeight(0) + " , " +((MixedDistribution)distributionSlot).getWeight(0) + " , "  + ((MixedDistribution)distributionPacketSize).getWeight(0));	 // DEBUG
//		 System.out.println("duration " + duration);	 // DEBUG
//		 System.out.println("codec " + codec.getName());	 // DEBUG
//		 System.out.println("movieName " + movieName + " fileName " + fileName);	 // DEBUG
    }

	protected GenericContinuousDistribution getDistribution(Element elt) {
		
		GenericContinuousDistribution distribution1=null;
		GenericContinuousDistribution distribution2=null;
		
		if (elt.getAttribute("mixed").equals("true")) {
			String name = elt.getAttribute("name2");
			try {
			  distribution2 = (GenericContinuousDistribution)Class.forName("statistic.distribution." + name+"Distribution").newInstance();
			}
			catch (IllegalAccessException iae) {
				   System.out.println(iae);
			}
		   catch(InstantiationException ie) { 
			   System.err.println("MGInfComplexGeneratorConstructor.getDistribution : the initialization provoked by this methode fails => Distribution Name : " + name + " "  + ie);
			}
			catch (ClassNotFoundException cnfe) {
			   System.err.println("MGInfComplexGeneratorConstructor.getDistribution : the class cannot be located => Distribution Name : " + name + " " + cnfe);
			}
				// Set the distribution parameters.
			double[] params;
			if (elt.getAttribute("param22").equals("")) {
				params = new double[1];
			}
			else {
				params = new double[2];
				params[1]=Double.valueOf(elt.getAttribute("param22")).doubleValue();
			}
			params[0]=Double.valueOf(elt.getAttribute("param21")).doubleValue();
			distribution2.setParams( params);
		}
			// Do the first distribution...
		String name1 = elt.getAttribute("name1");
		try {
		  distribution1 = (GenericContinuousDistribution)Class.forName("statistic.distribution." + name1+"Distribution").newInstance();
		}
		catch (IllegalAccessException iae) {
			   System.out.println(iae);
		}
	   catch(InstantiationException ie) { 
		   System.err.println("MGInfComplexGeneratorConstructor.getDistribution : the initialization provoked by this methode fails => Distribution Name : " + name1 + " "  + ie);
		}
		catch (ClassNotFoundException cnfe) {
		   System.err.println("MGInfComplexGeneratorConstructor.getDistribution : the class cannot be located => Distribution Name : " + name1 + " " + cnfe);
		}
			// Set the distribution parameters.
		double[] params;
		if (elt.getAttribute("param12").equals("")) {
			params = new double[1];
		}
		else {
			params = new double[2];
			params[1]=Double.valueOf(elt.getAttribute("param12")).doubleValue();
		}
		params[0]=Double.valueOf(elt.getAttribute("param11")).doubleValue();
		distribution1.setParams( params);

		if (distribution2 !=null) {
			GenericContinuousDistribution[] gd = new GenericContinuousDistribution[2];
			gd[0]=distribution1;
			gd[1]=distribution2;
			double[] w = new double[2];
			w[0]= Double.valueOf(elt.getAttribute("weight1")).doubleValue(); 
			w[1]= Double.valueOf(elt.getAttribute("weight2")).doubleValue(); 
			MixedDistribution md = new MixedDistribution(gd, w);
			return md;
		}
		else
			return distribution1;

	}
	
	protected void setModelParameters(Element elt) {
		org.w3c.dom.NodeList nlist = elt.getChildNodes() ;
		for (int i=0; i < nlist.getLength(); i++) {
			String name = nlist.item(i).getNodeName();
			if (name.equals("functions")) {
				// Get all function nodes...
				org.w3c.dom.NodeList fctlist = nlist.item(i).getChildNodes() ;
				int cmp=0;				
				double[] weights = new double[2];
				GenericFunction[] functs = new GenericFunction[2];
				
				for (int j=0; j < fctlist.getLength(); j++) {
					if (fctlist.item(j).getNodeName().equals("function")) {
						functs[cmp] =  getFunction((Element) fctlist.item(j));
				
						if (!((org.w3c.dom.Element)fctlist.item(j)).getAttribute("weight").equals(""))
							weights[cmp]= Double.valueOf(((org.w3c.dom.Element)fctlist.item(j)).getAttribute("weight")).doubleValue();
						cmp++; // count the number of function						
					}
				}
					// Create the mixedFunction...
				if (cmp==2)
					autoCorr = new MixedFunction(functs, weights);
				else if (cmp==1) { //TODO: To check if it's working...
					double[] weightsb = new double[1];
					GenericFunction[] functsb = new GenericFunction[1];
					weightsb[0]=weights[0];
					functsb[0]=functs[0];
					autoCorr = new MixedFunction(functsb, weightsb);
				}
				else
					System.err.println("Wrong number of function ("+cmp+") in MGInfComplex model");
			}
			else if(name.equals("lambda1")) {
				lambda = Double.valueOf(((org.w3c.dom.Element) nlist.item(i)).getAttribute("value")).doubleValue();
			}
			else if(name.equals("distribution1")) {
				distributionTest = getDistribution(((org.w3c.dom.Element)nlist.item(i)));
			}
			else if(name.equals("distribution2")) {
				distributionSlot = getDistribution(((org.w3c.dom.Element)nlist.item(i)));
			}
			else if(name.equals("distribution3")) {
				distributionPacketSize = getDistribution(((org.w3c.dom.Element)nlist.item(i)));
			}
		}
	}
//<<<<<<< MGInfComplexGeneratorConstructor.java
//	else  { distributionTest = mmp.getSpecifiedMixedDistributionPanel(0).getDistribution(0); }
//	
//	
//	if (mmp.getSpecifiedMixedDistributionPanel(1).getDistributions().length ==2) {
//	distributionSlot = mmp.getSpecifiedMixedDistributionPanel(1).getMixedDistribution();
//	((MixedDistribution)distributionSlot).setWeights(mmp.getSpecifiedMixedDistributionPanel(1).getDistributionsWeight());
//	}
//	else  { distributionSlot = mmp.getSpecifiedMixedDistributionPanel(1).getDistribution(0); }
//	
//	if (mmp.getSpecifiedMixedDistributionPanel(2).getDistributions().length ==2) {
//	distributionPacketSize = mmp.getSpecifiedMixedDistributionPanel(2).getMixedDistribution();
//	((MixedDistribution)distributionPacketSize).setWeights(mmp.getSpecifiedMixedDistributionPanel(2).getDistributionsWeight());
//	}
//	else  { distributionPacketSize = mmp.getSpecifiedMixedDistributionPanel(2).getDistribution(0); }
//	
//	
//	duration = cp.getDuration();
//	codec = cp.getModelPanel().getInterneModelPanel().getCodecPanel().getCodec();
//	fileName = sfp.createScenarioFileName(cp);
//	//fileName = "MGInfComplex_60";
//	movieName = cp.getModelPanel().getInterneModelPanel().getModelName();
//	unit = 0;  // fixer pour l'instant en bytes uniquement
//	
//	//System.out.println("name function " + mf.getEquation() + " => " + autoCorr[0].getName()); // DEBUG
//	//System.out.println("weight function " + ((MixedFunction)mf).getWeight()[0] + " " +mf.getName()+ " " + mf.getName());	 // DEBUG
//	//System.out.println("parameter function " + ((MixedFunction)mf).getFunctionsParameter()[0] + " " + ((MixedFunction)mf).getFunctionsParameter()[1]);	 // DEBUG
//	//System.out.println("name distributions " + ((MixedDistribution)distributionTest).getSpecificDistribution(0).getName() + "  " +((MixedDistribution)distributionSlot).getSpecificDistribution(0).getName() + " "  + ((MixedDistribution)distributionPacketSize).getSpecificDistribution(0).getName());	 // DEBUG
//	//System.out.println("parameter distributions " + ((MixedDistribution)distributionTest).getSpecificDistribution(0).getParam(0) + " , " +((MixedDistribution)distributionSlot).getSpecificDistribution(0).getParam(0) + " , "  + ((MixedDistribution)distributionPacketSize).getSpecificDistribution(0).getParam(0));	 // DEBUG
//	//System.out.println("weight distributions " + ((MixedDistribution)distributionTest).getWeight(0) + " , " +((MixedDistribution)distributionSlot).getWeight(0) + " , "  + ((MixedDistribution)distributionPacketSize).getWeight(0));	 // DEBUG
//	//System.out.println("duration " + duration);	 // DEBUG
//	//System.out.println("codec " + codec.getName());	 // DEBUG
//	//System.out.println("movieName " + movieName + " fileName " + fileName);	 // DEBUG
// }
//
// public void setModelParameters(NodeList nl) {
// }
// 
// 
//	/**
//	 * constructModelGenerator : 
//	 * 		- cree le model
//	 * 		- cree le model generator
//	 * @throws IOException
//	 */
// 	public void constructModelGenerator() throws IOException {
// 		
// 		mginfcomplexModel = new MGInfModel();
// 		mginfcomplexGenerator = new MGInfGeneration();
//	
//		// Positionner les differentes variables du model MGInfComplex
//		// Templates du model:
//		// public MGInfModel(MixedFunction mf, double lambda, GenericContinuousDistribution distributionTest, GenericDistribution distributionSlot, GenericDistribution distributionPacketSize) {
// 	
//		mginfcomplexModel.setParamsCount( 5 );
//		mginfcomplexModel.setAutoCorr(mf);
// 		//mginfcomplexModel.setSigmasDistribution(mf);
// 		mginfcomplexModel.setLambda( lambda );
// 		mginfcomplexModel.setDistributionTest( distributionTest );
// 		mginfcomplexModel.setDistributionSlot( distributionSlot );
// 		mginfcomplexModel.setDistributionPacketSize( distributionPacketSize );
// 		
// 		
// 		// Positionner les differentes variables du model generator MGInf
//		// Templates du model generator:
// 		// public MGInfGeneration(VideoDefinition vd, String fichierReq,short unit, boolean graphic)  throws IOException {
//	
//		mginfcomplexGenerator.setMGInfModel(mginfcomplexModel);
//		mginfcomplexGenerator.setDuration(duration);
//		mginfcomplexGenerator.setGraphicMode(true);  // Attention Importance de la pocition de cette ligne
//		try {
//			mginfcomplexGenerator.setResultFileName(fileName);
//		}
//		catch(IOException ioe) {
//			ioe.printStackTrace();	
//		}
//		mginfcomplexGenerator.setCodecAV(codec);
//		mginfcomplexGenerator.setMovieName(movieName);
//		mginfcomplexGenerator.setUnit(unit);
//		
// 	}
// 	
// 	public GenericAggregatedModel getModel() { return mginfcomplexModel;	}
// 	
// 	public GenericGeneration getGenerator() {	return mginfcomplexGenerator; }
// 	
//=======

    /**
     * constructModelGenerator :
     *                 - cree le model
     *                 - cree le model generator
     * @throws IOException
     */
    public void constructModelGenerator() throws IOException {
    	
		MGInfModel mginfcomplexModel = new MGInfModel(autoCorr, lambda, distributionTest, distributionSlot, distributionPacketSize);
		mginfcomplexModel.setName("MGInfComplex");
		MGInfGeneration mginfcomplexGenerator = new MGInfGeneration(mginfcomplexModel, duration, fileName, codec, srcNumber, unit, true);
        mginfcomplexGenerator.setMovieName(movieName);
        mginfcomplexGenerator.setUnit(unit);

        genericAggModel=mginfcomplexModel;
        genericGen = mginfcomplexGenerator;
    }
}
