package graphics2.listener.generator;

import graphics2.*;
import graphics2.panel.client.*;
import graphics2.panel.control.*;

import multimedia.generation.*;
import multimedia.model.*;

import org.w3c.dom.Element;
import java.io.*;

/**
 * ModelGeneratorConstructor
 *
 * @author mladoue
 * @version 0.1, 17/01/03
 */


public abstract class ModelGeneratorConstructor {
		
    protected GestionTest gt;
    protected Element nFlow;
    protected ClientPanel cp;
    protected ScenarioFilePanel sfp;
    protected int duration = 0;
    protected String fileName = "";
    protected int srcNumber = 1;	// initialized

	protected GenericAggregatedModel genericAggModel;
	protected GenericGeneration genericGen;
	
    public ModelGeneratorConstructor(GestionTest gt, Element nFlow) {
        this.gt = gt;
        this.nFlow = nFlow;
        this.cp = gt.getClientPanel();
        this.sfp = gt.getControlPanel().getScenarioFilePanel();
  
		setFlowParameters(nFlow);
        retrieveModel();
        
		//System.out.println("codec " + codec.getName()); //DEBUG
    }

    /**
     * This permits to position srcnumber, duration and fileName parameters
     * @param nl : it has to be a Flow list
     */
    public void setFlowParameters(Element nl) {
    	srcNumber = Integer.parseInt(((Element) nl).getAttribute("srcnumber"));
        duration = Integer.parseInt(((Element) nl).getAttribute("duration"));	//OK
        fileName = ((Element) nl).getAttribute("name"); // OK
    }

    /**
     * Extract the model and codec from the flow
     * Must be specialized depending if the flow is aggregated or not.
     */
    protected abstract void retrieveModel();

    /**
     * This permits to position model parameter
     * @param nl a sub xml tree representing a Model.
     * Note: Must be specialised by each sub classes.
     */
    protected abstract void setModelParameters(Element nl);

	public GenericAggregatedModel getModel() { return genericAggModel;	}
 	
	public GenericGeneration getGenerator() {	return genericGen;	}

    public void setGestionTest(GestionTest gt) { this.gt = gt;}

    public GestionTest getGestionTest() { return gt; }

    public abstract void constructModelGenerator() throws IOException;
}
