package graphics2.listener.generator;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import graphics2.*;
import multimedia.codec.*;


/**
 * @author dsagnol
 * @date Aug 8, 2003
 * 
 * Base class for all the "simple" model, ie the flow with only one model and one codec.
 * 
 */

public abstract class SimpleModelGeneratorConstructor extends ModelGeneratorConstructor {
	// Data types
	Element mCodec;
	Element mModel;

	protected CodecAV codec;

	public SimpleModelGeneratorConstructor(GestionTest gt, Element nFlow) {
		super(gt,nFlow);
	}

	/**
	 * This permits to position codec parameter
	 * @param nl a sub xml tree representing a Codec.
	 * Note: access is retricted to subclasses.
	 */
	protected void setCodecParameters( Element nl ) {
		String codecName =nl.getAttribute("name");
		if ( nl.getAttribute("type").equals("Audio") ) {
			double IPTime = Double.parseDouble(nl.getAttribute("iptime"));
			int psize = Integer.parseInt(nl.getAttribute("psize"));
			codec = new AudioCodec( psize, IPTime, codecName);
			System.out.println("iptime"+IPTime+" "+psize+" "+codecName); //DEBUG
		}
		else {

			int framesSecond = Integer.parseInt(((Element)nl).getAttribute("frames"));
			String comptype = ((Element)nl).getAttribute("compression");
			int M=-1; int N=-1;
			int pos = comptype.lastIndexOf(":");
			N = Integer.parseInt(comptype.substring(pos+1));
			comptype = comptype.substring(0, pos);
			pos = comptype.lastIndexOf(" , ");
			M = Integer.parseInt(comptype.substring(pos+3));
				// The rest is the name
			comptype= comptype.substring(0,pos);

			if (M!=-1 && N!=-1 && comptype.length() > 0) {
				CompressionType ct = new CompressionType(comptype, new GOP(M,N) );
				codec = new VideoCodec(framesSecond, ct, codecName);
			}
		}
	}

	/**
	 * Construct the different elements (model, codec) of the flow.
	 * The function also ensure that the subtree contains a model
	 * and a codec.
	 * 
	 * TODO: the subtree can have several codec or model and this
	 * is not check by the function even if it should not occur...
	 */
	protected void retrieveModel() {
		// Get all the childs of nFlow tree.
		NodeList nl = nFlow.getChildNodes();
		for (int i=0; i < nl.getLength() ; i++) {
			if (((Element)nl.item(i)).getNodeName().equals("model")) {
				setModelParameters(((Element)nl.item(i)));
				mModel = ((Element)nl.item(i));
			}
			if (((Element)nl.item(i)).getNodeName().equals("codec")) {
				setCodecParameters( ((Element)nl.item(i)));
				mCodec = ((Element)nl.item(i));
			}			
		}
		// Check that the two element are not null... otherwise there is a probleme with the given tree.
		if (mCodec==null || mModel == null) {
			System.err.println("Wrong XML subtree: "+nl.toString() );
		}
	}

}
