package graphics2.panel;


import javax.swing.tree.*;

import utils.MyDynamicTree;
import utils.definitions.MyExceptionDefinitions;

/**
 * @author mladoue
 * @version 0.3
 */

public class FlowTreePanel extends MyDynamicTree {


	public FlowTreePanel(String rootTitle) {
		super(rootTitle);
	}
	
	
	/**
	 * To add a node to the tree
	 * @param parent => true if this a flow add, false otherwise
	 * @param definition
	 * @return int  0 => ok, 1=> nok
	 */
	public int addChild( boolean parent, String definition) {
		DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();

		if ( selectedNode == null && parent) {
			DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(definition);
			treeModel.insertNodeInto( newNode, rootNode, rootNode.getChildCount());               
				// now display new node
               
				TreeNode[] nodes = treeModel.getPathToRoot(newNode);
				TreePath path = new TreePath(nodes);
				tree.scrollPathToVisible(path);
				newNode.setAllowsChildren(parent);
				return 0;
		}
		else if ( (parent && selectedNode == rootNode) || (!parent && selectedNode != rootNode && selectedNode!=null)) {
			DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(definition);
			treeModel.insertNodeInto( newNode, selectedNode, selectedNode.getChildCount());               
			// now display new node
               
			TreeNode[] nodes = treeModel.getPathToRoot(newNode);
			TreePath path = new TreePath(nodes);
			tree.scrollPathToVisible(path);
			newNode.setAllowsChildren(parent);
			return 0;
		}
		else {
			toolkit.beep();
			return 1;  // a tester
		}
	}

	public int addSpecificChild(  int childNumber, String definition) {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)rootNode.getChildAt(childNumber);

		if ( node != null  ) {
			DefaultMutableTreeNode newChildNode = new DefaultMutableTreeNode(definition);
			treeModel.insertNodeInto( newChildNode, node, node.getChildCount());               
				// now display new node
               
				TreeNode[] nodes = treeModel.getPathToRoot(newChildNode);
				TreePath path = new TreePath(nodes);
				tree.scrollPathToVisible(path);
				newChildNode.setAllowsChildren(false);
				return 0;
		}
		else {
			toolkit.beep();
			return 1;  // a tester
		}
	}
	
	
	
	public String getCurrentFlowDescription() {
		String ans = "";

		DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) (tree.getLastSelectedPathComponent());
		if( currentNode != null ) {	
			   ans = currentNode.getUserObject().toString();
			 }
			 return ans;
	}
	
	/**
	 * Pour savoir quel est le flot selectionne!!!! entre 0 et N
	 * @return int  0: for the first child(which is a flow) 1: for the second etc
	 */
	public int getCurrentFlowSelection() {
		int ans = 0;
		 TreePath parentPath = tree.getSelectionPath();
	 
		 if ( parentPath != null ) {  //There's no selection. Default to the root node.
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) (parentPath.getLastPathComponent());
	
			if ( (parentNode.getLevel()) == 2 ) {  // si on a selectionne un model ou un codec 
				parentNode = (DefaultMutableTreeNode)parentNode.getParent();
			}
			ans = rootNode.getIndex(parentNode);
			//System.out.println("MyDynamicTree.getCurrentFlowSelection => Flow selected: " + ans);  //DEBUG
			return ans;
		  }
		  toolkit.beep(); // pour signaler qu'il y a une erreur
		  return 0;
	}
	
	/**
	 * Pour savoir combien de flots vont etre utilises dans la simulation
	 * @return int
	 */
	public int getFlowNumber() {  return rootNode.getChildCount(); }
	
	/**
	 * @param i flow number
	 * @return a descrption of node which is a 
	*/
	public String getSpecificFlow(int i) {  return treeModel.getChild(rootNode,i).toString();	 }
	 
	
	/**
		 * Method getChildDefinition
		 * @param flowNumber it's an int between 0 and N
		 * @param TYPE => 0 corresponds to model and 1 corresponds to codec
		 * @param value
		 * @return a description of child in String form, as:
		 * <ul>
		 * <li> "Codec: <i>codecName<i> <i>packetSize</i> <i>IPTime</i>
		 * <li> "Model On-Off: <i>on-off parameters</i>"
		 * <li> "Video: <i>videoName</i>"
		 * </ul>
		 */
		public String getChildDefinition( int flowNumber, int TYPE) {
		
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode)treeModel.getChild(rootNode, flowNumber);
		
			if ( parentNode.getChildCount() == 1) {  // Normalement ce cas n'existe plus depuis 14/04/03
					Object tmp = parentNode.getChildAt(0);
					return tmp.toString();
			}
			else if ( parentNode.getChildCount() == 2 ) {
			
				Object tmp = parentNode.getChildAt(0);
				
					if( TYPE == 0 ) {
						if ( ((tmp.toString().split(" "))[0]).equals("Codec:") ) {
							tmp = parentNode.getChildAt(1);
						}
						return tmp.toString();
					}
					else if( TYPE == 1) {
						if( ((tmp.toString().split(" "))[0]).equals("Model:") ) {
							tmp = parentNode.getChildAt(1);
						}
						return tmp.toString();
					}
					else {
						throw 	MyExceptionDefinitions.DEFAULT_ERROR;
					}			
			}
			else {
				toolkit.beep();	
				return "";
			}
				
		}  
		
	public int removeCurrentNode( DefaultMutableTreeNode child ) {
			DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);
			//TreePath currentSelection = tree.getSelectionPath();
			  if ( child != null ) {
					//DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) (currentSelection.getLastPathComponent());
				DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) (tree.getLastSelectedPathComponent());
					//System.out.println(child.getChildCount() + " " + child.getLevel());
					if( child.getLevel() == 2 ) {  	
					  treeModel.removeNodeFromParent(childNode);
					  return 0;
					}
					else {
						toolkit.beep();
						return 1;
						//MessageManager.informationMessage("You must select a model definition!",true);
					}
				  }
		  
			  // Either there was no selection, or the root was selected.
			  toolkit.beep();
			  return 2;
	}
	
}
