package graphics2.panel;

import graphics2.listener.MyTreeSelectionListener;

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;

import utils.MyDynamicTree;
import utils.definitions.ImageList;

import graphics2.panel.client.*;
/**
 * It's a specific tree defines to be used in ModelsListPanel to represent Xml tree
 * 
 * @author mladoue
 * @version 0.1, Jun 13, 2003
 */

public class FlowXmlTreePanel extends MyDynamicTree {
 
	protected MyTreeSelectionListener tsl;
	protected MyScenarioRenderer msr = new MyScenarioRenderer();
	protected int childCount =0;
	protected int flowDescriptionNumber =0;
	protected ClientPanel cp;
 
 public FlowXmlTreePanel(ClientPanel cp, String rootTitle) {
 	super(rootTitle);
 	this.cp=cp;
 	tree.setEditable(false);
 	tree.setRootVisible(false);
 	// This is the default mode for the default tree selection model. With this mode, any combination of nodes can be selected.
	tree.getSelectionModel().setSelectionMode( TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
 	
 	// Listen for when the selection changes
	tsl = new MyTreeSelectionListener(tree, cp);
	tree.addTreeSelectionListener( tsl );
	
	//	Set the icon for leaf nodes
	tree.setCellRenderer( msr );
	
 }
 
 // On ne pourra pas agreges un scenario et une flow definition
 public void addAggregationFlowDefinitionAsNode(Object[] childs) {
 	
	DefaultMutableTreeNode parentNode = rootNode;
	DefaultMutableTreeNode flow = null;
	String childAggregatedName = "";
	
 	for (int i=0;i< childs.length;i++) {

		//Object nodeInfo = ((DefaultMutableTreeNode)childs).getUserObject();		
		childAggregatedName += (String)childs[i];
		//TreePath parentPath = tree.getSelectionPath();
 	}
 	
	
	flow = new DefaultMutableTreeNode( childAggregatedName);
	treeModel.insertNodeInto( flow, rootNode, rootNode.getChildCount());
 	
	//	now display new node
	TreeNode[] nodes = treeModel.getPathToRoot(flow);
	TreePath path = new TreePath(nodes);
	tree.scrollPathToVisible(path);
 	
 	childCount++;
 }

    public String newSelectededNode() {
        if (isAFirstLevelParent()) { // click on a parent node
            
        } // else, nothing change.
		return "empty";
    }
 
 /**
  * It will insert a node in the tree with this form:
  * 	Flow: <i>flowNB</i>  scenario: <i>name</i>  srcNb: <i>srcNumber</i>  duration: <i>duration<i>)
 * @param elt
 * @return a node flow
 * DefaultMutableTreeNode
 */
public  DefaultMutableTreeNode addFlowAsNode(org.w3c.dom.Element elt) {
 
	DefaultMutableTreeNode parentNode = rootNode; // on force l'ajout du node pour qu'il se fasse "a la racine"
	DefaultMutableTreeNode flow = null;
	
	// ajoute a l'arbre un element de type FlowDescription
//	String flowDesc = new String("Flow: " + flowDescriptionNumber + " "+elt.getAttribute("name")); 
	String flowDesc = new String(elt.getAttribute("name"));
	
	flow = new DefaultMutableTreeNode( flowDesc );	// on definit un nouveau noeud!
	treeModel.insertNodeInto( flow, rootNode, rootNode.getChildCount());    //  on l'ajoute a l'arbre
	 
	//	Set the icon for some leaf nodes
	//tree.setCellRenderer( msr );	
	 
	//	now display new node	<= juste pour que les nodes soient visibles qd on les ajoute
	TreeNode[] nodes = treeModel.getPathToRoot(flow);
	TreePath path = new TreePath(nodes);
	tree.scrollPathToVisible(path);
	flow.setAllowsChildren(true);

	childCount++;  // => ca me permet de savoir si on a ajouter un flow
	flowDescriptionNumber++; // => pour incrementer le numero de flot
	return flow;
 }
 
 public void addModelAsNode( DefaultMutableTreeNode flow, org.w3c.dom.Element elt) {
 
   DefaultMutableTreeNode parentNode = rootNode; // on force l'ajout du node pour qu'il se fasse "a la racine"
   DefaultMutableTreeNode model = null;
   	 
	// ajoute a l'arbre un element associe a flow qui represente le model
   String modelDesc = new String("Model: "+elt.getAttribute("name"));
   //FlowDescription modelD =  new FlowDescription(flowDesc);

   model = new DefaultMutableTreeNode( modelDesc );	// on definit un nouveau noeud!
   treeModel.insertNodeInto( model, flow, flow.getChildCount());    //  on l'ajoute a l'arbre
	 
   //	Set the icon for some leaf nodes
   //tree.setCellRenderer( msr );
	 
   //	now display new node	<= juste pour que les nodes soient visibles qd on les ajoute
   TreeNode[] nodes = treeModel.getPathToRoot(model);
   TreePath path = new TreePath(nodes);
   tree.scrollPathToVisible(path);
   model.setAllowsChildren(false);
}

public void addCodecAsNode( DefaultMutableTreeNode flow, org.w3c.dom.Element elt) {
 
   DefaultMutableTreeNode parentNode = rootNode; // on force l'ajout du node pour qu'il se fasse "a la racine"
   DefaultMutableTreeNode codec = null;
   	 
	 
   // ajoute a l'arbre un element associe a flow qui represente le codec
   String codeclDesc = new String("Codec: "+ elt.getAttribute("name"));

   codec = new DefaultMutableTreeNode( codeclDesc );	// on definit un nouveau noeud!
	treeModel.insertNodeInto( codec, flow, flow.getChildCount());    //  on l'ajoute a l'arbre
	 
   //	Set the icon for some leaf nodes
   //tree.setCellRenderer( msr );
	 
   //	now display new node	<= juste pour que les nodes soient visibles qd on les ajoute
   TreeNode[] nodes = treeModel.getPathToRoot(codec);
   TreePath path = new TreePath(nodes);
   tree.scrollPathToVisible(path);
   codec.setAllowsChildren(false);
   
}
 

 /**
  *  Permit to add a new scenario file to the list from the FileChooser only!!!!
  * @param child
  * void
  */
public void addScenarioAsNode(Object child) {
 	
	DefaultMutableTreeNode parentNode = rootNode;
	DefaultMutableTreeNode flow = null;
	//FlowDescription flowD =  new FlowDescription((String)child);
	String flowD = new String(child.toString());
	
	//TreePath parentPath = tree.getSelectionPath();
	flow = new DefaultMutableTreeNode( flowD);

	treeModel.insertNodeInto( flow, rootNode, rootNode.getChildCount());     
	 
	//	Set the icon for some leaf nodes
	//tree.setCellRenderer( msr );
	 
	//	now display new node
	TreeNode[] nodes = treeModel.getPathToRoot(flow);
	TreePath path = new TreePath(nodes);
	tree.scrollPathToVisible(path);
	flow.setAllowsChildren(false);

	//tree.scrollPathToVisible(new TreePath(flow.getPath()));
	childCount++;
 }
 
 public int removeCurrentNode() {
	 childCount--;
	 return super.removeCurrentNode();
  }
 
 public String[] getSelectedFlowDefinition() {
		TreePath[] paths = tree.getSelectionPaths();
		DefaultMutableTreeNode[] nodesList = new DefaultMutableTreeNode[paths.length];
		DefaultMutableTreeNode currentNode;
		String[] ans = new String[paths.length];
		String flowD;
	
		for (int i=0; i< nodesList.length; i++) {
			currentNode = (DefaultMutableTreeNode) (paths[i].getLastPathComponent());		

			 Object nodeInfo = currentNode.getUserObject();
			 if( currentNode.isLeaf() ) {
				flowD = (String)nodeInfo;
				ans[i] = flowD;
		 	}
		}
 	
 	return ans;
 }
 
 
 
 public void setTreeImage(Object[] childs) {
 	for(int i=0;i<childs.length;i++) {
 		addObject(childs[i]);
 	}
 }
 
 public Object[] getCurrentTreeImage() {
 	int flowNumber = treeModel.getChildCount(rootNode);
 	Object[] nodes = new Object[flowNumber];
 	
 	if( flowNumber !=0 ) {
 	
	 	for(int i=0;i<flowNumber;i++) {
	 		nodes[i] = treeModel.getChild(rootNode,i);
	 	}
 	
 	}
 	
 	return nodes;
 }
 
 /**
  * Test if the added node is a scenario file (=> .scf) or not 
 * @param value
 * @return boolean
 */
protected boolean isScenarioFile(Object value) {
 	
		DefaultMutableTreeNode node =  (DefaultMutableTreeNode)value;
		String nodeInfo;
		String title;
	
		 if( node != null && node != rootNode && node.getLevel() == 1 ) { 

			//nodeInfo  = (FlowDescription)( node.getUserObject() );
			nodeInfo  = node.getUserObject().toString();
			//title = nodeInfo.flowDescription;
		
			 if ( nodeInfo.endsWith(".scf") ) {  return true; }
		 }
		 return false;
}


public MyTreeSelectionListener getMyTreeSelectionListener() { return tsl; }

public JTree getJTree() { return tree; }

public int getFlowNumber() { return childCount; }

/**
 *  Private class
 */

private class MyScenarioRenderer extends DefaultTreeCellRenderer {

	ImageIcon scenarioIcon;
	
	public MyScenarioRenderer() {
		scenarioIcon = ImageList.scenarioIcon;
	}

	public Component getTreeCellRendererComponent( JTree tree, Object value, boolean sel,	boolean expanded,	boolean leaf, int row, boolean hasFocus) {
		
			super.getTreeCellRendererComponent(	tree, value, sel, expanded, leaf, row, hasFocus);
			
			if ( FlowXmlTreePanel.this.isScenarioFile(value) ) {
				setIcon( scenarioIcon );
				setToolTipText("This is a pre-generated scenario file");
			}
			else { setToolTipText("I don't know what it is"); }

			return this;
	}
	
}

}
