/**
 * ClientPanel : Lance l'interface graphique de l'onglet concernant: client features
 *
 * @author maria ladoue
 * @version 0.3, 03/04/03 
 */

package graphics2.panel.client;

import javax.swing.*;
import java.awt.*;
import java.util.*;

import graphics2.*;
import graphics2.info.*;
import graphics2.panel.client.model.*;
import multimedia.model.*;
import utils.*;
import utils.definitions.MyExceptionDefinitions;


public class ClientPanel extends JPanel implements ClientInfo {

    // JFrame auquel sera attache le JPanel
    protected GestionTest gestionTest;
    protected ModelPanel modelPanel;
    protected TimePanel timePanel;
    public ModelsListPanel modelsListPanel;
    
  	protected Hashtable listModel;
    protected Hashtable listMGInfModel;
        
        // All the information on the different scenarii created by the 
        // users are stored in this object.
    protected ConfigurationXMLFile listScenario;
   
/**
 * A ClientPanel is associated to GestionTest
 * @param unGestionTest
 */
    public ClientPanel( GestionTest gestionTest ) {

            // Create the empty XML tree for storing model information.
        try {
            listScenario = new ConfigurationXMLFile("");
        }
        catch (java.io.IOException ioe) { System.out.println(ioe); }
  
        
		this.gestionTest = gestionTest;
		
		makeLists();
		timePanel = new TimePanel(this);
		modelPanel = new ModelPanel( this, listModel, listMGInfModel);
		modelsListPanel = new ModelsListPanel(this);
		
		
		this.setLayout( new BoxLayout(this, BoxLayout.Y_AXIS));
	
		JPanel cPanel =  new JPanel();
		cPanel.setLayout( new BoxLayout(cPanel,BoxLayout.Y_AXIS) );
		
		
			
			// Time definition panel
			cPanel.add(  timePanel );
			cPanel.add( Box.createRigidArea(new Dimension(0,5)) );
			// Description du model et du codec
			cPanel.add( modelPanel  );
			cPanel.add( Box.createRigidArea(new Dimension(0,5)) );
			// Liste des flows et scenarios choisis
			cPanel.add( modelsListPanel );
	
			JScrollPane scroller = new JScrollPane(cPanel);
			this.add( scroller );
    }
    
   
   public void makeLists() {
		// Liste des modeles possibles
		listModel = new Hashtable();
		try {
			listModel.put("multimedia.modelName.OnOffModel" , "On-Off");
			listModel.put("multimedia.modelName.OnOffHeterogeneousAggregatedFlowModel" , "On-Off Heterogeneous Aggregated Flow");
			listModel.put("multimedia.modelName.MMPP2Model","MMPP-2");
			listModel.put("multimedia.modelName.MMPPNModel","MMPP-N");
			listModel.put("multimedia.modelName.MGInfModel" , "M/G/Inf");
			listModel.put("multimedia.modelName.MGInfComplexModel" , "M/G/Inf Complex");
			listModel.put("multimedia.modelName.MGInfAggregatedFlowModel" , "M/G/Inf Complex Aggregated Flows");
		}
		catch (NullPointerException e) { 
			System.out.println("ClientPanel.constructor :" + e.toString());
		}
		
		 // Liste des M/G/Inf modeles pre-definis possibles
		listMGInfModel = new Hashtable();
		try {
			listMGInfModel.put("None","None");
			listMGInfModel.put("multimedia.VideoExample.JurassicParkLow","Jurassic Park (Low Quality)");
			listMGInfModel.put("multimedia.VideoExample.JurassicParkMedium","Jurassic Park (Medium Quality)");
			listMGInfModel.put("multimedia.VideoExample.JurassicParkHigh ","Jurassic Park (High Quality)");
			listMGInfModel.put("multimedia.VideoExample.BeanLow","Bean (Low Quality)");
			listMGInfModel.put("multimedia.VideoExample.BeanMedium","Bean (Medium Quality)");
			listMGInfModel.put("multimedia.VideoExample.BeanHigh ","Bean (High Quality)");
		}
		catch (NullPointerException e) { 
			System.out.println("ClientPanel.constructor :" + e.toString());
		}
   }

    public ConfigurationXMLFile getListScenario() { return listScenario;}

	public void loadScenarioFile(String filename) {
			// Create the XML tree for storing model information.
		if (listScenario != null)
		{
			listScenario=null;
		}
		try {
			listScenario = new ConfigurationXMLFile(filename);
		}	
		catch (java.io.IOException ioe) { System.out.println(ioe);}
	
		// Update the Tree containing all the scenario...
		org.w3c.dom.Document doc = listScenario.getDocument();
			// First, update the global parameters
		org.w3c.dom.Element elt = doc.getDocumentElement ();
		gestionTest.getControlPanel().getFileTestParametersPanel().setErrorMax(elt.getAttribute("errormax"));
		gestionTest.getControlPanel().getFileTestParametersPanel().setIPtimeMax(elt.getAttribute("IPtimemax"));

			// second, iterate the XML Tree and add the elt.
		org.w3c.dom.NodeList nl = doc.getElementsByTagName( "Flow");
		for(int i=0; i <nl.getLength(); i++) {
			this.addFlowToListPanel( (org.w3c.dom.Element)nl.item(i));
		}
	}

	public void saveScenarioFile(String filename) {
		
			// Update (or create) attributes errormax and IPtimemax infos
		org.w3c.dom.Document doc = listScenario.getDocument();
		org.w3c.dom.Element elt = doc.getDocumentElement ();
		String errormax = gestionTest.getControlPanel().getFileTestParametersPanel().getErrorMax();
		String iptimemax = gestionTest.getControlPanel().getFileTestParametersPanel().getIPtimeMax();
		if (errormax.length() > 0)
			elt.setAttribute("errormax",errormax);
		if (iptimemax.length()>0)
			elt.setAttribute("IPtimemax",iptimemax);
			// Save the XML Tree in a file.
		listScenario.writeFile( filename);
	}
	
    public ModelPanel getModelPanel() { return modelPanel; }
    public GestionTest getGestionTest() { return gestionTest;	}
	

	//  pour positionner les donnees
    public void setModelTest(String modelName) { modelPanel.setModelTest(modelName); }
    // Pour pouvoir recuperer le nom du modele pour le fichier de configuration
    public void setModelTestName(String modelName) { modelPanel.setModelTestName(modelName); }


    // pour acceder aux donnees
    //public GenericModel getModelTest() { return modelPanel.getModelTest(); }  // avant le 02/04/03
    public GenericAggregatedModel getModelTest() { return modelPanel.getModelTest(); }
       
    
    public void  setCodecName(String codec) throws IllegalArgumentException { modelPanel.getInterneModelPanel().getCodecPanel().setCodecName(codec); }
    public String getCodecName() { return modelPanel.getInterneModelPanel().getCodecPanel().getCodecName(); }

	// ClientInfo compatibility
	public void setDuration(short duration) throws NumberFormatException { timePanel.setDuration(duration); }
 
    /**
     * Returns test duration specified in the GUI in minutes
	 * @see graphics.ClientInfo#getDuration()
	 */
	public short getDuration() throws NumberFormatException { return timePanel.getDuration(); }
   
	/**
	* @see graphics.ClientInfo#getModelTestName()
	*/
	public String getModelTestName() { return new String( SpecificObjectCreation.createModelName( modelPanel.getModelName() )); }

	/**
	 * @return a table of String with each parameter 
	 * @see graphics.ClientInfo#getModelParam()
	 * 
	 * -MMPP2: <i>start-state</i> <i>alpha</i> <i>beta</i> <i>lambda1</i> <i>lambda2</i>
	 * -MMPPN: <i>start-state</i> <i>states-number</i> <i>alpha</i> <i>beta</i> <i>lambda</i>
	 * -MGInf: <i>function name</i> <i>b</i> <i>lambda</i> <i>distribName1: amount of data</i> <i>distribName2: data distrib</i>
	 * 		 <i>distribName3: packet size distrib</i> <i>parameter(s) distrib1</i> <i>parameter(s) distrib2</i> <i>parameter(s) distrib3</i>
	 * -MGInfComplex: <i>mixed function?</i> <i>functions weight</i><i>function(s) name</i> <i>parameter(s) b</i> <i>lambda</i>
	 *						   <i>mixed distribution1?,2?,3?</i> <i>distrib 1,2,3 weight</i> <i>distribName(s)1: amount of data,2: data distrib,3: packet size distrib</i>
	 * 					   <i>parameter(s) distrib(s)1,2,3)</i>
	 * -OnOff: <i>start-state</i><i>distribName1: talkspurt length</i> <i>distribName2: idle period</i> <i>parameter(s) distrib1</i> <i>parameter(s) distrib2</i>
	 * -OnOffOnOffHeterogeneousAggregatedFlow: <i>flow type number</i> and for each flow:  <i>sources number</i> <i>OnOff model parameters</i> <i>|</i>
	 * -MGInfComplexAggregatedFlow: <i>flow type number</i> and for each flow:  <i>a video definition</i>
	 */
	public String[] getModelParam() {
		
		GenericModelPanel mgp = modelPanel.getInterneModelPanel().getGenericModelPanel();
		int parameterNb = mgp.getParametersNumber();
		//System.out.println("Parameter number: " + parameterNb);
		String[] ans = new String[parameterNb];
		String modelName = SpecificObjectCreation.createModelName(modelPanel.getModelName());
		
		if( modelName.equals("MMPP2") ) {
			ans[0] = new String("" + ((MMPP2ModelPanel)mgp).getStartState());
			for (int i=0;i<4;i++) {
			ans[i+1] =  new String("" +  ((MMPP2ModelPanel)mgp).getParam(i));
			}
		}
		
		else if( modelName.equals("MMPPN") ) {
			ans[0] = new String("" + ((MMPPNModelPanel)mgp).getStartState());
			ans[1] = new String("" + ((MMPPNModelPanel)mgp).getNbEtat());
			for ( int i=0;i<3;i++) {
				ans[i+2] =  new String("" + ((MMPPNModelPanel)mgp).getParam(i));
			}	
		}
		
		else if( modelName.equals("MGInf") ) {
			int distParamNb = 0, distParamNbOld = 0;
			ans[0] = ((MGInfModelPanel)mgp).getFunctionName();
			ans[1] = new String("" + ((MGInfModelPanel)mgp).getFunctionParameter() );
			ans[2] = new String("" + ((MGInfModelPanel)mgp).getLambda() );
			
			for ( int i=3;i<6;i++) {
				ans[i] = ((MGInfModelPanel)mgp).getDistributionName(i-3);
				distParamNb = ((MGInfModelPanel)mgp).getDistributionPanel(i-3).getDistribution().getParamsCount();
				for ( int j=0;j < distParamNb;j++ ) {
					try{
					ans[distParamNbOld+6+j] = (((MGInfModelPanel)mgp).getDistributionParameters(i-3))[j];
					}
					catch(ArrayIndexOutOfBoundsException aie) { System.out.println( aie); }
					
				}
				distParamNbOld += distParamNb;
				}
		}
		
		else if( modelName.equals("MGInfComplex")  ) {
			int k = 2;
 			ans[0] = Boolean.toString( ((MGInfComplexModelPanel)mgp).mixedFunctionPanel.isMixedFunc );
 			ans[1] = ((MGInfComplexModelPanel)mgp).getMixedFunctionPanel().getWeight(0);
 			
 			if ( Boolean.valueOf( ans[0] ).booleanValue() ) {
 			ans[k] = ((MGInfComplexModelPanel)mgp).getMixedFunctionPanel().getWeight(1);
 			ans[++k] = ((MGInfComplexModelPanel)mgp).getMixedFunctionName(0);			
 			ans[++k] = ((MGInfComplexModelPanel)mgp).getMixedFunctionName(1);
 			ans[++k] = ((MGInfComplexModelPanel)mgp).getMixedFunctionParameter(0);
 			ans[++k] = ((MGInfComplexModelPanel)mgp).getMixedFunctionParameter(1);
 			}
 			else {
 			ans[k] = ((MGInfComplexModelPanel)mgp).getMixedFunctionName(0);
 			ans[++k] = ((MGInfComplexModelPanel)mgp).getMixedFunctionParameter(0);
 			}
 			
 			ans[++k] = new String("" + ((MGInfComplexModelPanel)mgp).getLambda() );
 			int l = k+1;
 		
 			//Distrib are mixed or not!!
 			for (int i=0;i<3;i++) {
 			ans[++k] = Boolean.toString(  ((MGInfComplexModelPanel)mgp).getDistributionType(i) );
 		
 			}
 			//Distrib weight
 			ans[++k] = Double.toString( ((MGInfComplexModelPanel)mgp).getDistributionWeight(0,0) );
 			 if( Boolean.valueOf(ans[l]).booleanValue() ) {	
 			 		ans[++k] = Double.toString( ((MGInfComplexModelPanel)mgp).getDistributionWeight(0,1) );
 			 }
 			 
 			ans[++k] = Double.toString( ((MGInfComplexModelPanel)mgp).getDistributionWeight(1,0));
 			 if( Boolean.valueOf(ans[l+1]).booleanValue() ) {	
 			 		ans[++k] = Double.toString( ((MGInfComplexModelPanel)mgp).getDistributionWeight(1,1) );
 			 }
 
 			ans[++k] = Double.toString( ((MGInfComplexModelPanel)mgp).getDistributionWeight(2,0) );
 			  if(  Boolean.valueOf(ans[l+2]).booleanValue() ) {	
 			 		for (int j=0;j<2;j++) {
 			 		ans[++k] = Double.toString( ((MGInfComplexModelPanel)mgp).getDistributionWeight(2,1) );
 			 		}
 			 }
 			
 			//Distrib name
 			ans[++k] = ((MGInfComplexModelPanel)mgp).getDistributionName(0,0);
 			if( Boolean.valueOf(ans[l]).booleanValue() ) { ans[++k] = ((MGInfComplexModelPanel)mgp).getDistributionName(0,1);	 }	 
 			
 			ans[++k] = ((MGInfComplexModelPanel)mgp).getDistributionName(1,0); 
 			if( Boolean.valueOf(ans[l+1]).booleanValue() ) { ans[++k] = ((MGInfComplexModelPanel)mgp).getDistributionName(1,1);	}
 			
 			ans[++k] = ((MGInfComplexModelPanel)mgp).getDistributionName(2,0); 
 			if( Boolean.valueOf(ans[l+2]).booleanValue() ) { ans[++k] = ((MGInfComplexModelPanel)mgp).getDistributionName(2,1);	}
 		
 			   
 			//Distrib parameter 	
 			String[] tmp = ((MGInfComplexModelPanel)mgp).getDistributionParameters(0);
 			 		
 		 		for (int n=0;n<tmp.length;n++) {
 		 		ans[++k] = tmp[n];
 		 		}
 			 //System.out.println("ans[l]..." +  ans[l]+ " "+ ans[l+1] + " " + ans[l+2] );
 			
 			 tmp = ((MGInfComplexModelPanel)mgp).getDistributionParameters(1);
 			for (int n=0;n<tmp.length;n++) {
 					ans[++k] = tmp[n];
 				} 	
 			 tmp = ((MGInfComplexModelPanel)mgp).getDistributionParameters(2);
 			 	for (int n=0;n<tmp.length;n++) {
 			 	ans[++k] = tmp[n];
 			 	}
		}
		
		else if( modelName.equals("OnOff") ) {			
			int distParamNb =0, distParamNbOld=0;
			
			ans[0] = new String ("" + ((OnOffModelPanel)mgp).getStartState());
			
			for ( int i=0;i<2;i++) {
				ans[i+1] = ((OnOffModelPanel)mgp).getDistributionName(i);  //nom des distribs				
				distParamNb = ((OnOffModelPanel)mgp).getDistributionPanel(i).getDistribution().getParamsCount();
				for ( int j=0;j<distParamNb;j++ ) {
					try{
						ans[distParamNbOld+3+j] = (((OnOffModelPanel)mgp).getDistributionParameters(i))[j];
					}
					catch(ArrayIndexOutOfBoundsException aie) { System.out.println(aie);	}
				}
				distParamNbOld += distParamNb;
			}
		}
		
		else if( modelName.equals("OnOffHeterogeneousAggregatedFlow") ) {	
					
				int currentIndex = 0;
				int flowsNumber = ((OnOffHeterogeneousAggregatedFlowModelPanel)mgp).getFlowTypeNumber();
				ans[currentIndex] = new String ("" + flowsNumber);
				currentIndex++;
				
				String flowDescription = "", modelDescription = "";
				int modelSize = 0;
				
				for ( int i=0;i<flowsNumber;i++ ) {
					flowDescription = ((OnOffHeterogeneousAggregatedFlowModelPanel)mgp).getSpecificFlowDefinition(i+1);
					modelDescription = ((OnOffHeterogeneousAggregatedFlowModelPanel)mgp).getSpecificOnOffModelDefinition(i+1);
					String[] flow = flowDescription.split(" ");  // "Flow " + newNodeSuffix + " : " + nb.getText() + " source(s)."
					String[] model = modelDescription.split(" "); // "Model On-Off: " + modelParams
					modelSize = model.length;
					//System.out.println(modelSize);
					//System.out.println(SpecificObjectCreation.transformSTS(flow));
					//System.out.println(SpecificObjectCreation.transformSTS(model));
					
					ans[currentIndex] = flow[3];  // correspond to the number of sources of this type!!
					currentIndex++;
					for ( int k=2;k<modelSize;k++) {
						ans[currentIndex] = model[k];
						currentIndex++;
					}
					if (i < flowsNumber-1) {
						ans[currentIndex] = new String("|");  // ajout un delimiteur entre les definitions de modeles
						currentIndex++;
					}
				}
			}
		else if( modelName.equals("MGInfComplexAggregatedFlow") ) {
			int currentIndex = 0;
			int flowsNumber = ((MGInfComplexAggregatedFlowModelPanel)mgp).getFlowTypeNumber();
			ans[currentIndex] = new String ("" + flowsNumber);
			currentIndex++;
			String flowDescription = "", videoDescription = "";
			
			for ( int i=0;i<flowsNumber;i++ ) {
				flowDescription = ((MGInfComplexAggregatedFlowModelPanel)mgp).getSpecificFlowDefinition(i+1);
				videoDescription = ((MGInfComplexAggregatedFlowModelPanel)mgp).getSpecificVideoDefinition(i+1);
				String[] flow = flowDescription.split(" ");  // "Flow " + newNodeSuffix + " : " + nb.getText() + " source(s)."
				String[] video = videoDescription.split(": "); // "Video: " + videoName
				ans[currentIndex] = flow[3];  // correspond to the number of sources of this type!!
				currentIndex++;
				ans[currentIndex] = video[1];
				currentIndex++;
			}
		}
		
	
		else {
			throw new IllegalArgumentException("The model : " + modelName +" doesn't exist!");
		}
		return ans;
	}


	/**
	 * Method setModelParam.
	 * @param params
	 *  -MMPP2: <i>start-state</i> <i>alpha</i> <i>beta</i> <i>lambda1</i> <i>lambda2</i>
	 * -MMPPN: <i>start-state</i> <i>states-number</i> <i>alpha</i> <i>beta</i> <i>lambda</i>
	 * -MGInf: <i>function name</i> <i>b</i> <i>lambda</i> <i>distribName1: amount of data</i> <i>distribName2: data distrib</i>
	 * 		 <i>distribName3: packet size distrib</i> <i>parameter(s) distrib1</i> <i>parameter(s) distrib2</i> <i>parameter(s) distrib3</i>
	 * -MGInfComplex: <i>mixed function?</i> <i>functions weight</i><i>function(s) name</i> <i>parameter(s) b</i> <i>lambda</i>
	 *						   <i>mixed distribution1?,2?,3?</i> <i>distrib 1,2,3 weight</i> <i>distribName(s)1: amount of data,2: data distrib,3: packet size distrib</i>
	 * 					   <i>parameter(s) distrib(s)1,2,3)</i>
	 * -OnOff: <i>start-state</i><i>distribName1: talkspurt length</i> <i>distribName2: idle period</i> <i>parameter(s) distrib1</i> <i>parameter(s) distrib2</i>
	 * -OnOffOnOffHeterogeneousAggregate: <i>flows number</i> and for each flow:  <i>sources number</i> <i>OnOff model parameters</i>
	 * -MGInfComplexAggregatedFlow:  <i>flow type number</i> and for each flow:  <i>a video definition</i> 
	 */
	public void setModelParam(String[] params) throws IllegalArgumentException {
		
		GenericModelPanel mgp = modelPanel.getInterneModelPanel().getGenericModelPanel();
		int parameterNb = mgp.getParametersNumber();
		String modelName = SpecificObjectCreation.createModelName(modelPanel.getModelName());
		
		if( modelName.equals("MMPP2") ) {
			((MMPP2ModelPanel)mgp).setStartState( Integer.parseInt(params[0]) );
			for (int i=1;i<parameterNb;i++) {
				((MMPP2ModelPanel)mgp).setParam(i-1,Double.parseDouble(params[i]));
			}
		}
		
		else if ( modelName.equals("MMPPN") ) {
			((MMPPNModelPanel)mgp).setStartState( Integer.parseInt(params[0]));
			((MMPPNModelPanel)mgp).setNbEtat(Integer.parseInt(params[1]));
			for ( int i=2;i<parameterNb;i++) {
				((MMPPNModelPanel)mgp).setParam(i-2,Double.parseDouble(params[i]));
			}	
		}
	
		else if ( modelName.equals("MGInf") ) {
			int distParamNbOld = 0;
		
			((MGInfModelPanel)mgp).setFunctionName(params[0]);
			((MGInfModelPanel)mgp).setFunctionParameter( Double.parseDouble(params[1]) );
			((MGInfModelPanel)mgp).setLambda(Double.parseDouble(params[2]));
			// set des noms des distribs
			for ( int i=3;i<6;i++) {
				((MGInfModelPanel)mgp).setDistributionName(i-3,params[i]);
			}
			// set des parametres des distribs
			for ( int i=0;i<3;i++) {
				int distParamNb = ((MGInfModelPanel)mgp).getDistributionPanel(i).getDistribution().getParamsCount();
				String[] ans = new String[distParamNb];
				
				for (int k=0;k<distParamNb;k++) {
						ans[k] = params[ 6 + distParamNbOld + k ];	
				}			
				
				distParamNbOld += distParamNb;
				((MGInfModelPanel)mgp).setDistributionParameters(i,ans);
			}
		}
		
		else if ( modelName.equals("MGInfComplex") ) {
			int k = 2;
			((MGInfComplexModelPanel)mgp).setIsMixedFunction(Boolean.valueOf(params[0]).booleanValue());
			((MGInfComplexModelPanel)mgp).setMixedFunctionWeight(0, Double.parseDouble(params[1]) );
			if (Boolean.valueOf(params[0]).booleanValue()) { ((MGInfComplexModelPanel)mgp).setMixedFunctionWeight(1,Double.parseDouble(params[k])); }
			((MGInfComplexModelPanel)mgp).setMixedFunctionName(0,params[++k]);
			if (Boolean.valueOf(params[0]).booleanValue()) { ((MGInfComplexModelPanel)mgp).setMixedFunctionName(1,params[++k]); }
			((MGInfComplexModelPanel)mgp).setMixedFunctionParameter(0, Double.parseDouble(params[++k]));
			if (Boolean.valueOf(params[0]).booleanValue()) { ((MGInfComplexModelPanel)mgp).setMixedFunctionParameter(1, Double.parseDouble(params[++k])); }
			((MGInfComplexModelPanel)mgp).setLambda(Double.parseDouble(params[++k]));
			int l = k+1;
			for (int i=0;i<3;i++) {
				if (params[++k].equals("true"))
					((MGInfComplexModelPanel)mgp).setDistributionType(i, true);
				else
					((MGInfComplexModelPanel)mgp).setDistributionType(i, false);
			}
			
			for (int i=0;i<3;i++) {
				if (Boolean.valueOf(params[l+i]).booleanValue()) {	
					 double[] tmp = { Double.parseDouble(params[++k]), Double.parseDouble(params[++k]) };
					((MGInfComplexModelPanel)mgp).setDistributionWeights( i, tmp);
				}
				else {
					 double[] tmp = { Double.parseDouble(params[++k]) };
					 ((MGInfComplexModelPanel)mgp).setDistributionWeights(i, tmp); 
					 }
			}
			
			for (int i=0;i<3;i++) {
				if (Boolean.valueOf(params[l+i]).booleanValue()) {	
					 String[] tmp = { params[++k], params[++k] };
					((MGInfComplexModelPanel)mgp).setDistributionNames( i, tmp);
				}
				else {
					 String[] tmp = { params[++k] };
					 ((MGInfComplexModelPanel)mgp).setDistributionNames(i, tmp); 
					 }
			}
			
			for (int i=0;i<3;i++) {
				int panelNb = 1;
				if(Boolean.valueOf(params[l+i]).booleanValue() ) { panelNb = 2; }
				
					for (int j=0;j<panelNb;j++) {
					int distParamNb = ((MGInfComplexModelPanel)mgp).getSpecifiedMixedDistributionPanel(i).getDistribution(j).getParamsCount();
					if(distParamNb==2) {
							String[] tmp = { params[++k], params[++k] };
							((MGInfComplexModelPanel)mgp).setDistributionParameters( i,j, tmp);
							}
					else { 
					String[] tmp = { params[++k] }; 
					((MGInfComplexModelPanel)mgp).setDistributionParameters( i,j, tmp);
					}
				
			}
			}	
		}
		
		else if( modelName.equals("OnOff") ) {
			((OnOffModelPanel)mgp).setStartState( Integer.parseInt(params[0]));
			
			for ( int i=0;i<2;i++) {
				((OnOffModelPanel)mgp).setDistributionName(i,params[i+1]);
			}
			for ( int i=0;i<2;i++) {
				int distParamNb = ((OnOffModelPanel)mgp).getDistributionPanel(i).getDistribution().getParamsCount();
				int distParamNbOld=0;
				String[] ans = new String[distParamNb];
					for (int k=0;k<distParamNb;k++) {
						ans[k] = params[3+distParamNbOld+k];	
					}
					distParamNbOld += distParamNb;
					((OnOffModelPanel)mgp).setDistributionParameters(i,ans);
			}
		}
		
		else if( modelName.equals("OnOffHeterogeneousAggregatedFlow") ) {
			int currentIndex = 0;
			((OnOffHeterogeneousAggregatedFlowModelPanel)mgp).setFlowNumber(Integer.parseInt(params[currentIndex]));
			currentIndex++; //DEBUG 20/05/03 A verifier
			String modelDescription = "";
			
			for (int j=0; j<Integer.parseInt(params[currentIndex]);j++) {
				((OnOffHeterogeneousAggregatedFlowModelPanel)mgp).setSpecificFlowDefinition(j+1,j+1, Integer.parseInt(params[currentIndex]));  // set flow j definition
				currentIndex++;
				while ( ! params[currentIndex].equals(new String("|")) ) {
						modelDescription += params[currentIndex] + " ";
						//parameters.removeElementAt(0);
						currentIndex++;
				}
				((OnOffHeterogeneousAggregatedFlowModelPanel)mgp).setSpecificOnOffModelDefinition( j+1,  modelDescription);  //set model j definition
				currentIndex++;  // on comptablise caractere "|"
			}	
		}
		else if( modelName.equals("MGInfComplexAggregatedFlow") ) {
			int currentIndex = 0;
			((MGInfComplexAggregatedFlowModelPanel)mgp).setFlowNumber(Integer.parseInt(params[currentIndex]));
			currentIndex++; 
			
			for (int j=0; j<Integer.parseInt(params[0]);j++) {
				((MGInfComplexAggregatedFlowModelPanel)mgp).setSpecificFlowDefinition(j+1,  j+1,Integer.parseInt(params[currentIndex]));  // set flow j definition
				currentIndex++;
				((MGInfComplexAggregatedFlowModelPanel)mgp).setSpecificVideoDefinition(j+1, params[currentIndex]);
				currentIndex++;
			}
		}
		
		else {
			new IllegalArgumentException("ClientPanel.setModelParam : The model : " + modelName +" don't exist!");		
		}
	}  // end of setModelParam()
	
	
	/**
	 * Method setCodecParam
	 * @see graphics.ClientInfo#setCodecParam(String[])
	 */
	public void setCodecParam(String[] params) throws IllegalArgumentException {
		if ( params.length == 2 ) { 
			modelPanel.getInterneModelPanel().getCodecPanel().setCodecParams(params);
		}
		else if( params.length > 2 ) { // cas des flux aggreges
			String modelName = SpecificObjectCreation.createModelName(modelPanel.getModelName());
			GenericModelPanel mgp = modelPanel.getInterneModelPanel().getGenericModelPanel();
			
			if(modelName.equals("OnOffHeterogeneousAggregatedFlow")) {
				
				int flowsNumber = ((OnOffHeterogeneousAggregatedFlowModelPanel)mgp).getFlowTypeNumber();
				int currentIndex = 0;
				String codecDescription = "";
			
			for (int i=0;i<flowsNumber;i++ ) {
				for (int j=0; j<3;j++) {
					codecDescription += params[currentIndex] + " ";
					currentIndex++;
					}
				((OnOffHeterogeneousAggregatedFlowModelPanel)mgp).setSpecificCodecDefinition( i+1, codecDescription);  //set model j definition
				}	
			}
			else {  new IllegalArgumentException("You don't have the good argument number : " + params.length); }
		}
		else { throw MyExceptionDefinitions.PARAMETERS_TABLE_SIZE; }
	}
	
/**
 * Returns codec parameters
 * @see graphics.ClientInfo#getCodecParam()
 */
public String[] getCodecParam() {	
	
	String modelName = SpecificObjectCreation.createModelName(modelPanel.getModelName());
	GenericModelPanel mgp = modelPanel.getInterneModelPanel().getGenericModelPanel();
			
	if( modelName.equals("OnOffHeterogeneousAggregatedFlow") ) {
		//int currentIndex = 0;
			
		String codecDescription = "";
		int flowsNumber = ((OnOffHeterogeneousAggregatedFlowModelPanel)mgp).getFlowTypeNumber();
		String[] ans = new String[(flowsNumber*3)];
		String codecs = "";
				
		for (int i=0; i<flowsNumber;i++) {
			codecDescription = ((OnOffHeterogeneousAggregatedFlowModelPanel)mgp).getSpecificCodecDefinition(i+1);
			String[] ansTmp = codecDescription.split(" ",2);
			//codecs += codecDescription + " " ;  // "Codec: " + codecName + codecParam"
			codecs += ansTmp[1] + " ";
		}	
		ans = codecs.split(" ");
		return ans;
	}
	else if( modelName.equals("MGInfComplexAggregatedFlow") ) {
		
		String codecDescription = "";
		int flowsNumber = ((OnOffHeterogeneousAggregatedFlowModelPanel)mgp).getFlowTypeNumber();
		String[] ans = new String[(flowsNumber*3)];
		String codecs = "";
				
		for (int i=0; i<flowsNumber;i++) {
			codecDescription = ((OnOffHeterogeneousAggregatedFlowModelPanel)mgp).getSpecificCodecDefinition(i+1);
			String[] ansTmp = codecDescription.split(" ",2);
			//codecs += codecDescription + " " ;  // "Codec: " + codecName + codecParam"
			codecs += ansTmp[1] + " ";
		}	
		ans = codecs.split(" ");
		return ans;
	}
	else {
		return modelPanel.getInterneModelPanel().getCodecPanel().getCodecParam();
	}

	}

public int getParameterNb() { return modelPanel.getInterneModelPanel().getGenericModelPanel().getParametersNumber(); }

public Hashtable getListVideo() { return listMGInfModel; }

public void displayScenario(String scname) {
	// find the node in the XML tree wirh scname name
	org.w3c.dom.Element elt = listScenario.findScenario(scname);
	listScenario.printScenario(elt);
		// Inform the ModelPanel to update model/codec frames.
	modelPanel.setModelTestName(((org.w3c.dom.Element)elt.getChildNodes().item(0)).getAttribute("name"));
	
	modelPanel.getInterneModelPanel().getGenericModelPanel().updateWithXmlNode((org.w3c.dom.Element)elt.getChildNodes().item(0));
		// Now, the codec... but only for none aggregated flows !
	if (((org.w3c.dom.Element)elt.getChildNodes().item(0)).getAttribute("aggregated").length() >0 &&
	((org.w3c.dom.Element)elt.getChildNodes().item(0)).getAttribute("aggregated").equalsIgnoreCase( "true")) {}
	else
		modelPanel.getInterneModelPanel().getCodecPanel().updateWithXmlNode((org.w3c.dom.Element)elt.getChildNodes().item(1));
		
		// The src number
	modelPanel.getInterneModelPanel().setSourceNumber(Integer.valueOf (elt.getAttribute("srcnumber")).shortValue() );
		// The duration.
	timePanel.setDuration(Integer.valueOf (elt.getAttribute("duration")).shortValue() );
}

void addFlowToListPanel(org.w3c.dom.Element elt) {
	javax.swing.tree.DefaultMutableTreeNode flow = modelsListPanel.fxtpClient.addFlowAsNode(elt);
	org.w3c.dom.NodeList nodes = elt.getChildNodes();
	for (int i=0; i < nodes.getLength(); i++) {
		if (nodes.item(i).getNodeName().equals("model"))
			modelsListPanel.fxtpClient.addModelAsNode(flow,(org.w3c.dom.Element)nodes.item(i));
		if (nodes.item(i).getNodeName().equals("codec"))
			modelsListPanel.fxtpClient.addCodecAsNode(flow,(org.w3c.dom.Element)nodes.item(i));
	}
}

public void freezeInfo(boolean b) {
	timePanel.freezeInfo(b);
	modelPanel.freezeInfo(b);
	modelsListPanel.freezeInfo(b);
}

public void reset() {
	timePanel.reset();
	modelPanel.reset();
	modelsListPanel.reset();
}

public Object[] getTreeElements() {
	return modelsListPanel.fxtpClient.getCurrentTreeImage();
}
// used by InterneModelPanel when we charge test duration from a configuration file
public void updateResultMinutes() {	timePanel.updateResultMinutes(); }

  
    /**
     * Methode main destinee a tester la methode
     */
    public static void main(String[] args) {
		GestionTest gt = new GestionTest();
		ClientPanel sp1 = new ClientPanel(gt);
		
		String[] param = sp1.getModelParam();
		for (int i=0;i<param.length;i++){
		System.out.println(param[i]);	
	}
  }
}
