/**
 * DistributionPanel : this defines the panel where you specify distributions and their parameters
 * 
 * @author maria ladoue
 * @version 0.1, 26/08/02 
 */

package graphics2.panel.client;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import statistic.distribution.*;


public class DistributionPanel extends JPanel implements ActionListener {

  public int nbParams;
  protected GenericDistribution distribution;

  public JComboBox distributionName = new JComboBox();
  protected JLabel[] paramName = new JLabel[2];
  protected JTextField[] paramValue = new JTextField[2];
  protected TitledBorder tb;
  protected Hashtable listDistribution;
  protected JLabel meanL,varianceL;
  protected String meanS = "mean = ",varianceS = "variance = ";
  protected DecimalFormat format = new DecimalFormat();
 
  
  protected ParamListener paramListener;
  
 
/**
 * Method DistributionPanel.
 * @param name
 * @param listDistribution
 */
  public DistributionPanel(String name, Hashtable listDistribution) {
    super();
    this.listDistribution = listDistribution;
	
    format.setMinimumIntegerDigits(1);
    format.setMaximumFractionDigits(3);
    format.setMinimumFractionDigits(1);
 
 	createComboBox();
	paramListener = new ParamListener();
 	
	this.setLayout( new BoxLayout(this,BoxLayout.Y_AXIS) );  
  
    // Creation des Labels contenant le nom des parametres
	JPanel[] panel = new JPanel[3];
	
    panel[0] = new JPanel();
    panel[0].setLayout(new BoxLayout(panel[0],BoxLayout.X_AXIS));
	panel[0].add(Box.createRigidArea(new Dimension(5,0)));
	
	panel[1] = new JPanel();
    panel[1].setLayout(new BoxLayout(panel[1],BoxLayout.X_AXIS));
	panel[1].add(Box.createRigidArea(new Dimension(5,0)));
	
    for (int i=0;i<2;i++) {
	    paramName[i] = new JLabel();
	    paramName[i].setHorizontalAlignment(JLabel.RIGHT);
	     
		panel[i].add( paramName[i]);
		panel[i].add(Box.createRigidArea(new Dimension(5,0)));
	   	panel[i].add(paramValue[i] = new JTextField("",5));
	    paramValue[i].addActionListener(paramListener);
		paramValue[i].setMaximumSize(new Dimension(paramValue[i].getPreferredSize().width, 
																				 paramValue[i].getPreferredSize().height));
	
   }
 
 	panel[2] = new JPanel();
 	panel[2].setLayout(new BoxLayout(panel[2],BoxLayout.X_AXIS));
	panel[2].add(Box.createRigidArea(new Dimension(5,0)));
	panel[2].add(distributionName);
	panel[2].add(Box.createHorizontalGlue());
	

	panel[0].add(Box.createRigidArea(new Dimension(20,0))); // 40,0    
	panel[0].add(meanL = new JLabel(meanS+ " ?    "));
	panel[0].add(Box.createRigidArea(new Dimension(10,0))); // 30,0
	
	panel[1].add(Box.createRigidArea(new Dimension(20,0)));
	panel[1].add(varianceL = new JLabel(varianceS+ " ?    "));
	panel[1].add(Box.createRigidArea(new Dimension(10,0)));


	add(panel[2]);
	add(Box.createRigidArea(new Dimension(0,10)));
	add(panel[0]);
	add(Box.createRigidArea(new Dimension(0,5)));
	add(panel[1]);
	add(Box.createRigidArea(new Dimension(0,5)));
	
    updateDistribution(getDistributionName());
    updateParamName();

    // Set the Border
    setBorder( tb = new TitledBorder(name) );
  }
  
  public void createComboBox() {
  
	// Creation de la liste des lois disponibles
	// pour acceder aux elements de la HashTable 
	Enumeration valDistributionName = listDistribution.elements();

	while (valDistributionName.hasMoreElements()) {
	  distributionName.addItem( valDistributionName.nextElement() );
	}

	try {
		distributionName.setEditable(false);  
		distributionName.setSelectedIndex(0); // 0 is first item selected and -1 it's for no selection
		distributionName.setMaximumRowCount( 4 );
		distributionName.setMaximumSize(new Dimension(distributionName.getPreferredSize().width, 
																					distributionName.getPreferredSize().height));

		distributionName.addActionListener(this);
	}
	catch (IllegalArgumentException iae) {
		iae.printStackTrace();
	}
  }

  /**
   * Restore les parametres a partir de l'object Distribution
   * @param GenericDistribution
   */
  public void setDistribution(GenericDistribution distribution) {

    this.distribution = distribution;
    distributionName.setSelectedItem( listDistribution.get(distribution.getClass().getName()));
    updateParamName();
  
     nbParams = distribution.getParamsCount();  // avant avec int devant
	
    for (int i = 0; i < nbParams; i++) {
      paramValue[i].setText( ""+distribution.getParam(i) );
    }
    updateMoments();
  }

  // retour la loi choisie
  public GenericDistribution getDistribution() {  return distribution;  }

/**
 * Method freeze. to do a configuration file
 * @param b
 */
  public void freezeInfo(boolean b) {
    distributionName.setEnabled(!b);
  
    for(int i=0;i<distribution.getParamsCount();i++) {
      paramName[i].setEnabled(!b);
      paramValue[i].setEditable(!b);
    }
  }
  
  public void freezeDistributionName(boolean b) {
  	distributionName.setEnabled(!b);
  }
  
  public void reset() {
	distributionName.setSelectedIndex(0);
  
	  for(int i=0;i<distribution.getParamsCount();i++) {
		paramValue[i].setText("");
	  }
  }
  

  // change le nom du parametre de la loi choisie
  public void actionPerformed(ActionEvent e) {
    updateDistribution(getDistributionName());
    nbParams = distribution.getParamsCount();
    updateParamName();
    updateMoments();
  }

  // met a jour le DistributionPanel ( essentiellement modifie le nom des parametres + equation en tooltip en fonction du nom de la loi
   protected void updateDistribution(String classDistributionName) {
   	
     try {
       distribution = (GenericDistribution)Class.forName("statistic.distribution." + classDistributionName+"Distribution").newInstance();
	   distributionName.setToolTipText( distribution.getEquation());
     }
    
	 catch (IllegalAccessException iae) {
    		System.out.println(iae);
    	}
  	catch(InstantiationException ie) { 
     	System.out.println("DistributionPanel.updateDistribution : the initialization provoked by this methode fails => Distribution Name : " + classDistributionName + " "  + ie);
     }
     catch (ClassNotFoundException cnfe) {
     	System.out.println("DistributionPanel.updateDistribution : the class cannot be located => Distribution Name : " + classDistributionName + " " + cnfe);
     }
  }

  public void updateParamName() {
    String[] pName = distribution.getParamsNames();
	distributionName.setToolTipText( distribution.getEquation());

    for (int i = 0;i<2;i++) {
      if (i<pName.length) {
        paramName[i].setText( pName[i] + " :");
        paramValue[i].setEditable(true);
      }
      else {
          paramName[i].setText(" ");
          paramValue[i].setText("");
          paramValue[i].setEditable(false);
      }
    }
  }
 
 
/**
 * Method updateMoments.
 *   Pour mettre a jour les valeurs de mean et var
 */
  protected void updateMoments(){
      double mean,var;
      double[] param = new double[distribution.getParamsCount()];
      
      try {
	  for (int i=0;i<distribution.getParamsCount();i++) {
	      if (paramValue[i].getText().length() == 0 ) {
		  param[i] = 0.0f;
	      }
	      else {
		  param[i] = Double.valueOf( paramValue[i].getText() ).doubleValue();
	      }
	  }
       	
      distribution.setParams(param);
      
      mean  = distribution.mean();
      var  = distribution.var();
      } 
      catch (Exception e) {
	  	meanL.setText(meanS + " ?    ");
	  	varianceL.setText(varianceS + " ?    ");
	  	
	  return;
      }
    meanL.setText(meanS + format.format(mean));
    varianceL.setText(varianceS + format.format(var));
  }

// pour positionner et recuperer les donnees
public void setParams(String[] paramsV) {
	
	double[] params = new double[paramsV.length];
	for (int i=0;i<paramsV.length;i++) {
		params[i] = Double.parseDouble(paramsV[i]);
		paramValue[i].setText("" + params[i] );	
	}
}

public String[] getParams() {	
	String[] ans = new String[distribution.getParamsCount()];
	
	for (int i=0;i<distribution.getParamsCount();i++) {	
		if (!paramValue[i].getText().equals(new String(""))) {
		ans[i] = paramValue[i].getText();
		}
		else {
			//JOptionPane.showMessageDialog(null, "Alert: there is no value specify for the parameter: " + ((paramName[i].getText()).split(" ",2)[0]) + "\n of the \"" + tb.getTitle() + "\" field!" , "Error parameter", JOptionPane.ERROR_MESSAGE); 
		}
	}
	return ans;
}


public void setDistributionName( String distributionN ) { distributionName.setSelectedItem(distributionN);}

public String getDistributionName() { return distributionName.getSelectedItem().toString(); }


    // Classes Internes
 protected class ParamListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      updateMoments();
    }
  }
  
  private boolean isAFloat(String str, String str2) {
	  try {
		  if (str==null || str.equalsIgnoreCase(""))
			  throw new NumberFormatException();
		  double value = Double.parseDouble(str);
	  }
	  catch (NumberFormatException nfe) {
		  JOptionPane.showMessageDialog(this, "Wrong format for "+str2+". This is not a float!","Float format error",JOptionPane.ERROR_MESSAGE);
		  // Please Insert a number !
		  return false;
	  }
	  return true;
  }
  	
    
  /**
   * Check that all fields are correctly filled ! If not, popup an alert
   * window.
   * @return false if a field cotains a wrong value.
   */
	public boolean isAValidPanel() {
		double[] par = new double[distribution.getParamsCount()];
		for (int i=0;i<distribution.getParamsCount();i++) {	
			if (!isAFloat(paramValue[i].getText(), distribution.getParamsNames()[i]))
				return false;
			par[i]=Double.parseDouble(paramValue[i].getText());
		}
		// Now, check that these values are ok for the corresponding distribution !
		if (!distribution.hasValidParam(par)) {
			utils.MessageManager.warningMessage("Wrong parameters for the distribution "+ distribution.getName(), true); 			
			return false;
		}
		return true;
	}
    
    /**
     * Main methode : pour tester la classe
     */

    public static void main(String[] args) {
	
	JFrame win = new JFrame("DistributionPanel");
	Hashtable ll = new Hashtable();
	ll.put("statistic.distribution.ExponentialDistribution","Exponential");
	ll.put("statistic.distribution.WeibullDistribution","Weibull");
	DistributionPanel distribution = new DistributionPanel("Panel Distribution", ll);
	System.out.println(distribution.getDistributionName());
	String[] param = distribution.getParams();
	for (int i=0;i<param.length;i++){
		System.out.println(param[i]);
	}
	win.getContentPane().add(distribution);
	
	win.pack();
	win.setVisible(true);
    }   
}
