package graphics2.panel.client;

import statistic.distribution.*;
import utils.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;


/**
 * MixedDistributionPanel
 *
 * @author mladoue
 * @version 0.1, 31/01/03
 */
public class MixedDistributionPanel extends JPanel implements ActionListener {
    protected double[] weigthDistribution = new double[2]; // cas particuliers ou il n'y a que de distrib possibles
    protected JCheckBox mixedDistribution;
    protected DistributionPanel[] distribPanels = new DistributionPanel[2];
    protected JTextField[] coeff = new JTextField[2];
    protected JLabel[] coeffName = new JLabel[2];
    protected TitledBorder tb;
    protected DistributionNameListener dnl;
    public boolean isMixedDist = false;
    protected int nbPanel;

    // protected ParamListener paramListener;
    protected DecimalFormat format;

    // cas particulier ou il n'y a que 2 distributions!!!!!
    public MixedDistributionPanel(String name,
        Hashtable listContinuousDistribution, int nbPanel) {
        super();
        this.nbPanel = nbPanel;

        if (nbPanel == 1) {
            isMixedDist = false;
        } else {
            isMixedDist = true;
        }

        mixedDistribution = new JCheckBox("Mixed Distribution", isMixedDist);
        distribPanels[0] = new DistributionPanel("", listContinuousDistribution);
        distribPanels[1] = new DistributionPanel("", listContinuousDistribution);

        dnl = new DistributionNameListener();

        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        JPanel panel1 = new JPanel();
        panel1.setLayout(new BoxLayout(panel1, BoxLayout.X_AXIS));

        panel1.add(Box.createRigidArea(new Dimension(5, 0)));
        panel1.add(mixedDistribution);
        mixedDistribution.addItemListener(dnl);
        mixedDistribution.addActionListener(this);
        panel1.add(Box.createRigidArea(new Dimension(30, 0)));
        panel1.add(MessageManager.createHelpMessageDialogButton(
                "Help window...to do"));
        panel1.add(Box.createRigidArea(new Dimension(10, 0)));
        panel1.add(Box.createRigidArea(new Dimension(10, 0)));

        JPanel panel2 = new JPanel();
        panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));

        JPanel[] panel = new JPanel[2];

		WeightListener wlistener = new WeightListener();
       for (int i = 0; i < 2; i++) {
            panel[i] = new JPanel();
            panel[i].setLayout(new BoxLayout(panel[i], BoxLayout.X_AXIS));
            panel[i].add(Box.createRigidArea(new Dimension(5, 0)));
            panel[i].add(coeffName[i] = new JLabel("Distribution " + (i + 1) +
                        " weight:"));
            panel[i].add(Box.createRigidArea(new Dimension(10, 0)));
            panel[i].add(coeff[i] = new JTextField("1.0", 6));
			coeff[i].setMaximumSize(new Dimension(coeff[i].getMaximumSize().width, 
																		  coeff[i].getPreferredSize().height));

			WeightFocusListener wflistener = new WeightFocusListener(i);
			coeff[i].addActionListener(wlistener);
			coeff[i].addFocusListener(wflistener);
  
            if (isMixedDist) {
                coeff[i].setEditable(true);
                coeffName[i].setEnabled(true);
            } else {
                coeff[i].setEditable(false);
                coeffName[i].setEnabled(false);
            }
        }

        if (isMixedDist) {
            coeff[1].setEditable(true);
        } else {
            coeff[1].setEditable(false);
        }

        JPanel panel3 = new JPanel();
        panel3.setLayout(new BoxLayout(panel3, BoxLayout.Y_AXIS));

        panel3.add(panel[0]);
        panel3.add(Box.createRigidArea(new Dimension(0, 10)));
        panel3.add(distribPanels[0]);
        panel3.add(Box.createRigidArea(new Dimension(0, 5)));

        JPanel panel4 = new JPanel();
        panel4.setLayout(new BoxLayout(panel4, BoxLayout.Y_AXIS));

        panel4.add(panel[1]);
        panel4.add(Box.createRigidArea(new Dimension(0, 10)));
        panel4.add(distribPanels[1]);
        distribPanels[1].freezeInfo(!isMixedDist);
        panel4.add(Box.createRigidArea(new Dimension(0, 5)));

        panel2.add(Box.createRigidArea(new Dimension(5, 0)));
        panel2.add(panel3);
        panel2.add(Box.createRigidArea(new Dimension(5, 0)));
        panel2.add(panel4);
        panel2.add(Box.createRigidArea(new Dimension(5, 0)));

        add(panel1);
        add(Box.createRigidArea(new Dimension(0, 5)));
        add(panel2);

        updateDistributions(getDistributionsNames());

        // Set the Border
        setBorder(tb = new TitledBorder(name));
    }

    /**
     * Restore les parametres a partir de l'object DistributionPanel[]
     * @param dps
     */
    public void setPanel(DistributionPanel[] dps) {
        this.distribPanels = dps;
    }

    public DistributionPanel[] getPanel() {
        return distribPanels;
    }

    /**
     * Method freezeInfo to do a configuration file
     * @param b
     */
    public void freezeInfo(boolean b) {
        mixedDistribution.setEnabled(!b);

        for (int i = 0; i < nbPanel; i++) {
            coeffName[i].setEnabled(!b);
            coeff[i].setEditable(!b);
            distribPanels[i].freezeInfo(b);
        }
    }

    public void reset() {
        mixedDistribution.setEnabled(true);

        for (int i = 0; i < nbPanel; i++) {
            coeff[i].setText("");
            distribPanels[i].reset();
        }
    }

    public void actionPerformed(ActionEvent e) {
    }

    public int getPanelNumber() {
        return nbPanel;
    }

    public void setPanelNumber(int nbPanel) {
        this.nbPanel = nbPanel;

        if (nbPanel == 1) {
            isMixedDist = false;
        } else {
            isMixedDist = true;
        }

        System.out.println("value of bool=" + isMixedDist);
        mixedDistribution.setSelected(isMixedDist);
    }

    public GenericContinuousDistribution getDistribution(int idx)
        throws IllegalArgumentException {
        return (GenericContinuousDistribution) distribPanels[idx].getDistribution();
    }

    public GenericContinuousDistribution[] getDistributions() {
        GenericContinuousDistribution[] gcd = new GenericContinuousDistribution[2];
        gcd[0] = (GenericContinuousDistribution) distribPanels[0].getDistribution();
        gcd[1] = (GenericContinuousDistribution) distribPanels[1].getDistribution();

        return gcd;
    }

    public void setDistribution(int panelNb, GenericContinuousDistribution gcd)
        throws IllegalArgumentException {
			distribPanels[panelNb].setDistribution( gcd);
    }

    public void setDistributions(GenericContinuousDistribution[] gcd) {
        for (int i = 0; i < gcd.length; i++) {
            distribPanels[i].setDistribution(gcd[i]);
        }
    }

    public void setMixedDistribution(MixedDistribution md) {
        setDistributions(md.getSpecificDistributions());
    }

    public MixedDistribution getMixedDistribution() {
		GenericContinuousDistribution[] gdist = new GenericContinuousDistribution[distribPanels.length];
    	for (int i=0; i<distribPanels.length;i++)
    		gdist[i]=(GenericContinuousDistribution)distribPanels[i].getDistribution();
       return new MixedDistribution(gdist, weigthDistribution);
   }

    // Distribution name
    public void setDistributionName(int idx, String distName)
        throws IllegalArgumentException {
        distribPanels[idx].setDistributionName(distName);
    }

    public String getDistributionName(int idx) throws IllegalArgumentException {
        return distribPanels[idx].getDistributionName();
    }

    public void setDistributionsNames(String[] distNames)
        throws IllegalArgumentException {
        for (int i = 0; i < distNames.length; i++) {
            distribPanels[i].setDistributionName(distNames[i]);
        }
    }

    public String[] getDistributionsNames() {
        String[] ans = new String[getPanelNumber()];

        for (int i = 0; i < getPanelNumber(); i++) {
            ans[i] = distribPanels[i].getDistributionName();
        }

        return ans;
    }

    // Distribution weight
    public void setDistributionsWeight(double[] weight) {
        for (int i = 0; i < weight.length; i++) {
            coeff[i].setText("" + weight[i]);
        }
    }

    public void setDistributionWeight(int distNb, double weight) {
        coeff[distNb].setText("" + weight);
    }

    public double[] getDistributionsWeight() {
        double[] ans = new double[nbPanel];

        for (int i = 0; i < nbPanel; i++) {
            if (!coeff[i].getText().equals(new String(""))) {
                ans[i] = Double.parseDouble(coeff[i].getText());
            } else {
                JOptionPane.showMessageDialog(null,
                    "Alert: there is no value specify for the parameter: " +
                    (coeffName[i].getText()) + "\n of the \"" + tb.getTitle() +
                    "\" field!", "Error parameter", JOptionPane.ERROR_MESSAGE);
            }
        }

        return ans;
    }

    public double getDistributionWeight(int distNb) {
        return Double.parseDouble(coeff[distNb].getText());
    }

    // Distribution parameter
    // pour positionner et recuperer les donnees
    //Format:  <i>parameter(s) distrib(s)1</i> <i>parameter(s) distrib(s)2</i> <i>parameter(s) distrib3(s)</i>
    public void setParams(String[] paramsV) {
        double[] params = new double[paramsV.length];
        int k = 0;

        for (int i = 0; i < nbPanel; i++) {
            for (int j = 0; j < distribPanels[i].getDistribution().getParamsCount(); j++) {
                params[k] = Double.parseDouble(paramsV[k]);
                distribPanels[i].paramValue[j].setText("" + params[k]);
                k++;
            }
        }
    }

    public void setParams(int distNb, String[] params) {
        for (int j = 0; j < distribPanels[distNb].getDistribution().getParamsCount(); j++) {
            distribPanels[distNb].paramValue[j].setText("" + params[j]);
        }
    }

    public String[] getParams() {
        String[] ans = new String[getParamCount()];
        int k = 0;

        for (int i = 0; i < nbPanel; i++) {
            for (int j = 0; j < distribPanels[i].getParams().length; j++) {
                ans[k] = distribPanels[i].getParams()[j];
                k++;
            }
        }

        return ans;
    }

    /**
     * Method getParamCount. To get the parameter count of all the distributions
     * @return int
     */
    public int getParamCount() {
        int length = 0;

        for (int i = 0; i < nbPanel; i++) {
            length +=  distribPanels[i].getDistribution().getParamsCount();
        }

        return length;
    }

    public void updateDistribution(int idx, String classDistributionName) {
        try {
            GenericContinuousDistribution dist = (GenericContinuousDistribution) Class.forName(
                    "statistic.distribution." + classDistributionName +
                    "Distribution").newInstance();
			distribPanels[idx].setDistribution(dist);
        } catch (IllegalAccessException iae) {
            System.out.println(iae);
        } catch (InstantiationException ie) {
            System.out.println(
                "MixedDistributionPanel.updateDistributions : the initialization provoked by this methode fails => Distribution Name : " +
                classDistributionName + " " + ie);
        } catch (ClassNotFoundException cnfe) {
            System.out.println(
                "MixedDistributionPanel.updateDistributions :The class cannot be located => Distribution Name : " +
                classDistributionName + " " + cnfe);
        }
    }

    // met a jour le MixedDistributionPanel ( essentiellement modifie le nom des parametres + equation en fonction du nom de la loi
    protected void updateDistributions(String[] classDistributionName) {
        try {
            for (int i = 0; i < classDistributionName.length; i++) {
				GenericContinuousDistribution dist  = (GenericContinuousDistribution) Class.forName(
                        "statistic.distribution." + classDistributionName[i] +
                        "Distribution").newInstance();
				distribPanels[i].setDistribution(dist);
            }
        } catch (IllegalAccessException iae) {
            System.out.println(iae);
        } catch (InstantiationException ie) {
            System.out.println(
                "MixedDistributionPanel.updateDistributions : the initialization provoked by this methode fails => Distribution Name : " +
                classDistributionName + " " + ie);
        } catch (ClassNotFoundException cnfe) {
            System.out.println(
                "MixedDistributionPanel.updateDistributions :The class cannot be located => Distribution Name : " +
                classDistributionName + " " + cnfe);
        }
    }

    protected class DistributionNameListener implements ItemListener {
        public DistributionNameListener() {
        }

        public void itemStateChanged(ItemEvent e) {
            //Object source = e.getItemSelectable();
            //Now that we know which button was pushed, find out
            //whether it was selected or deselected.
            if (e.getStateChange() == ItemEvent.DESELECTED) {
                isMixedDist = false;
                nbPanel = 1;
                coeffName[0].setEnabled(true);
                coeff[0].setEditable(true);
                coeffName[1].setEnabled(false);
                coeff[1].setEditable(false);
                distribPanels[1].freezeInfo(true);
            } else {
                isMixedDist = true;
                nbPanel = 2;
                coeffName[0].setEnabled(true);
                coeff[0].setEditable(true);
                coeffName[1].setEnabled(true);
                coeff[1].setEditable(true);
                distribPanels[1].freezeInfo(false);
            }
        }
    }
    
	private boolean isAFloat(String str, String str2) {
		try {
			if (str==null || str.equalsIgnoreCase(""))
				throw new NumberFormatException();
			double value = Double.parseDouble(str);
		}
		catch (NumberFormatException nfe) {
			JOptionPane.showMessageDialog(this, "Wrong format for "+str2+". This is not a float!","Float format error",JOptionPane.ERROR_MESSAGE);
			// Please Insert a number !
			return false;
		}
		return true;
	}

   /**
     * Check that all fields are correctly filled ! If not, popup an alert
     * window.
     * @return false if a field cotains a wrong value.
     */
    public boolean isAValidPanel() {
    	if (distribPanels[0].isAValidPanel()) {
    		if (isMixedDist &&  
				(!isAFloat(coeff[0].getText(), "Distribution weigth1") ||
				!isAFloat(coeff[1].getText(), "Distribution weigth2") ||
	    		!distribPanels[1].isAValidPanel()))
    			return false;
    		return true;
    	}
    	else
    		return false;
    }
    
	// Respond to weigth modification
	/**
	 * Return a String that is the result of 1.0 - number
	 * number MUST be a number starting with "0."
	 * The float is converted into integer to prevent rounded error.
	 */
	private String getOtherValue(String number) {
		int inc=0;
		
		// First, check that the value is valid !
		if (number.equalsIgnoreCase("")  || Double.parseDouble(number) > 1.0) {
			coeff[0].setText("0.5"); // Default the first TextField to 0.5 !
			coeff[1].setText("0.5");
			return ("0.5");
		}

		// Secon, remove the "0." in the string....
		number=number.substring(2);
		inc = number.length(); // length numebr
		long val = Long.parseLong(number);
		return new String("0."+((long)Math.pow(10,inc) - val));
	}

	// Classe interne
	// Listen weith modification (Must press ENTER to execute this action)
	 class WeightListener implements ActionListener {
			public void actionPerformed(ActionEvent e) {
				   // La modification de l'un des champs weigth entraine la modif de l'autre.
				   // getActionCommand du ActionEvent contient la valeur du champs que l'on vient de modifier.
				   if (e.getActionCommand().equals(coeff[0].getText())) {
						coeff[1].setText(getOtherValue(coeff[0].getText()));
				   }
				   else {
						coeff[0].setText(getOtherValue(coeff[1].getText()));
				   }
			}
	  }
	
	class WeightFocusListener implements FocusListener {
		private int pos;
				
		public WeightFocusListener(int pos) {
			this.pos=pos;
		}
		public void focusGained(FocusEvent e) {
			// Do nothing			
		}

		public void focusLost(FocusEvent e) {
			if (pos==0) {
				coeff[1].setText(getOtherValue(coeff[0].getText()));
		   }
		   else {
				coeff[0].setText(getOtherValue(coeff[1].getText()));
		   }
		}
	}

}
