package graphics2.panel.client.model;

/**
 * MGInfModelPanel  Class ou on definit toute l'interfece graphique permettant de definir un modele M/G?Inf
 * 
 * @author mladoue
 * @version 0.1, 17/01/03
 */

import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

import statistic.distribution.*;
import statistic.function.*;
import multimedia.*;
import graphics2.panel.client.*;

public class MGInfModelPanel extends GenericModelPanel { //implements ActionListener {

	protected DistributionPanel[] distPanel = new DistributionPanel[3];
	protected FunctionPanel functionPanel;
	protected JLabel lambdaL;
	protected JTextField lambdaF;
	protected JTabbedPane jTabbedpanel;
	
	protected Hashtable listFunction;
	protected Hashtable listDistribution;
	
    public MGInfModelPanel() {
		super();
		modelType = VIDEO;
		modelParametersNumber = 6;  // minumum number of parameters
	}
	
	public MGInfModelPanel(InterneModelPanel imp) {
            super(imp);	
		modelType = VIDEO;
		modelParametersNumber = 6;  // minumum number of parameters
		listFunction = imp.getListFunction();
		listDistribution = imp.getListDistribution();
		
		distPanel[0] = new DistributionPanel("Amount of data per slot",listDistribution);
		distPanel[1] = new DistributionPanel("Data distribution in slot",listDistribution);
		distPanel[2] = new DistributionPanel("Packet size distribution",listDistribution);
		functionPanel = new FunctionPanel("AutoCorrelation Function",listFunction);
		
		jTabbedpanel = new JTabbedPane();
		//jTabbedpanel.setLayout(new BoxLayout(jTabbedpanel,BoxLayout.X_AXIS));
		jTabbedpanel.add("data/slot",distPanel[0]);
		jTabbedpanel.add("dist. in slot",distPanel[1]);
		jTabbedpanel.add("size dist.",distPanel[2]);

		JPanel panel0 = new JPanel();
		panel0.setLayout(new BoxLayout(panel0,BoxLayout.X_AXIS));
		panel0.add(jTabbedpanel);
		panel0.add( Box.createRigidArea(new Dimension(20,0)));
		
		this.setLayout(new BoxLayout(this,BoxLayout.X_AXIS));
	
		JPanel mginfPanel = new JPanel();
		mginfPanel.setLayout( new BoxLayout(mginfPanel,BoxLayout.X_AXIS) );
	
		
		JPanel panel1 = new JPanel();
		panel1.setLayout(new BoxLayout(panel1,BoxLayout.Y_AXIS));
	
		JPanel panel11 = new JPanel();
	    panel11.setLayout(new BoxLayout(panel11,BoxLayout.X_AXIS));
	    
		panel11.add( Box.createRigidArea(new Dimension(10,0)));
		panel11.add( lambdaL = new JLabel("Lambda : "));
		panel11.add( Box.createRigidArea(new Dimension(10,0)));
    	panel11.add ( lambdaF = new JTextField("1",7) );
		lambdaF.setMaximumSize(new Dimension(lambdaF.getPreferredSize().width, 
																		lambdaF.getPreferredSize().height));
		
		panel11.add( Box.createRigidArea(new Dimension(10,0)));
		
		panel1.add(functionPanel);
		panel1.add(Box.createRigidArea(new Dimension(0,10)));
		panel1.add(panel11);
		panel1.add(Box.createRigidArea(new Dimension(0,10)));
		
		mginfPanel.add(Box.createRigidArea(new Dimension(5,0)));
		mginfPanel.add(panel0);
		mginfPanel.add(Box.createRigidArea(new Dimension(5,0)));
		mginfPanel.add(panel1);
		mginfPanel.add(Box.createRigidArea(new Dimension(5,0)));
		
		add(mginfPanel);
		
		//Positionner le nb de parametre
		setModelParametersNumber();
		
		 // Set the Border
    	setBorder( new TitledBorder("M/G/Infinity Model features") );
	}
	
	/**
	 * Method getDistributionPanel.
	 * @param idx
	 * @return DistributionPanel 0: amount of data, 1: data distribution, 2: packet size distribution
	 * @throws IllegalArgumentException
	 */
	public DistributionPanel getDistributionPanel(int idx) throws IllegalArgumentException { return distPanel[idx]; 	}
	
	public FunctionPanel getFunctionPanel() { return functionPanel; 	}
	
	/**
	 * Method setDistribution.
	 * @param idx 0: amount of data, 1: data distribution, 2: packet size distribution
	 * @param distName
	 */
	public void setDistributionName(int idx,String distName) throws IllegalArgumentException { distPanel[idx].setDistributionName(distName);	}
	
	public void setDistributionParameters(int idx,String[] paramsV) { distPanel[idx].setParams(paramsV); 	}
	
	public void setFunctionName(String funcName) { functionPanel.setFunctionName( funcName );	}
	
	public void setFunctionParameter(double b) { functionPanel.setParam(b);	}
	
	public void setLambda(double value) { lambdaF.setText("" + value); }
	
	public String getFunctionName() { return functionPanel.getFunctionName();}
	
	public double getFunctionParameter() { return functionPanel.getParam();	}

	public double getLambda() {
		String lambda =  lambdaF.getText();
		if ( (! lambda.equals("")) && ( Double.parseDouble(lambda) > 0)) {
			return Double.parseDouble(lambda);
		}
		else {
			throw new NullPointerException("There is no specified value in the lambda field.");
		}
	}		

	/**
	 * Method getDistributionName.
	 * @param idx	0: amount of data, 1: data distribution, 2: packet size distribution
	 * @return String
	 * @throws IllegalArgumentException
	 */
	public String getDistributionName(int idx) throws IllegalArgumentException { 	return distPanel[idx].getDistributionName();	}
	
	public String[] getDistributionParameters(int idx) {		return distPanel[idx].getParams();	}
	
protected void setModelParametersNumber() {
	modelParametersNumber = 6;
	for (int i=0;i<3;i++) {
		modelParametersNumber += distPanel[i].getDistribution().getParamsCount();
	}
}
  
  public int getParametersNumber() {
		setModelParametersNumber();
		//System.out.println("Nb de parametre" + modelParametersNumber);
		return modelParametersNumber;	
	}
	
	
// Methodes definies dans le cas d'un utilisation des MGInfModel pre-definis: JurassicPark

	public void setDistributions(VideoDefinition vd) {
		GenericContinuousDistribution distributionTest = vd.getModel().getDistributionTest();
		distPanel[0].setDistribution(distributionTest);
	
		GenericDistribution distributionSlot = vd.getModel().getDistributionSlot();
		distPanel[1].setDistribution(distributionSlot);
	
		GenericDistribution distributionPacketSize = vd.getModel().getDistributionPacketSize();
		distPanel[2].setDistribution(distributionPacketSize);
	}	

	public void updateParam(VideoDefinition vd) {
		setDistributions(vd);
		setFunctionName( (((MixedFunction)vd.getModel().getAutoCorr()).getSpecificFunctions()[0]).getName() );
		setFunctionParameter( (((MixedFunction)vd.getModel().getAutoCorr()).getSpecificFunctions()[0]).getParam()) ;
		setLambda(vd.getModel().getLambda());
	}


//permet de positionner tous les champs si ca n'est pas fait et bloquer leur acces
public void freezeInfo(boolean b) {
	for (int i=0;i<3;i++) {
		distPanel[i].freezeInfo(b);	
	}
	functionPanel.freezeInfo(b);
	lambdaF.setEditable(!b);
}

// permet de mettre les valeurs par defaut!
public void reset() {
	for (int i=0;i<3;i++) {
		distPanel[i].reset();
	}
	functionPanel.reset();
	lambdaF.setText("1.0");  // default value
}

	public boolean isAValidModel() {
		if (functionPanel.isAValidPanel()  &&
			isAFloat(lambdaF.getText(), "Lambda") ) {
				// Now, check the distribution...
				for (int j=0; j < distPanel.length; j++) {
					if (!distPanel[j].isAValidPanel())
						return false;
				}
				return true;
			}
		else
			return false;
	}
    public org.w3c.dom.Element buildNewXmlNode() {
            // data values in the Panel should be ok
            // Access Configuration Tree.
        graphics2.ConfigurationXMLFile conftree = imp.getModelPanel().getClientPanel().getListScenario();
        org.w3c.dom.Element elem = conftree.createNewXMLElement("scenario_mginf");
        
            // Add the model.
        org.w3c.dom.Element modelchild = conftree.createElement("model");
        modelchild.setAttribute("name", "MGInf");
        elem.appendChild(modelchild);
        
            // Add the Function
        org.w3c.dom.Element fctchild = conftree.createElement("function");
        fctchild.setAttribute("name",  getFunctionName());
        fctchild.setAttribute("param1", String.valueOf( getFunctionParameter()));
        modelchild.appendChild(fctchild);

            // Add lambda1 value node to the model
        org.w3c.dom.Element lambdachild = conftree.createElement("lambda1");
        lambdachild.setAttribute("value", String.valueOf(getLambda()));
        modelchild.appendChild(lambdachild);
      
           // Add the 3 distributions...
        for (int j=0; j < distPanel.length; j++) {
            org.w3c.dom.Element dischild = conftree.createElement("distribution"+(j+1));
            dischild.setAttribute("name", getDistributionName(j));
            for(int i=0; i <distPanel[j].getParams().length; i++)
                dischild.setAttribute("param"+(i+1), distPanel[j].getParams()[i]);
            modelchild.appendChild(dischild);
        }

	return elem;
    }

	public void updateWithXmlNode(org.w3c.dom.Element elt) {
		org.w3c.dom.NodeList nlist = elt.getChildNodes() ;
		for (int i=0; i < nlist.getLength(); i++) {
			String name = nlist.item(i).getNodeName();
			if (name.equals("function")) {
				this.setFunctionName( ((org.w3c.dom.Element) nlist.item(i)).getAttribute("name"));
				this.setFunctionParameter( Double.valueOf(((org.w3c.dom.Element) nlist.item(i)).getAttribute("param1")).doubleValue() );
			}
			else if(name.equals("lambda1")) {
				setLambda(Double.valueOf(((org.w3c.dom.Element) nlist.item(i)).getAttribute("value")).doubleValue());
			}
			else if(name.equals("distribution1")) {
				setDistributionName(0, ((org.w3c.dom.Element) nlist.item(i)).getAttribute("name"));
				String[] params;
				if (((org.w3c.dom.Element) nlist.item(i)).getAttribute("param2").equals("")) {
					params = new String[1];
				}
				else {
					params = new String[2];
					params[1]=((org.w3c.dom.Element) nlist.item(i)).getAttribute("param2");
				}
				params[0]=((org.w3c.dom.Element) nlist.item(i)).getAttribute("param1");
				setDistributionParameters(0, params);
			}
			else if(name.equals("distribution2")) {
				setDistributionName(1, ((org.w3c.dom.Element) nlist.item(i)).getAttribute("name"));
				String[] params;
				if (((org.w3c.dom.Element) nlist.item(i)).getAttribute("param2").equals("")) {
					params = new String[1];
				}
				else {
					params = new String[2];
					params[1]=((org.w3c.dom.Element) nlist.item(i)).getAttribute("param2");
				}
				params[0]=((org.w3c.dom.Element) nlist.item(i)).getAttribute("param1");
				setDistributionParameters(1, params);
			}
			else if(name.equals("distribution3")) {
				setDistributionName(2, ((org.w3c.dom.Element) nlist.item(i)).getAttribute("name"));
				String[] params;
				if (((org.w3c.dom.Element) nlist.item(i)).getAttribute("param2").equals("")) {
					params = new String[1];
				}
				else {
					params = new String[2];
					params[1]=((org.w3c.dom.Element) nlist.item(i)).getAttribute("param2");
				}
				params[0]=((org.w3c.dom.Element) nlist.item(i)).getAttribute("param1");
				setDistributionParameters(2, params);
			}
		}
	}
}
