package graphics2.panel.client.model;

/**
 * OnOffModelPanel : Definition of the panel where you specify all the parameters of an On-Off Model
 * 
 * @author mladoue
 * @version 0.1, 09/01/03 
 */


import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;


import graphics2.panel.client.*;
import multimedia.codec.AudioCodec;
import multimedia.model.OnOffModel;
import utils.*;


public class OnOffModelPanel extends GenericModelPanel implements ActionListener {
	
	protected DistributionPanel[] distPanel = new DistributionPanel[2];
	
	protected Hashtable listDistribution;
	private JLabel etatDepartL = new JLabel("start states : ");
	private JTextField etatDepartF =  new JTextField("1", 7);

//	protected JTabbedPane jTabbedpanel;


/**
 * OnOffModelPanel : default constructor
 */
public OnOffModelPanel() {
	super();
 	modelType = AUDIO;
 	modelParametersNumber = 3;   // minumum number of parameter
}


public OnOffModelPanel(InterneModelPanel imp, String name) {
		super(imp);	
		modelType = AUDIO;
 		modelParametersNumber = 3;   // minumum number of parameter
		this.listDistribution = imp.getListDistribution();
		 
		distPanel[0] = new DistributionPanel("Talkspurt length", listDistribution);
		distPanel[1] = new DistributionPanel("Idle period",listDistribution);
		 
	    JPanel onoffPanel = new JPanel();
		onoffPanel.setLayout( new BoxLayout(onoffPanel,BoxLayout.X_AXIS) );
	
		this.setLayout( new BoxLayout(this,BoxLayout.X_AXIS) );
	

		
//		jTabbedpanel = new JTabbedPane();
//		jTabbedpanel.add("Talkspurt length",distPanel[0] );
//		jTabbedpanel.add("Idle period",distPanel[1] );
		
		onoffPanel.add( Box.createRigidArea(new Dimension(10,0)) );
		onoffPanel.add(distPanel[0]);
		onoffPanel.add(Box.createRigidArea(new Dimension(10,0)));
		onoffPanel.add(distPanel[1]);
//		onoffPanel.add( Box.createRigidArea(new Dimension(10,0)) );
//		onoffPanel.add(jTabbedpanel);
		distPanel[0].setDistributionName("Exponential");
		distPanel[1].setDistributionName("Exponential");
		distPanel[0].freezeDistributionName(true);
		distPanel[1].freezeDistributionName(true);
		onoffPanel.add(Box.createRigidArea( new Dimension(20,0)));
		
		JPanel panel1 = new JPanel();
		panel1.setLayout(new BoxLayout(panel1, BoxLayout.Y_AXIS));
		
		JPanel panel11 = new JPanel();
		panel11.setLayout(new BoxLayout(panel11,BoxLayout.X_AXIS));
		
		panel11.add( Box.createRigidArea(new Dimension(10,0)));
		panel11.add(etatDepartL);
		panel11.add( Box.createRigidArea(new Dimension(10,0)));
		panel11.add(etatDepartF);
		etatDepartF.setMaximumSize(new Dimension(etatDepartF.getPreferredSize().width, 
																			 etatDepartF.getPreferredSize().height));
		panel11.add( Box.createRigidArea(new Dimension(30,0)));
		
		panel1.add(Box.createRigidArea(new Dimension(0,20)));
		panel1.add(panel11);
		panel1.add(Box.createRigidArea(new Dimension(0,20)));
		panel1.add(MessageManager.createHelpMessageDialogButton("Help window...to do"));
		panel1.add(Box.createRigidArea(new Dimension(0,20)));
		
		onoffPanel.add(panel1);
		
		this.add(onoffPanel);
		
		//Positionner le nb de parametre
		setModelParametersNumber();
		
		 // Set the Border
    	//setBorder( new TitledBorder("On-Off Model features") );
    	setBorder( new TitledBorder(name) );
	
}

public DistributionPanel getDistributionPanel(int idx) { return distPanel[idx]; }

protected void setModelParametersNumber() {
	modelParametersNumber = 3;
	for (int i=0;i<2;i++) {
		modelParametersNumber += distPanel[i].getDistribution().getParamsCount();
	}
}

public int getParametersNumber() {
	setModelParametersNumber();
	return modelParametersNumber;	
}

// Quelques methodes pour pouvoir enregistrer/charger des configs
	/**
	 * Method setDistributionName
	 * @param idx 0: ON distribution, 1: OFFdistribution
	 * @param distName
	 */
	public void setDistributionName(int idx,String distName) throws IllegalArgumentException {
		distPanel[idx].setDistributionName(distName);
	}

	public void setDistributionParameters(int idx,String[] paramsV) { distPanel[idx].setParams(paramsV); }

	public void setStartState(int state) throws IllegalArgumentException { etatDepartF.setText("" + state); }

	/**
	 * Method getDistributionName.
	 * @param idx	0: ON distribution, 1: OFFdistribution
	 * @return String
	 * @throws IllegalArgumentException
	 */
	public String getDistributionName(int idx) throws IllegalArgumentException { return distPanel[idx].getDistributionName(); }
	
	/**
	 * Method getDistributionParameters.
	 * @param idx	0: ON distribution, 1: OFFdistribution
	 * @return String[]
	 */
	public String[] getDistributionParameters(int idx) { return distPanel[idx].getParams(); }

	public int getStartState() {
		int ans = 0;
    	if ( !((etatDepartF.getText()).equals(new String(""))) ) {
    		ans = Integer.parseInt( etatDepartF.getText() );
    	}
    	else {
			//JOptionPane.showMessageDialog(null, "Warning: There is no start state specify for the On-Off Model!" , "Error parameter", JOptionPane.ERROR_MESSAGE);
			MessageManager.warningMessage("There is no start state specify for the On-Off Model!", true); 
		}
		return ans;
	}

// ajout du 26/05/03
	public OnOffModel getModel() {
		OnOffModel model = new OnOffModel();
		model.setEtatDepart( getStartState());
		model.setOnDistribution(distPanel[0].getDistribution());
		model.setOffDistribution(distPanel[1].getDistribution());
		model.setCodec( new AudioCodec() ); // le codec n'est pas specifie!!!! => codec par defaut
		return model;
	}

public void actionPerformed(ActionEvent e) {	setModelParametersNumber(); }
 
 /**
 * Method reset : Permit to put to default value all the parameters of this Panel
 */
public void reset() {
	etatDepartF.setText("");
	for (int i=0;i<2;i++) {
		distPanel[i].reset();	
		distPanel[i].freezeDistributionName(true); // The distribution name is always freezed !
	}
 }
 
public void freezeInfo(boolean b) {
	for (int i=0;i<2;i++) {
		distPanel[i].freezeInfo(b);
		distPanel[i].freezeDistributionName(true); // The distribution name is always freezed !
	}
	etatDepartF.setEditable(!b);
}
	
	public boolean isAValidModel() {
		if (!isAnInt(etatDepartF.getText(), "Start state")) 
			return false;
		for (int j=0; j < distPanel.length; j++) {
			 for(int i=0; i <distPanel[j].getParams().length; i++) {
			 	if (!isAFloat(distPanel[j].getParams()[i], "Distribution parameters"))
			 		return false;
			 }
		 }
		 return true;
	}

    public org.w3c.dom.Element buildNewXmlNode() {
            // data values in the Panel should be ok
            // Access Configuration Tree.
        graphics2.ConfigurationXMLFile conftree = imp.getModelPanel().getClientPanel().getListScenario();
        org.w3c.dom.Element elem = conftree.createNewXMLElement("scenario_onoff");
        
            // Add the model.
        org.w3c.dom.Element modelchild = conftree.createElement("model");
        modelchild.setAttribute("name", "OnOff");
        elem.appendChild(modelchild);

            // Add state value node to the model
        org.w3c.dom.Element statechild = conftree.createElement("sstate");
        statechild.setAttribute("value", String.valueOf(getStartState()));
        modelchild.appendChild(statechild);

            // Add the two distributions...
        for (int j=0; j < distPanel.length; j++) {
            org.w3c.dom.Element dischild = conftree.createElement("distribution"+(j+1));
            dischild.setAttribute("name", getDistributionName(j));
            for(int i=0; i <distPanel[j].getParams().length; i++)
                dischild.setAttribute("param"+(i+1), distPanel[j].getParams()[i]);
            modelchild.appendChild(dischild);
        }
            // Return the new XML node
        return elem;
    }
	
	public void updateWithXmlNode(org.w3c.dom.Element elt) {
		if (! elt.getAttribute("name").equals("OnOff")) {
			// Error
			System.out.println("Wrong model name: "+elt.getAttribute("name")+" in OnOffModelPanel");	
		}
		else {
			/* @param idx 0: alpha, 1:beta, 2:lambda1 3:lambda2
			* @param value
			* @throws IllegalArgumentException
			*/
			//reset();
			org.w3c.dom.NodeList nlist = elt.getChildNodes() ;
			for (int i=0; i < nlist.getLength(); i++) {
				String name = nlist.item(i).getNodeName();
				if (name.equals("sstate"))
					setStartState(Integer.valueOf( ((org.w3c.dom.Element) nlist.item(i)).getAttribute("value")).intValue() );
				else if(name.equals("distribution1")) {
					setDistributionName(0, ((org.w3c.dom.Element) nlist.item(i)).getAttribute("name"));
					String[] params;
					if (((org.w3c.dom.Element) nlist.item(i)).getAttribute("param2").equals("")) {
						params = new String[1];
					}
					else {
						params = new String[2];
						params[1]=((org.w3c.dom.Element) nlist.item(i)).getAttribute("param2");
					}
					params[0]=((org.w3c.dom.Element) nlist.item(i)).getAttribute("param1");
					setDistributionParameters(0, params);
				}
				else if(name.equals("distribution2")) {
					setDistributionName(1, ((org.w3c.dom.Element) nlist.item(i)).getAttribute("name"));
					String[] params;
					if (((org.w3c.dom.Element) nlist.item(i)).getAttribute("param2").equals("")) {
						params = new String[1];
					}
					else {
						params = new String[2];
						params[1]=((org.w3c.dom.Element) nlist.item(i)).getAttribute("param2");
					}
					params[0]=((org.w3c.dom.Element) nlist.item(i)).getAttribute("param1");
					setDistributionParameters(1, params);
				}
			}
		}

	}

}
