/**
 * VideoDefinition : Utilise pour faire d ela generation de videos definies par un modelName M/G/Inf, un codec, un tps de simul
 * 						et un nom
 *
 * @author maria ladoue
 * @version 0.1, 06/11/02 
 */

package multimedia;

import java.util.*;

import utils.SpecificObjectCreation;
import multimedia.model.MGInfModel;
import multimedia.codec.*;


public class VideoDefinition implements VideoCodecExample,VideoExample {

    protected MGInfModel model;
    protected VideoCodec codec;
    protected int tpsSimul;
    protected String name;


	public VideoDefinition() {
		this.model = new MGInfModel();
		this.codec = MPEG;
		this.tpsSimul = 30;
		this.name = "None";
	}

	/**
	 * VideoDefinition 
	 * @param model an M/G/Infinity Model
	 * @param codec	a video codec
	 * @param tpsSimul	flow duration
	 * @param name	document name
	 */
    public VideoDefinition( MGInfModel model, VideoCodec codec, int tpsSimul, String name) {
		this.model =  model;
		this.codec = codec;
		this.tpsSimul = tpsSimul;
		this.name = name;
    }

    public MGInfModel getModel() { return model; }

    public VideoCodec getCodec() { return codec; }

    public int getTpsSimul() { return tpsSimul;  }

    public String getName() { return name; }
    
  // public String getFileName() { return }
    
    
    /**
     * Returns a simplified name:
     * 	Jurassic Park (Medium Quality) becomes JurassicParkMedium...
	 * @return a String
	 */
	public static String getSimplifiedName(String name) {
    	String ans = name;
		ans = SpecificObjectCreation.removeCharacters(ans, '(');
		ans = SpecificObjectCreation.removeCharacters(ans, ' ');
		ans = SpecificObjectCreation.removeCharacters(ans, ')');
		ans = ans.replaceAll("Quality","");
    	return ans;
    }
    
	/**
	 * Returns a complete name from simplified name
	 * 	 JurassicParkMedium becomes Jurassic Park (Medium Quality) 
	 * @return a String
	 */
	   public static String getCompleteName( String name ) {
		   String ans = "", tmp ="";
		   char[] tmpArray;
		   Vector idxUpperCase = new Vector(2);
		   
		   if( name.endsWith("Low")) {
		  	tmp += name.replaceAll("Low","");
		  	
			tmpArray =tmp.toCharArray();
		  	for (int i=0; i< tmpArray.length;i++ ) {
		  		if( ! Character.isLowerCase((char)(tmpArray[i])) ) {
					idxUpperCase.add( new Integer(i) );
		  		}
		  	}
		  	if ( idxUpperCase.size() < 2 ) {
		  		ans = tmp+" (Low Quality)" ;
		  	}
		  	else {
		  		for (int i=0;i<idxUpperCase.size()-1;i++) {
		  			ans += tmp.substring( ((Integer)idxUpperCase.elementAt(i)).intValue(), ((Integer)idxUpperCase.elementAt(i+1)).intValue() ) + " ";
		  		}
		  		ans += tmp.substring( ((Integer)idxUpperCase.elementAt(idxUpperCase.size()-1)).intValue()) + " (Low Quality)" ;
		  	}
		   }
		   else if(name.endsWith("Medium")) {
			tmp += name.replaceAll("Medium","");
			
			tmpArray =tmp.toCharArray();
				for (int i=0; i< tmpArray.length;i++ ) {
					if( ! Character.isLowerCase((char)(tmpArray[i])) ) {
						idxUpperCase.add( new Integer(i) );
					}
				}
				if ( idxUpperCase.size() < 2 ) {
					ans = tmp + " (Medium Quality)" ;
				}
				else {
					for (int i=0;i<idxUpperCase.size()-1;i++) {
						ans += tmp.substring( ((Integer)idxUpperCase.elementAt(i)).intValue(), ((Integer)idxUpperCase.elementAt(i+1)).intValue() ) + " ";
					}
					ans += tmp.substring( ((Integer)idxUpperCase.elementAt(idxUpperCase.size()-1)).intValue()) + " (Medium Quality)" ;
				}
		   }
			else if(name.endsWith("High")) {
				tmp += name.replaceAll("High","");
		  	
				tmpArray =tmp.toCharArray();
			for (int i=0; i< tmpArray.length;i++ ) {
				if( ! Character.isLowerCase((char)(tmpArray[i])) ) {
					idxUpperCase.add( new Integer(i) );
				}
			}
			if ( idxUpperCase.size() < 2 ) {
			ans = tmp+" (High Quality)" ;
		}
		else {
			for (int i=0;i<idxUpperCase.size()-1;i++) {
				ans += tmp.substring( ((Integer)idxUpperCase.elementAt(i)).intValue(), ((Integer)idxUpperCase.elementAt(i+1)).intValue() ) + " ";
			}
			ans += tmp.substring( ((Integer)idxUpperCase.elementAt(idxUpperCase.size()-1)).intValue()) + " (High Quality)" ;
		}
		 	}
		 	else { 
		 		throw new IllegalArgumentException( "We don't know this quality type: " +(name.split(" ",2)[1]) );
		 	}
		   return ans;
	   }
    
    
    /**
     *  Main methode : pour tester la classe
     */

    public static void main(String[] args) {
  
//	GenericDistribution dSlot = new ConstantDistribution(1d);
//	GenericDistribution dPacketSize = new ConstantDistribution(1000d);
//	double[] coeffFuncjpl = { 0.443, 0.557};
//	double[] coeffFuncjpm = {0.334, 0.566};
//	double[] coeffDisjpm = {0.041, 0.959};
//	GenericContinuousDistribution[] gdjpm = { new GammaDistribution(19.129,1499.9), new LogNormalDistribution(9.508,0.543) };
//	GenericFunction[] gftjpl = { new MarkovianFunction(0.02), new SRDFunction(0.333)};
//	GenericFunction[] gftjpm = {new MarkovianFunction(0.021),new SRDFunction(0.269)};
//	GenericFunction gfjpl = new MixedFunction(gftjpl ,coeffFuncjpl  );
//	GenericFunction gfjpm = new MixedFunction( gftjpm,coeffFuncjpm  );
//
//	GenericContinuousDistribution mdjpm = new MixedDistribution(gdjpm  , coeffDisjpm  );
//
//	
//	MGInfModel mginfjpl = new MGInfModel(gfjpl , new LogNormalDistribution(9.0002, 0.5127),dSlot , dPacketSize);
//	MGInfModel mginfjpm = new MGInfModel(gfjpm, mdjpm ,dSlot,dPacketSize);
//
//	VideoDefinition JurassicParkLow = new VideoDefinition ( mginfjpl ,MPEG4  , 60 , "Jurassic Park (Low Quality)");	
VideoDefinition JurassicParkMedium = new VideoDefinition( mginfjpm, MPEG4, 60, "Jurassic Park (Medium Quality)");

System.out.println( VideoDefinition.getCompleteName("JurassicParkSuperDenferHigh"));
System.out.println( VideoDefinition.getCompleteName("JurassicParkLow"));
System.out.println( VideoDefinition.getCompleteName("JurassicParkMedium"));
System.out.println( VideoDefinition.getCompleteName("BeanLow"));
System.out.println( VideoDefinition.getSimplifiedName("Jurassic Park Super Denfer (High Quality)"));
System.out.println( VideoDefinition.getSimplifiedName("Bean (Low Quality)"));
	//System.out.println( "Functions weight: " + SpecificObjectCreation.transformSTS(TestParameters.transformDTST(BeanLow.getModel().getAutoCorr().getWeight()))); 
    }
 
}
