/**
 * MGInfGeneration: Pour generer les paquets suivant le modele M/G/Infini
 *
 * @author maria ladoue
 * @version 1.0, 07/25/03 
 */

package multimedia.generation;

import java.io.*;
import java.util.*;

import multimedia.*;
import multimedia.codec.*;
import multimedia.model.*;
import statistic.*;
import statistic.distribution.*;
import utils.*;


public class MGInfGeneration extends GenericGeneration implements RandomGenerator, AudioCodecExample, VideoCodecExample, VideoExample {

    protected MGInfModel model;
    protected CodecAV codec;
    protected int N = 0;	// slots number
    protected double slotDuration = 0; // slot duration
    protected String documentName;
    
  
	
	 public MGInfGeneration() throws IOException {

		Calendar calendar = new GregorianCalendar();
	 	this.fileName = new String("./mginf" + calendar.get(Calendar.YEAR) + 
											(calendar.get(Calendar.MONTH) + 1) + 
											calendar.get(Calendar.DAY_OF_MONTH) + "_" + 
											calendar.get(Calendar.HOUR_OF_DAY) + "h" + 
											calendar.get(Calendar.MINUTE));
        unit = BYTES;
        graphic = false;
        model = new MGInfModel();
        tpsSimul = 30;
        documentName = "Unknown document";
//        codec= VideoCodecExample.MJPEG;
//		setSlotsNumber();
//		setSlotDuration();
	 }
	
    /**    
     *@param modelName	it's a M/G/Infinity Model
     *@param tpsSimul			test duration
     *@param codec			it's a codec
     *@param srcnumber	number of source: 1 by default
     *@throws IOException
     */
    public MGInfGeneration( MGInfModel model, int tpsSimul, CodecAV codec, int srcNumber) throws IOException {
      	this();      
        this.model = model;
        this.tpsSimul = tpsSimul;
        this.codec = codec;
        this.srcNumber = srcNumber;
   		
   		// Plus simple: mais pourrait etre supprime et dans ce cas la on supprimerait les variables internes: N et slotDuration
		setSlotsNumber();
		setSlotDuration();
    }

    /**
     * MGInfGeneration  Methode standard ou on definit tout
     *     
     *@param modelName	it's a M/G/Infinity Model
     *@param tpsSimul			test duration
     *@param fichierReq	scenario file name
     *@param codec	to specify a codec (video codec in general)
     *@param srcnumber
     *@param name	server name
     *@param unit	 you could choose between: BYTES, KILOBYTES, MEGABYTES, GIGABYTES
     *@param graphic To choose a graphical mode or not
     *@throws IOException
     */
    public MGInfGeneration(MGInfModel model, int tpsSimul, String fichierReq,  CodecAV codec, int srcNumber,  short unit, boolean graphic) throws IOException {
  	
        this( model, tpsSimul, codec, srcNumber);
        this.fileName = fichierReq;
        this.unit = unit;
        this.graphic = graphic;
    }

    /**
    * Definit afin de generer du trafic a partir d'un objet " video definition" comprenent modele, tpsSimul, codec...
    *     
    *@param vd
    *@param srcNumber
    *@param fichierReq	scenario file name
    *@param unit  you could choose between:  you could choose betwen: BYTES, KILOBYTES, MEGABYTES, GIGABYTES
    *@param graphic	To choose a graphical mode or not
    *@throws IOException
    */
    public MGInfGeneration(VideoDefinition vd, int srcNumber, String fileName, short unit, boolean graphic)  throws IOException {
        this( vd.getModel(), vd.getTpsSimul(), fileName,vd.getCodec(), srcNumber,  unit, graphic);
        documentName = vd.getName();
    }
 
      
	/**
	 * Create a file formated as follow:<BR>
     *    <i>date</i> : <i>serverName</i> requested document :  <BR>
     *    <i>modelName</i>    IAmoy : <i>tpsIP_codec</i> numero de port : <i>number</i> packet size :<i>size</i> bytes<BR>
     *    
     * 	date: t + <i>testTime</i> s,  IA : <i>interpacketIime</i> : 0.000000 s <BR>
     * 
	 * @param serverName gives the server name
     * @param serverPort gives the number port
     * @return throughput : WARNING to get the unit used getUnit method
	 * @see multimedia.generation.GenericGeneration#scenarioGenericGeneration(boolean, java.lang.String, int)
	 */
	protected double scenarioGenericGeneration( boolean playable) throws IOException {
		
			
			if(!playable) {
				// Premiere ligne du scenario: date + nom_du_serveur
		   		fichierReq.write( new Date().toString() + " :  \t requested document : " + documentName + "\n");
		   		fichierReq.write( model.getCompleteName() + "\t Test Duration : " + tpsSimul + " mn \n");
				fichierReq.write("Scenario format: date (in seconds) \t inter-packet time (in milliseconds) \t packet size (in Bytes)\n\n" );// Format du fichier scenario
			}
			
		//System.out.println("Slots: " + N); //DEBUG
			
		   int[] slotNew = dataPerSlotDistributionTransformation(model, N);
		   
		   
		   double time = 0; //en secondes
		   // Definit le nb de bytes deja envoyes
		   int nbBytesTotal = 0; // en bytes


			// Corresponds to distributionSlot and distributionPacketSize
		   String dist1Name = (((GenericDistribution)(model.getParameter(3))).getName());
		   String dist2Name = (((GenericDistribution)(model.getParameter(4))).getName());
         
			// Time inter-packet, correspond to the result of dureeSlot/counterPacket or distributionsSlot.rand() (if the distribution is defined)
			double IPTime = 0, meanIPTime =0;
         
		   // For each slots we compute the number of packets, their size and their temporal distribution
		   for (int l = 0; l <= N; l++) {

			 // correspond to KBytes (or Bytes, or MBytes....)
			 //slotNew[l]=slotNew[l]*1395; // dans le cas de RealPlayer
			 //slotNew[l]=(int)(((GammaDistribution)((GenericDistribution)(modelName.getParameter(2)).inverseCdf((new PoissonDistribution(((Double)(modelName.getParameter(1))).doubleValue())).cdf(slots[l]));
		   //System.out.println("Nombre de clients presents sur le slot no:" + l + " , Poisson : " + slots[l] + ", Tranform :"  + slotNew[l]+ " cdfPoisson : " + cdfPoisson + "\n");
            
			if ( nbBytesTotal >(Integer.MAX_VALUE-slotNew[l]) && unit==BYTES) {
			 unit = KILOBYTES;
			 nbBytesTotal = (int)nbBytesTotal/1000;  // on passe de B->kB
			 nbBytesTotal += (int)slotNew[l]/1000;	// on passe de B->kB
			}
			else if ( nbBytesTotal >(Integer.MAX_VALUE-slotNew[l]) && unit==KILOBYTES) {
			 unit = MEGABYTES;
			 nbBytesTotal = (int)nbBytesTotal/1000;  // on passe de kB->MB
			 nbBytesTotal += (int)slotNew[l]/1000000;	// on passe de B->MB
			}
			else if ( nbBytesTotal > (Integer.MAX_VALUE-slotNew[l]) && unit==MEGABYTES ) {
					unit = GIGABYTES;
					nbBytesTotal = (int)nbBytesTotal/1000;  // on passe de MB->GB
					nbBytesTotal += (int)slotNew[l]/1000000000;	// on passe de B->GB
		   }
		   else if ( unit==KILOBYTES || unit==MEGABYTES || unit==GIGABYTES) {
				 nbBytesTotal += (int)slotNew[l]/1000;
			}
			else {
				 nbBytesTotal += slotNew[l];
			 }
   	
   		
   	
			   // Definit le nb de bytes deja envoye sur le slot courant
			   int nbBytes = 0; // en bytes
			   // Compte le nombre de paquets deja creer pour le slot courant
			   int counterPacket = 0;
			   
			   double startTimeSlot = time;
			   // Packet size in Bytes: it could be a constant or a distributionsPacketSize.rand()
			   int packet = 0;
			   // slot last packet size in Bytes => uniquement dans le cas d'une taille de paquet fixe
			   int lastPacket = 0;
			   // definit la taille de chacun des paquets qui sera genere a l'interieur d'un slot.
			   Vector packetLength = new Vector(25);

			   if (slotNew[l] != 0) {

		   // Determine la taille des paquets a l'interieur d'un slot.
			// Si la taille des paquets est fixes
         // TODO: Modifier, ne pas faire la difference entre type de distribution... de toute facon, le code est le meme !
         // En plus, ca semble faux pour le cas mixed... puisque l'on ne regarde que la premiere distribution...
				   if ( dist1Name.compareTo("Constant") == 0 ||
				   	( dist1Name.compareTo("Mixed") == 0 && ( ((MixedDistribution)(model.getParameter(3))).getSpecificDistribution(0).getName().compareTo("Constant") == 0) ) ) {
                	
						if ( dist1Name.compareTo("Constant") == 0 ) {
						   packet = (int)((ConstantDistribution)(model.getParameter(4))).getParam(0);
						   
						}
						else {
						   packet = (int)((ConstantDistribution)((MixedDistribution)(model.getParameter(4))).getSpecificDistribution(0)).getParam(0);
					   	}

					   while (true) {
						   if ((packet + nbBytes) <= slotNew[l]) {
							   counterPacket++;
							   nbBytes += packet;
						   } 
						   else {
							   lastPacket = slotNew[l] - nbBytes;
							   counterPacket++;
							   nbBytes += lastPacket; // Et devrait etre egal a slotNew[l];
							   break;
						   }
					   }
				   } 
				   else {
					   while (true) {
						   // Determine la distribution de la taille des paquets a l'interieur d'un slot.
						   packet = (int)((GenericDistribution)(model.getParameter(4))).rand();
						  

						   // Determine la taille du paquet
						   if ((packet + nbBytes) <= slotNew[l]) {
							   packetLength.add(counterPacket, new Integer(packet));
							   counterPacket++;
							   nbBytes += packet;
						   } 
						   else {
							   packet = slotNew[l] - nbBytes;
							   packetLength.add(counterPacket, new Integer(packet));
							   counterPacket++;
							   //nbBytes += packet;
							   nbBytes = slotNew[l];
							   break;
						   }
					   }
				   }
                
//	  Determine la distribution des paquets a l'interieur du slot ainsi que leur taille
//	 System.out.println("Donnees: " + slotNew[l] + " Nb de paquets: " + counterPacket + " duree Slot: " + dureeSlot + " time: " + startTimeSlot);

	//TODO: A modifier egalement, complexe a souhait...
	// Cependant, le calcul de IPTime se revele different entre une distro Constant et une autre...
   if (
	( dist1Name.compareTo("Constant") == 0 ||  ( dist1Name.compareTo("Mixed") == 0 && ( ((MixedDistribution)(model.getParameter(3))).getSpecificDistribution(0).getName().compareTo("Constant") == 0) ) ) &&
	( dist2Name.compareTo("Constant") == 0 ||  ( dist2Name.compareTo("Mixed") == 0 && ( ((MixedDistribution)(model.getParameter(4))). getSpecificDistribution(0).getName().compareTo("Constant") == 0) ))
	 )  {
		
				// In the slot l : distribution of packets
				 IPTime = slotDuration / counterPacket;
				//System.out.println(	"Packet: " + counterPacket);	//DEBUG
				for ( int i = 0; i < (counterPacket - 1); i++) {
						
						if( playable ) {
							fichierReq.write( (IPTime*1000) + "\t" + packet + "\n");
							//System.out.println("Bytes: " + packet); 	//DEBUG
						}
    				    else {
							fichierReq.write( "date : t + " + MyNumberFormat.myTimeFormatter.format(time) + " s \t IA: " + 
							MyNumberFormat.myIAFormatter.format(IPTime * 1000) + " ms \t size: " + MyNumberFormat.mySizeFormatter.format(packet) + 
							" Bytes \t rate: " + 
							MyNumberFormat.myDebitFormatter.format( (((counterPacket - 1) * packet + lastPacket) * 8 / slotDuration / 1000)) + " kb/s \n");
						}
						
					   time += IPTime;
			   	}
			   	
				if( playable ) {
					fichierReq.write( (IPTime*1000) + "\t" + lastPacket + "\n"); 
				}
				else {   
				   fichierReq.write("date : t + " + MyNumberFormat.myTimeFormatter.format(time) + " s \t IA: " + MyNumberFormat.myIAFormatter.format(IPTime * 1000) + 
					   " ms \t size: " + MyNumberFormat.mySizeFormatter.format(lastPacket) + " Bytes \t rate: " + 
					MyNumberFormat.myDebitFormatter.format( ((counterPacket - 1) * packet + lastPacket) * 8 / slotDuration / 1000) + " kb/s \n");
				}
				   time = startTimeSlot + slotDuration;
	 } 
                
	   else if ( dist2Name.compareTo("Constant") == 0 ||
			   (  dist2Name.compareTo("Mixed") == 0 && ( ((MixedDistribution)(model.getParameter(4))). getSpecificDistribution(0).getName().compareTo("Constant") == 0) )) {

					   // In the slot l : distribution of packets
					   IPTime = slotDuration / counterPacket;

					   for (int i = 0; i < counterPacket; i++) {
					   	
						if( playable ) {
							fichierReq.write( (IPTime*1000) + "\t" + packet + "\n");
						}
						else {   					   	
						   fichierReq.write("date : t + " + MyNumberFormat.myTimeFormatter.format(time) + " s \t IA: " +	MyNumberFormat.myIAFormatter.format(IPTime * 1000) + 
							 " ms \t size: " + MyNumberFormat.mySizeFormatter.format(packet) + " Bytes \t rate: ");
						   // + myDebitFormatter.format((((counterPacket-1)*packet + lastPacket)*8/dureeSlot/1000))  + " kb/s \n");
						}
						   time += IPTime;
					   }
		}
		  
		else {
		   // In the slot l : distribution of packets
			   for (int i = 0; i < counterPacket; i++) {
				   IPTime = ((GenericDistribution)(model.getParameter(3))).rand();

						   // Determine le temps inter-paquet en s
						   if ((time + IPTime) <= (startTimeSlot + slotDuration)) {
								if( playable ) {
									fichierReq.write( (IPTime*1000) + "\t" + packetLength.elementAt(i) + "\n");
								}
								else {
								   fichierReq.write( "date : t + " + MyNumberFormat.myTimeFormatter.format(time) + " s \t IA: " + MyNumberFormat.myIAFormatter.format(IPTime * 1000) + " ms \t size : " + 
								   MyNumberFormat.mySizeFormatter.format( packetLength.elementAt(i)) + " Bytes \t rate: " + 
								   MyNumberFormat.myDebitFormatter.format( ((counterPacket - 1) * packet + lastPacket) * 8 / slotDuration / 1000) + " kb/s \n");
								}  
							   time += IPTime;
						   } 
						   else {
							   for (int j = i; j < counterPacket; j++) {
								   nbBytes -= ((Integer)(packetLength.elementAt(j))).intValue();
							   }
							   break;
						   }
					   }
					   time = startTimeSlot + slotDuration;
				   } // end of else
			   } //end of if ( slotNew[l] != 0 ) {
		   } // fin de la boucle for

		fichierReq.flush();
		fichierReq.close();

		   double debit = nbBytesTotal / time;
		   // Compute the theoritical throughput
		   double debitTheo = ((GenericContinuousDistribution)(model.getParameter(2))).mean() / slotDuration * 8 / 1000;
		   //System.out.println(nbBytesTotal);
        
			// Dans tous les cas
		   String messageInfo ="";        
		   if (unit==BYTES) {
			   messageInfo =  new String("Taille du document transmit: " + nbBytesTotal / 1000000 + " MBytes.\n"
			   + "Le debit applicatif moyen est de: " +  MyNumberFormat.myDebitFormatter.format(debit / 1000) + " kB/s, soit : " + MyNumberFormat.myDebitFormatter.format(debit * 8 / 1000) + " kb/s.\n"
			   + "Debit applicatif theorique: " +  MyNumberFormat.myDebitFormatter.format( debitTheo ) + " kb/s.");
						}
		   else if(unit==KILOBYTES) {
			   messageInfo =  new String("Taille du document transmit: " + nbBytesTotal / 1000 + " MBytes.\n"
			   + "Le debit applicatif moyen est de: " +  MyNumberFormat.myDebitFormatter.format( debit ) + " kB/s, soit : " + MyNumberFormat.myDebitFormatter.format(debit * 8 / 1000) + " Mb/s.\n"
			   + "Debit applicatif theorique: " +  MyNumberFormat.myDebitFormatter.format( ((GenericContinuousDistribution)(model.getParameter(2))).mean() / slotDuration * 8 / 1000) + " Mb/s.");	
		   }    	
		   else { messageInfo = new String("No information available"); }
        
		 MessageManager.informationMessage(messageInfo,graphic);
	
        
		MessageManager.informationMessage("File created as: " + resultFile.getCanonicalPath(), graphic);
		
		 return debit;        
	}
	
	
	public void setSlotsNumber() { this.N = this.slotsNumberComputation(); }
	
	/**
	 *  To compute the number of slots which defines the time evolution
	 *  Number of slots: => N = test duration / T
	 *  with T = Number of frame between two I frames /  number of frames in one second
	 * For example, for a MPEG4 film of one hour, 3600/(12/25)=7500 slots
	 * @return the number of slots 
	 */
	protected  int slotsNumberComputation() {
		return tpsSimul*60* (((VideoCodec)codec).getFramesNumber()) / (((VideoCodec)codec).getCompression().getGOP().getLength());
	}


	/**
	 * To compute the data amount per slot with a poisson process
	 * @param model
	 * @param N => slot number
	 * @return an array of int => size : [ N ]
	 */
	public static int[] dataPerSlotDistributionComputation( MGInfModel model, int N) {
		/**
		* Methode utilisant des arbres tries
		* Chaque noeud est compose d'une cle  et d'une valeur associee (la cle sert a distinguer les doubles)
		* Attention si 2 noeuds ont la meme cle, on en elimine 1
		*/
	   //Processus d'occupation de la file M/G/infini
	   // Nombre de clients "actif" a un instant donne.
	   int[] slots = new int[N + 1];
	   // L'arbre est utilise pour stocker le nombre de clients "mourrant" a un instant donne.
	   SortedMap ts1 = new TreeMap();
	   int deadTime, sigmaIJ, sigmaXo, A, poids;
	   int alive = 0, dead = 0;

	   // Nombre de clients presents dans le systeme a To
	   slots[0] = model.calculNbClients(0);
	   alive = slots[0]; // Nombre de clients vivants a t=0

	   if (slots[0] != 0) {

		   for (int j = 1; j <= slots[0]; j++) {
			   // Temps de service en slots de chacun des Xo (=j) clients presents a To (suivant Loi Chapeau Sigma)
			   sigmaXo = (int)(model.calculT(MGInfModel.CHAPEAUSIGMA));

				   if (sigmaXo > 1) {
					   // Temps de fin de vie de chacun des j clients cree a To
					   deadTime = sigmaXo - 1;

					   if (ts1.containsKey(new Integer(deadTime))) {
						   poids = ((Integer)(ts1.get(new Integer(deadTime)))).intValue();
						   ts1.put(new Integer(deadTime), new Integer(poids + 1));
					   } 
					   else {
						   ts1.put(new Integer(deadTime), new Integer(1));
					   }
				   } 
				   else if (sigmaXo == 1) { dead++;  } // Nombre de clients qui meurent a t=0
			   }
		   } //end if (slots[0] != 0) {

		   for (int i = 1; i <= N; i++) {
			   A = model.calculNbClients(1);

			   // Generation de nouveaux clients au slot i
			   alive += A;
			   slots[i] = alive - dead;

			   if (A != 0) {
				   for (int j = 1; j <= A; j++) {
				   // Generation du temps de service (en slots) du client j present sur le slot i
				   sigmaIJ = (int)(model.calculT(MGInfModel.SIGMA));
				   deadTime = sigmaIJ + i - 1;

					   if (ts1.containsKey(new Integer(deadTime))) {
						   poids = ((Integer)(ts1.get(new Integer(deadTime)))).intValue();
						   ts1.put(new Integer(deadTime), new Integer(poids + 1));
				   		} 
				  	 	else {
						   	ts1.put(new Integer(deadTime), new Integer(1));
					   	}
			  	 	}
			   } // end  if (A != 0 ) {

			  // Pour supprimer les clients correspondant au slot courant (dont on n'a plus besoin)
			  if (ts1.containsKey(new Integer(i))) {
				   int newDead = ((Integer)(ts1.get(new Integer(i)))).intValue();
				   ts1.remove(ts1.firstKey());
				   dead += newDead;
			   }
		   } //  end for (int i=1;i<=N;i++) {
		
	   return slots;
	}

/**
 * Method dataPerSlotDistributionTransformation
 * @param model
 * @param slots
 * @param N
 * @return
 */
public static int[] dataPerSlotDistributionTransformation(MGInfModel model, int N) {
	int[] slotNew = new int[N + 1];
	int[] slots = dataPerSlotDistributionComputation(model,N);
	/**
	 *  We want Y like : Y=F<sub>dist2</sub><sup>-1</sup>(F<sub>Poiss</sub>(Xi))<BR>
	 *      where Xi =  nb of Kbytes generate on slot i<BR>
	 *      where F<sub>Law2</sub><sup>-1</sup> is the inverse of distribution function for the dist2<BR>
	 *      where F<sub>Poiss</sub> is the distribution function for a Poisson distribution<BR>
	 */

	 for (int l = 0; l <= N; l++) {
		  // Transforme une distribution poissonienne (qui repond au parametre d'autocorrelation) en une autre distribution.
		  //double cdfPoisson = (new PoissonDistribution(((GenericDiscreteDistribution)(model.getParameter(5))).mean())).cdf( slots[l]);
		 double cdfPoisson = ( new PoissonDistribution( ((SigmaDistribution)(model.getParameter(5))).mean()) ).cdf( slots[l] );
		 GenericContinuousDistribution distributionTest = (GenericContinuousDistribution)(model.getParameter(2));  // correspond to distribution which gives data distribution in a slot

		  // fournit le nb de KBytes ou de Bytes... qu'il faut generer pdt le slot
		  slotNew[l] = (int)(distributionTest .inverseCdf(cdfPoisson));
		  
		// ajouter le 04/04/2003 pour palier a l'apparition de valeurs nulles => en remplace(arbitrairement ) valeur nulle par la valeur moyenne
		//if (slotNew[l] == 0) { slotNew[l] = Math.round((float)distributionTest.mean()); }
		  
		// ajouter le 07/04/03 pour palier a des valeurs nulles:
		if (slotNew[l] == 0) { slotNew[l] = (int)(distributionTest.inverseCdf(cdfPoisson)); }		
	}
	return slotNew;
}

  

  public void setSlotDuration() { this.slotDuration = this.slotDurationComputation(); }


	/**
	 * Method getSlotDuration : to compute slot duration 
	 * @return slot duration in seconds 
	 */
	protected double slotDurationComputation() {
		 return (double)tpsSimul / (double)N * 60; // en secondes
	 }

public void setMGInfModel(MGInfModel mginfModel) { this.model = mginfModel; }
public void setCodecAV(CodecAV codec) { this.codec = codec; }
public void setMovieName(String movieName) { this.documentName = movieName; }

    /**
     * Method getName : return the name/type of the transmit document
     */
    public String getDocumentName() {  return documentName; }

    /**
     * Method getCodec : return the codec 
     */
    public CodecAV getCodec() { return codec;  }
    

    /**
     * Methode: Main pour tester la classe
     */
    public static void main(String[] args) throws IOException {

       	 String f1 = new String("MGInfGene");
       	 String f2 = new String("JPLow1mn");
         String f4 = new  String("MGInfJPMedium_0404032");
         //String f5 = new  String("MGInfBLow");
         
         //String f6 = new  String("MGInfBMedium");
         //String f7 = new  String("MGInfBHigh");
		
		MGInfModel mgit = new MGInfModel(new statistic.function.SRDFunction(0.126), 1d, new LogNormalDistribution(8.74,0.4274), new ConstantDistribution(1.0), new ConstantDistribution(1000.0));

		MGInfGeneration mgig1 = new MGInfGeneration( mgit,30,"fileout",MPEG4,3,(short)0,false);
		mgig1.scenarioGeneration();
		/**
        MGInfModel mgi1 = new MGInfModel(new SRDFunction(0.04),1d,new GammaDistribution(5d,1d/10d),new ExponentialDistribution(0.2d),new ExponentialDistribution(0.3d));
        //MGInfGeneration mgig1 = new MGInfGeneration( mgi1,40,f1,new VideoCodec(30,new CompressionType(new GOP(1,1))));
        MGInfGeneration mgig1 = new MGInfGeneration( mgi1,30,f1,MPEG4,new String("indefinite"),(short)0,false);
        System.out.println("Debut: " + new Date().toString());
        mgig1.genererScenario("fripotin",5000);
		System.out.println("Fin: " + new Date().toString());
        */
//		
//		MGInfModel mgi2 = new MGInfModel(new SRDFunction(0.08),2d,new GammaDistribution(5d,1d/10d),new ExponentialDistribution(0.2d),new ExponentialDistribution(0.3d));
//		
//		//	public MGInfGeneration(MGInfModel model, int tpsSimul, String fichierReq,  CodecAV codec, int srcNumber,  short unit, boolean graphic) throws IOException {
//		MGInfGeneration mgig2 = new MGInfGeneration( mgi2,30,f1,MPEG4,1,(short)0,false);
//		System.out.println("Debut: " + new Date().toString());
//		mgig2.scenarioGeneration();
//		System.out.println("Fin: " + new Date().toString());
         
//        GenericDistribution dSlot = new ConstantDistribution(1d);
//        GenericDistribution dPacketSize = new ConstantDistribution(1000d);
//        double[] coeffFuncjpl = { 0.443, 0.557 };
//        GenericFunction[] gftjpl = { new MarkovianFunction(0.02), new SRDFunction(0.333)};
//        GenericFunction gfjpl = new MixedFunction(gftjpl ,coeffFuncjpl  );
//        MGInfModel mginfjpl = new MGInfModel(gfjpl , new LogNormalDistribution(9.0002, 0.5127),dSlot , dPacketSize);
        
        //VideoDefinition JurassicParkLow = new VideoDefinition ( mginfjpl , MPEG4, 1 , "Jurassic Park (Low Quality)");    
        //MGInfGeneration mgig2 = new MGInfGeneration(JurassicParkLow,1, f2,BYTES,false);
        //mgig2.genererVideoScenario("fripotin",1260);
       	//mgig2.scenarioGeneration();
       // MyNumberFormat.timeSizeFormatage(f2, new String(f2+"format"));
        
//		MGInfGeneration mgig4 = new MGInfGeneration( BeanLow,1,"BeanLow60mn", BYTES, false);
//		MGInfGeneration mgig2 = new MGInfGeneration( JurassicParkHigh,1,"JurassicParkHigh60mn",BYTES, false);
//       MGInfGeneration mgig3 = new MGInfGeneration( BeanMedium,1,"BeanMedium60mn",BYTES, false);
	
		
		//mgig4.statisticsGeneration();
//		mgig4.scenarioGeneration();
		//MyNumberFormat.timeSizeFormatage("scenario.tmp", new String( "BeanLow60mn"+"format"));
//		mgig3.scenarioGeneration();
		//MyNumberFormat.timeSizeFormatage("scenario.tmp", new String( "BeanMedium60mn"+"format"));
//		mgig2.scenarioGeneration();
		//MyNumberFormat.timeSizeFormatage("scenario.tmp", new String( "JurassicParkHigh60mn"+"format"));
		//int[] slots2 = dataPerSlotDistributionTransformation(JurassicParkLow.getModel(),mgig2.slotsNumberComputation());
		//int[] slots3 = dataPerSlotDistributionTransformation(BeanMedium.getModel(),mgig3.slotsNumberComputation());
		//int[] slots4 = dataPerSlotDistributionTransformation(BeanLow.getModel(),mgig4.slotsNumberComputation());
		
		
		//System.out.println(" [ " +slots2[0]  + " , " + slots2[50] + " ] "  + " ; " + " [ " +slots3[0]  + " , " + slots3[1] + " ] " + mgig3.slotsNumberComputation() + " ; "+ " [ " +slots4[0]  + " , " + slots4[1] + " ] "+ mgig4.slotsNumberComputation() + " ; ");
        //mgig3.scenarioGeneration("fripotin",1260);
        
		//mgig4.scenarioGeneration("fripotin",1260);
		//mgig4.videoScenarioGeneration("fripotin",1260);
		

        
        //MyNumberFormat.timeSizeFormatage("MGInfJPHigh_060203","MGInfJPHigh_060203Format");
       // System.out.println("Debut: " + new Date().toString());
        //MGInfGeneration mgig3 = new MGInfGeneration(JurassicParkLow, "JPLow",BYTES);
        //mgig3.genererVideoScenario("fripotin",1260);
       
        //mgig3.scenarioGeneration();
       //MyNumberFormat.timeSizeFormatage("BeanMedium","BeanMediumFormat" );
        //System.out.println(mgig3.getUnite()); 
        /**
       MyNumberFormat.timeSizeFormatage(mgig3.getResultFileName(), 
                                          "JPLowFormat");
        if (!FileLineCounter.sameLineNumber(mgig3.getResultFileName(), 
                                            "JPLowFormat")) {
            System.out.println(
                    "Warning: there was a problem with file formatting.");
        }
        System.out.println("Fin: " + new Date().toString());
        */
       /** System.out.println("Debut: " + new Date().toString());
       VideoCodec MPEG4 = new VideoCodec(25,new CompressionType( new GOP(12,2)),"MPEG4" );
       //VideoDefinition[] vd =  { JurassicParkLow,BeanLow};
        //double[] weight ={2d,3d} ;  
        VideoDefinition[] vd =  { JurassicParkLow};
        double[] weight ={2d} ;  
        
        MGInfAggregationModel mginfa = new MGInfAggregationModel(vd , weight);
        MGInfGeneration mgig4 = new MGInfGeneration( mginfa, 60, "result", MPEG4,BYTES);
        mgig4.genererVideoScenario("fripotin",1260);
        System.out.println("Fin: " + new Date().toString());
        */
    }
}