/**
 * MMPP2Generation : Pour creer un scenario de generation de paquets suivant le modele MMPP-2
 *                   Remarque: cela marche mieux si on genere un paquet des qu'on entre dans un nouvel etat
 *
 * @author maria ladoue
 * @version 0 , 25/09/02 
 */

package multimedia.generation;

import java.io.*;
import java.util.*;  // pour Date

import statistic.distribution.*;	
import multimedia.codec.*;
import multimedia.model.*;


public class MMPP2Generation extends GenericGeneration implements AudioCodecExample { 

    protected MMPP2Model model;
    
     public MMPP2Generation () {
     	graphic = false;
     }

	/**
	 * Method MMPP2Generation.
	 * @param model
	 * @param tpsSimul  it is the test duration in minutes
	 * @param srcNumber
	 * @param fichierReq  it's a file where results will be enregistrated
	 * @param graphic
	 * @throws IOException
	 */
    public MMPP2Generation (MMPP2Model model, int tpsSimul, int srcNumber, String fichierReq, boolean graphic) throws IOException {	
		this.model = model;
		this.tpsSimul = tpsSimul;
		this.srcNumber = srcNumber;
		this.fileName = fichierReq;
		this.graphic = graphic;
    }

 
   
	/**
	 * File format:<br>
	 * 	-playable:<br>
	 * 		<i>IPTime in ms<\i><br>
	 * 	-not playable:<br>
	 *			<i>date</i>
	 * 
	 * @see multimedia.generation.GenericGeneration#scenarioGenericGeneration(boolean, java.lang.String, int)
	 */
	protected double scenarioGenericGeneration( boolean playable) throws IOException {
	
		// Definition des variables
		double time = 0, s;
		int newTpsSimul = tpsSimul*60;
		int packetSize = model.codec.getSize();
		int etat = model.getEtatDepart();
		double debitIns = 0;
		int nbPacket1 = 0, nbPacket2 = 0, nbPeriod = 0, nboctet = 0, cpt =0;
		double T1 = 0, T2 = 0, tpsIP1 = 0, tpsIP2 = 0;
		double r, sOld = 0d;	
	
	
		if(!playable) {           
			// Premiere ligne du scenario: date + nom_du_serveur
			fichierReq.write ( new Date().toString() + "\n");
			fichierReq.write( model.getName() + "\t Test Duration : " + tpsSimul  +  " mn \n");
			fichierReq.write("Scenario format: date (in seconds) \t Inter-Packet Time(in milliseconds) \t packet size(in Bytes) \n\n" );// Format du fichier scenario
		}


	while ( time < newTpsSimul ) {
	    if (etat == 0) {
				s = time;
				T1 = model.calculT(model.getParam(0,"beta"));	// Duree de l'etat 1
	
			if( T1 != 0d ) {
				
				if( playable )	{
					fichierReq.write( (s - sOld)*1000 + "\t" + packetSize + "\n" );
				}
				else { fichierReq.write("date: t + " + s + " s \t IA: " + ( s - sOld)*1000 + " ms \t" + packetSize+ " Bytes \n");	}
				nbPacket1++;
			
				while ( s < Math.min( time + T1 , newTpsSimul) ) {
				    
				    // Temps inter-paquet pour etat 1
				    tpsIP1 = model.calculT(model.getParam(0,"lambda"));
				   //TODO TJ: Cet enchainement de tests peut se reveler problematique si:
				   // - tpsIP1 est toujours 0 (c'est le cas si on utilise le constructeur qui met lambda a l'infini.
				   // - si tpsIP1 est plus grand que T1 (mais est-ce que cela peut se produire ???
				    if ( tpsIP1 != 0d ) {
						if ( (tpsIP1+s) < (time+T1) ) {
							if( playable )	{
								fichierReq.write( ( (tpsIP1 + (s-sOld))*1000) + "\t" + packetSize + "\n");
							}
							else {   
					    		fichierReq.write("date: t + " + (s+tpsIP1) + " s \t IA: " + (tpsIP1+(s-sOld))*1000 + " ms \t" + packetSize+ " Bytes \n");
							 }
					    	nbPacket1++; // Pour calculer le debit moyen...
					    	s += tpsIP1;
					    	sOld = s;
						}
						else { break;  }
				    }
				}
			}
			etat = 1;
			time += T1;	
	    } // fin du if

	    
	    else {
			r = time;
			// Duree de l'etat 2
			T2 = model.calculT( model.getParam(1,"alpha") );
			if( T2 != 0d ) {
				if( playable )	{
					fichierReq.write( ( r - sOld)*1000 + "\t" + packetSize + "\n");
				}
				else {
					fichierReq.write("date: t + " + r + " s \t IA: " + ( r - sOld)*1000 + " ms \t" + packetSize+ " Bytes \n");
				}
				nbPacket2++;
				
				while ( r < Math.min( time + T2 , newTpsSimul) ) {
				    
				    // Temps inter-paquet pour etat 2
				    tpsIP2 = model.calculT( model.getParam(1,"lambda") );
				    if ( tpsIP2 != 0d ) {
						if ( (tpsIP2+r) < (time+T2) ) { 
							if( playable )	{
								fichierReq.write( (tpsIP2+(r-sOld))*1000 + "\t" + packetSize + "\n");
							}
							else {
						    	fichierReq.write("date: t + " + (r+tpsIP2) + " s \t IA: " + (tpsIP2+(r-sOld))*1000 + " ms \t" + packetSize+ " Bytes \n");
							}
						    nbPacket2++; // Pour calculer le debit moyen...
						    r += tpsIP2;
						    sOld =  r;
						}
						else { break;  }
				    } // fin du while
				}
			}
			etat = 0;
			time += T2;	
	    } // fin du else
	
	
	    if (cpt != 1 ) { cpt++;   }
	    
	    else {
			nbPeriod++;
			// Calcul du debit moyen: debitmoy = ((n-1)* debitmoy(n-1) + debitIns(n))/n
			debitIns = packetSize*nbPacket2/(T1 + T2) ; // Donner le debit instantane pour une periode: T1 + T2
			nboctet += (nbPacket2+nbPacket1)*packetSize;
			nbPacket1 = 0;
			nbPacket2 = 0;
			cpt = 0;
	    }// fin du else
	} // fin du while

	fichierReq.flush();
	fichierReq.close();

	double debit =  nboctet/time;
	
	System.out.println("Le debit moyen est de: " +  debit/1000  + " kB/s, soit : " + debit*8/1024  + " kb/s.\n");
	System.out.println("Debit theorique: " + Math.round(model.meanRate()) + " kb/s");
	//System.out.println("Le debit moyen est de: " + (nboctet/tpsSimul/1000)  + " kB/s, soit : " + nboctet/tpsSimul*8/1024  + " kb/s" );
	//System.out.println("IA moy: " + time/nboctet*(modelName.getCodec()).getTaille() );
		return debit;
    }// fin scenarioGenericScenario(palyable,"fripotin",1260)


public void setMMPP2Model(MMPP2Model mmpp2m) { model = mmpp2m; }


/**
 * Methode: Main pour tester la classe
 */

    public static void main (String[] args) throws IOException {
    		// First test
    	MMPP2Model ipp1 = new MMPP2Model(1,2.8409091,1.5384615, 1.123, 1/0.02,new AudioCodec(136,0.02));
    	MMPP2Generation g1 = new MMPP2Generation(ipp1,60,1, "MMPP2Gene1", false);
		System.out.println(new Date().toString());
	   	g1.scenarioGeneration();
		g1.formatOutputFile("scenario.tmp");
		System.out.println(new Date().toString());

			// Second test, get a MMPP2 model from an aggregated onoff
		OnOffModel m1 = new OnOffModel(1,new ExponentialDistribution(2.8571429),new ExponentialDistribution(1.5384615),G711);
		OnOffHomogeneousAggregatedFlowModel cm1 = new OnOffHomogeneousAggregatedFlowModel(10,m1);
		MMPP2Model model = cm1.approximation();

		System.out.println("alpha " + model.getParameter(2));
		System.out.println("beta " + model.getParameter(3));
		System.out.println("lambda2 " + model.getParameter(4));
		System.out.println("lambda1 " + model.getParam(0,"lambda"));

		MMPP2Generation g2 = new MMPP2Generation(model,20,1,"MMPP2Gene2",false);

		System.out.println(new Date().toString());
		g2.scenarioGeneration();
		g2.formatOutputFile("scenario.tmp");
		System.out.println(new Date().toString());
    }
}
