package multimedia.generation;

import java.io.*;
import java.util.StringTokenizer;

import utils.*;


/**
 * ScenarioFileAnalyzer :
 * 
 * 
 * @author mladoue
 * @version 0.1, 06/06/03
 */


public class ScenarioFileAnalyzer {

	protected String scenarioFileName;
	protected boolean graphic;


	public ScenarioFileAnalyzer( String scenarioFileName ) throws FileNotFoundException, IllegalArgumentException {
		if ( FileTest.isExistent(scenarioFileName)) {
			if ( scenarioFileName.endsWith(".scf")) {
				this.scenarioFileName = scenarioFileName;
				this.graphic = false;
			}
			else { throw new IllegalArgumentException( scenarioFileName + " is not a valid scenario file."); }
		}
		else { throw new FileNotFoundException(scenarioFileName + " doesn't exist!");  }	 
	}
	
	public ScenarioFileAnalyzer( String scenarioFileName, boolean graphic ) throws FileNotFoundException, IllegalArgumentException {
		this(scenarioFileName);
		this.graphic = graphic;
	}
				

	/**
	 * This permits to know if a file could be played by the system or not.
	 * scenario file format:
	 * 	millis		nanos	[octect]
	 * @param IPTime in ms
	 * @param acceptancePercentage in %
	 * @return	true if the scenario respects conditions
	 * @throws FileNotFoundException
	 * @throws IllegalArgumentException
	 */
	public static boolean isValidScenario( String scenarioFileName, double IPTime, int acceptancePercentage, boolean graphic ) throws FileNotFoundException, IllegalArgumentException {
		
		if (acceptancePercentage>=0 && acceptancePercentage<=100) {

			FileReader fr = new FileReader(scenarioFileName);
			BufferedReader br = new BufferedReader(fr);
			StringTokenizer st;
			String c= "";
			int k=0, kRef, s1;
		
			try {
				while ((c = br.readLine()) != null) {
					st = new StringTokenizer(c);	
					s1 = Integer.parseInt( st.nextToken() );
					if ( s1 < (int)Math.round( IPTime ) ) { k++; }	// si on est inferieur IPTime => pb et donc le pourcentage d'erreur augmente
				} 
			}
			catch( IOException ioe ) { ioe.printStackTrace(); }
		
			kRef = FileLineCounter.countLines(scenarioFileName);
			if( (int)Math.round( ((double)k/(double)kRef)*100 ) < acceptancePercentage ) {
				MessageManager.informationMessage( " The scenario is valid  (error: " + MyNumberFormat.myPercentFormatter.format( ((double)k/(double)kRef)*100) +"%).", graphic);
				return true;
			}
			else { 
				MessageManager.informationMessage( " This scenario is not valid: \n error percentage is: " + MyNumberFormat.myPercentFormatter.format( ((double)k/(double)kRef)*100 ) + " %.", graphic);
				return false;
			}
		}
		else { throw new IllegalArgumentException("ScenarioFileAnalyzer.isValidScenario: The acceptance percentage has to be express in % => between 0 and 100 " + acceptancePercentage); }
	}

	/**
	 * IPTime<sub>avg<sub><i>N</i></sub></sub> = [ (N-1)*IPTime<sub>avg<sub><i>N-1</i></sub></sub> +IPTime<sub>N<sub>] / N <BR>
	 * @param scenarioFileName
	 * @param graphic
	 * @throws FileNotFoundException
	 * @throws IllegalArgumentException
	 * void
	 */
	public static void getMeanIPTime( String scenarioFileName, boolean graphic ) throws FileNotFoundException, IllegalArgumentException {
		
			FileReader fr = new FileReader(scenarioFileName);
			BufferedReader br = new BufferedReader(fr);
			StringTokenizer st;
			String c= "";
			int k=0, kRef, s1;
			double meanIPTime = 0;
		
			try {
				while ((c = br.readLine()) != null) {
					st = new StringTokenizer(c);	
					s1 = Integer.parseInt( st.nextToken() );
					meanIPTime = (double)( meanIPTime*k + s1)/(double)(k+1);
					k++;
				} 
			}
			catch( IOException ioe ) { ioe.printStackTrace(); }
		
			MessageManager.informationMessage( " Average inter-packet time is: " + MyNumberFormat.myPercentFormatter.format( meanIPTime ) +" ms.", graphic);			
	}


	/**
	 * To test this class
	 * @throws FileNotFoundException
	 * @throws IllegalArgumentException
	 */
	public static void main(String[] args) throws FileNotFoundException, IllegalArgumentException {
		
		isValidScenario("BeanMediumFormat.scf", 34d, 50, false );
		getMeanIPTime("BeanMediumFormat.scf", false);
		
		
	}
}
