/**
 * OnOffHeterogenousAggregate : Generation de trafic voix pour N sources => flux aggreges
 *                              Ces flux sont differents: par leur duree, leur caracteristiques => sources heterogene.
 *                              Dans ce fichier on recupere la liste de tous les flux audio OnOff qui sont crees et on cree des flux homogenes aggreges.
 *
 *                              N heterogeneousSource = N1homogeneousSourceType1 + N2homogeneousSourceType2 + ... + NnhomogeneousSourceTypen
 *                              with N= N1 + N2 +.. +Nn
 *
 * @author maria ladoue
 * @version 0 , 25/09/02 
 */

package multimedia.model;

import statistic.distribution.*;
import statistic.*;
import utils.definitions.MyExceptionDefinitions;
import multimedia.codec.*;

public class OnOffHeterogeneousAggregatedFlowModel extends GenericAggregatedModel implements RandomGenerator, AudioCodecExample {
    
    protected OnOffModel[] models;
    protected int[] weights;
    protected int nbDifferentFlow;
    
    public OnOffHeterogeneousAggregatedFlowModel() {
    	name = new String("OnOffHeterogeneousAggregatedFlow");
    	paramsCount = 4 * 1; // nb de param d'un model * le nombre de flots 
	}
    
    /**
     * OnOffHeterogeneousAggregatedFlowModel
	 * @param models
	 * @param weights
	 * @throws IllegalArgumentException
	 */
	public OnOffHeterogeneousAggregatedFlowModel( OnOffModel[] models, int[] weights) throws IllegalArgumentException {
		this();
		nbDifferentFlow = weights.length;
		if( models.length == nbDifferentFlow ) {
	    	this.models = models;
	    	this.weights = weights;
	    	paramsCount = 4*nbDifferentFlow;
		}
		else { 
	    	throw new IllegalArgumentException("OnOffHeterogeneousAggregatedFlowModel.constructor : for each source type you have to define an OnOff Model.");
		}
    }

	/**
	 * To transform N OnOff heterogeneous flows in K MMPP-2 Model.
	 * @return MMPP2Model[]
	 */
    public MMPP2Model[] transformation() {
		MMPP2Model[] newFlow = new MMPP2Model[nbDifferentFlow];
		//OnOffHomogeneousAggregate[] homogeneousFlow = new OnOffHomogeneousAggregate[nbDifferentFlow];

		for (int i=0;i<nbDifferentFlow;i++) {
	    	newFlow[i] = (new OnOffHomogeneousAggregatedFlowModel(weights[i],models[i])).approximation();
		}
		return newFlow;
    }

    public MMPP2Model getSpecificMMPP2Model(int idx) { return ((this.transformation())[idx]); }
    
    /**
     * getDifferentSrcTypeNumber
     * @return the number of source type 
     */
    public int getDifferentSrcTypeNumber() { return weights.length;  }

    /**
     * To get the number of source which have same characteristics
     *@param idx it's the number of the source between 0 and 
     *@return the source number of this type weights.length-1
     */
    public int getTypeSrcNumber(int idx) { return weights[idx]; }

     /**
      * getTypeSrcsNumbers()
      *@return the source number of each type in String format
      */
    public String getTypeSrcsNumbers() {
		String result = "{ " ;
		for (int i=0;i<getDifferentSrcTypeNumber()-1;i++) {
	    	result += getTypeSrcNumber(i) + ", ";
		}
		result += getTypeSrcNumber(getDifferentSrcTypeNumber()-1) + " }";
		return result;
    }

 	public String getCompleteName() { return new String("Model On-Off Heterogeneous Aggregated Flows"); }

	public int[] getSourceNumber() { return weights; }
	
	
	public OnOffModel[] getModels() { return models; }
	
	public void setModels( OnOffModel[] models ) throws IllegalArgumentException {
		if( models.length == nbDifferentFlow ) {
		  	this.models = models;
		}
		else { throw MyExceptionDefinitions.PARAMETERS_TABLE_SIZE; }
	} 

	public int[] getWeights() { return weights; }
	public void setWeights(int[] weights) throws IllegalArgumentException { 
		if( weights.length == nbDifferentFlow ) {
			this.weights = weights; 
		}
		else  { throw MyExceptionDefinitions.PARAMETERS_TABLE_SIZE; }
	}	
	
	public void setFlowTypeNumber( int nbDifferentFlow ) { this.nbDifferentFlow = nbDifferentFlow; }

    /**
     * Methode: Main pour tester la classe
     */
    public static void main (String[] args) {
		//AudioCodec c1 = new AudioCodec(200,0.1);
		//AudioCodec c2 = new AudioCodec(300,0.05);
	OnOffModel[] srcType = { new OnOffModel(1,new ExponentialDistribution(0.2),new ExponentialDistribution(0.1),G728),new OnOffModel(0,new ExponentialDistribution(2.8409),new ExponentialDistribution(1.5385),G711)};
	int[] nbTypeSrc = { 150,50 };
	OnOffHeterogeneousAggregatedFlowModel oohga1 = new OnOffHeterogeneousAggregatedFlowModel(srcType,nbTypeSrc);

	System.out.println(((oohga1.transformation())[0]).getParam(0,"lambda"));
	System.out.println(((oohga1.transformation())[0]).getParam(1,"lambda"));
	System.out.println(((oohga1.transformation())[1]).getParam(0,"lambda"));
	System.out.println(((oohga1.transformation())[1]).getParam(1,"lambda"));
	System.out.println(((oohga1.transformation())[0]).getParam(0,"beta"));
	System.out.println(((oohga1.transformation())[0]).getParam(1,"alpha"));
	System.out.println(((oohga1.transformation())[1]).getParam(0,"beta"));
	System.out.println(((oohga1.transformation())[1]).getParam(1,"alpha"));

    }

}

