/**
 * OnOffModel : Modele On-Off de modelisation d'une source audio
 * 
 * @author maria ladoue
 * @version 0.1, 04/07/02
 */

package multimedia.model;

import statistic.distribution.*;
import multimedia.codec.*;


public class OnOffModel extends GenericAudioModel {

    // Definition des parametres

    public int etatDepart =1;  // Represente le premier etat du processus
    public GenericDistribution loi1; // Loi representant le temps passe dans l'etat 1 
    public GenericDistribution loi2; // Loi representant le temps passe dans l'etat 2
    
    
    //Definition des constructeurs

    public OnOffModel () {
		name = new String("OnOff");	// model name
		etatCount = 2;						// le nb d'etat de ce model est tjs 2!
		paramsCount = 4;					// etat depart, loi1, loi2 et codec
		codec = new AudioCodec();
		loi1 = new ExponentialDistribution();	// initialisation des loi1 et loi2
		loi2 = new ExponentialDistribution();
    }

    /**
     * OnOffModel 
     *
     * @param unEtatDepart it defines the state of the system at the start: it could be 0 or 1
     * @param nomLoi1 it defines the distribution which represents time passed in state 1
     * @param nomLoi2 it defines the distribution which represents time passed in state 2
     * @param unCodec it's the associated Codec
     */
   public OnOffModel (int unEtatDepart, GenericDistribution nomLoi1,GenericDistribution nomLoi2, AudioCodec unCodec) throws IllegalArgumentException {
	this(); // Pour recuperer le nom
       if ( unCodec.getTpsIP() < (1/getParam(1,"alpha")) ) {  // T must be smaller than 1/alpha
		setEtatDepart(unEtatDepart);
		this.loi1 = nomLoi1;
		this.loi2 = nomLoi2;
		this.codec = unCodec;
       }
	else {
	    throw new IllegalArgumentException("OnOffModel.constructor : we must have T < 1/alpha");
	}
    }


    // Definit un certain nombre de methodes

    /*
     * DS: Probleme avec cette methode. Elle part du principe que les lois utilises n'ont qu'un
     * seul parametre... c'est ok si on a des lois exponentielles mais ca ne fonctionnera pas
     * dans le cas d'une distribution a deux parametres (pareto, etc.).
     */
    public double getParam( int unEtat, String unParam) throws IllegalArgumentException { 
		if (unEtat == 0) { return loi1.getParam(0); }
		else if (unEtat == 1) { return loi2.getParam(0); }
		else { throw new IllegalArgumentException("OnOffModel.getParam : Les deux valeurs possibles pour un etat sont 0 ou 1.");}
    }

    // A implementer!!!!
    public void setParam( String unParam, double uneValeur) throws IllegalArgumentException {
 		//TODO
		throw new IllegalArgumentException("OnOffModel.setParam : This function MUST be implemented...");
    }


    public Object getParameter(int idx) throws IllegalArgumentException {
 	switch (idx) {
	case 0: return loi1;
	case 1: return loi2;
	default: throw new IllegalArgumentException("OnOffModel.getParameter : it's not a valid parameter");
	}
    }
    

    // Methode destinee a etre compatible avec GenericModel...peut-etre pas tres propre
    public double calculT( double etat )  throws IllegalArgumentException {
		if ( etat==0 ) { return loi1.rand(); }
		else if ( etat==1 ) { return loi2.rand(); }
		else { throw new IllegalArgumentException("Les deux valeurs possibles pour un etat sont 0 ou 1."); }
    } // fin de la methode calculT


    /**
     * getParamOn : to calculate Pon = beta/(alpha + beta)
     *  where alpha = 1/Ton  and beta = 1/Toff
     */
    public double getParamOn() {
	return loi2.getParam(0)/( loi2.getParam(0)+ loi1.getParam(0));
    }

    /**
     * meanRate : to calculate mean of generated packet number per second
     *   mean = 1/InterPacketTime*Pon = lambda*Pon
     * @return mean rate in kbits/s
     */
    public double meanRate() {
	return getParamOn()*(1d/(getCodec().getTpsIP()))*(getCodec().getSize())*8d/1000;
    }

	public String getCompleteName() { return new String( name + " Model");  }
    
	public void setOnDistribution(GenericDistribution onDistrib) { loi1 = onDistrib; }
	public void setOffDistribution(GenericDistribution offDistrib) { loi2 = offDistrib; }


    /**
     * Methode: Main pour tester la classe OnOffModel
     */

    public static void main (String[] args) {
		AudioCodec c1 = new AudioCodec(200,4);
		OnOffModel m1 = new OnOffModel(0,new ExponentialDistribution(0.2),new ExponentialDistribution(0.1),c1);
		System.out.println(m1.getEtatDepart()+" " +  m1.getName() );
    }
}

