/**
 * ChapeauSigmaDistribution : Definition de la loi de chapeausigma
 *
 * @author maria ladoue
 * @version 0.1, 01/09/02 
 */

package statistic.distribution;
  
import java.lang.Math;
import java.util.Vector;
import java.util.Date;


import statistic.function.*;

public class ChapeauSigmaDistribution extends GenericDiscreteDistribution  {  //extends ServiceTimeDistribution { 

    /**    
     * loi de chapeausigma : <BR> 
     *  P(chapeausigma=k) = P(sigma>=k)/E(sigma) = P(sigma(0,j)=k) for k>=1 <BR> 
     *     where E is mean <BR> 
     */

    public SigmaDistribution sd;
	
	 int KINIT =1000;
	 
	 //Create a vector where we'll stock sumCdf(k) of rank k
     int size = 1001;
     Vector F = new Vector(size);
     
	 
/**
 * ChapeauSigmaDistribution
 * 
 * @param the associated sigma distribution
 */
    public ChapeauSigmaDistribution (SigmaDistribution sigmaDistribution) {
		name = new String("Chapeausigma");
		this.sd = sigmaDistribution;
		cdf = false;
    }
    
  /**
 * ChapeauSigmaDistribution
 * 
 * @param the associated sigma distribution
 */
    public ChapeauSigmaDistribution (SigmaDistribution sigmaDistribution,boolean cdf) {
		this(sigmaDistribution);
		this.cdf = cdf;
		if(cdf == true && sd.getInitialisationStatus() == true ) {
			//if ( sd.getKMAX(KFACTOR) < getKMAX(KFACTOR) )   {
		 	//	sd.initializeSumCdf(sd.getKMAX(KFACTOR), getKMAX(KFACTOR)) ;
			//}
			initializeSumCdf(1, KINIT);
		}
    }

	public boolean hasValidParam(double[] desParams) {
		return true; //TODO Should we do another test here ?
	}
	
	/**
	 * Method cdf.  chapeauSigmacdf(k) = P(chapeauSigma=k)
	 * 													 = P(sigma>=k)/E(sigma)
	 * 													 = 1/ E(sigma) *[1-sum(sigmacdf(l),1,k-1)]
	 * 
	 * @param k
	 * @return double
	 * @see statistic.GenericDiscreteDistribution#cdf(int)
	 * @throws IllegalArgumentException 
	 */
    public double cdf(int k) throws IllegalArgumentException {
		if (k >= 1) {
		    double sumCdf = 0d;
		      
	    	for (int l=1;l<k;l++) {
			sumCdf += sd.cdf(l);	
	    }	
	    return ( (1d - sumCdf)/ sd.mean() );
	}
	else {
	    throw new IllegalArgumentException("ChapeauSigmaDistribution.cdf` : define for k>=1, k" + k);
	}
    }
    
	public void initializeSumCdf(int a,int b)  {
   	 		F.add(0, new Double(0));
   		 	double sumCdf = 0d;
   		 	
    		for (int l=a;l<=b;l++) {
				sumCdf += cdf(l);
				F.insertElementAt(new Double(sumCdf), l);
	    	}
    }


 public double getSumCdf(int k)   throws IllegalArgumentException {
 	
 	if ( (getInitialisationStatus() == true) && k>=0 && k<= Math.max(getKMAX(KFACTOR),sd.getKMAX(KFACTOR))) {
 		
 		if(  k>=F.size() ) {
 			initializeSumCdf(F.size() +1, k);
 		}
 		//System.out.println("k: " +k);
    	  return ((Double)(F.elementAt(k))).doubleValue();
 	}
 	else  {
 		throw new IllegalArgumentException("ChapeauSigmaDistribution. getSumCdf : you have to SigmaDistribution.initializeSumCdf() and k has to be > 0" + k);
 	}
    }

     public double getFMAX()  {
     	if (getInitialisationStatus() == true) {
    	 return getSumCdf(KINIT);
     	}
     	else  {
 			return super.getFMAX();
 		}
    }
    
   
 
  /**
	 * Method mean. 
	 * 			E(ChapeauSigma) = sum(ro(k),0,inf) = 1/2( 1 +E(sigma^2)/E(sigma) )
	 * @return double the mean value
	 * @see statistic.GenericDistribution#mean()
	 */
    public double mean() {
	  		double sumRoK = 0d;
		  	int it = 1000;
		  	GenericFunction gf = sd.getAutocorr();
		  
		  	while ( Math.abs((gf.function(it+1)) - (gf.function(it)) ) > Math.pow(10,-5)  && it < (Integer.MAX_VALUE-101) )  {
		  		it += 100;
		  	}
		  	
			for (int i=0;i<=it;i++) {
        			sumRoK += gf.function(i);
        	}
        	//System.out.println("iteration number: " + it + " Moyenne de SigmaChapeau: " + sumRoK);
			return sumRoK; 
    }

public double var() {
return 0d;	
}

	public String getEquation() {
		return new String("P(chapeausigma=k) = P(sigma>=k)/E(sigma) = P(sigma(0,j)=k) for k>=1 where E is mean"); 
	}
	
    /**
     * Methode: Main pour tester la classe
     */
    
    public static void main (String[] args) {
		SigmaDistribution sd1 = new SigmaDistribution(new SRDFunction(0.333), true);
		ChapeauSigmaDistribution csd1 = new ChapeauSigmaDistribution(sd1,true);
		//System.out.println("Debut: " + new Date().toString());
		//System.out.println(csd1.mean());
		//System.out.println("Fin: " + new Date().toString());
		
		System.out.println("Debut: " + new Date().toString());
		System.out.println("KMAX: " + csd1.getKMAX(1000));
		System.out.println("FMAX: " + csd1.getFMAX());
		System.out.println("Mean: " + csd1.mean());
		for (int i=0;i<50;i++) {
		System.out.println(csd1.randInt());
		}
		System.out.println("Fin: " + new Date().toString());
    }


}
