/**
 * ExponentialDistribution : Definition de la loi exponentielle
 *
 * @author 
 * @version 0.1, 04/07/02 
 */

package statistic.distribution;  // Permet de specifier que cette classe appartient au package Statistique

import java.lang.Math;

import statistic.*;

public class ExponentialDistribution  extends GenericContinuousDistribution implements RandomGenerator  {

    /**
     * Equation de la densite: 
     *    f(x)= 1/lambda * exp(-x/lambda) pour x>0, lambda>0
     *    where 1/lambda = mean time of a process 
     */
    
    public static int LAMBDA = 0;
   
   
    //Definition des constructeurs

    public ExponentialDistribution() {
	paramsCount = 1;
	name = new String("Exponential");

	params = new double[paramsCount]; // initialisation du tableau des valeurs des parametres
	paramNames = new String[paramsCount]; // initialisation du tableau des noms des parametres
	params[LAMBDA] = 0d;  //0.1d normalement debug 17/01/0.
	paramNames[LAMBDA] = new String("lambda");
    }
    
    public ExponentialDistribution(double unLambda) throws IllegalArgumentException {
		this();
		if (unLambda >0)
			setParam(LAMBDA,unLambda);
		else
			throw new IllegalArgumentException("ExponentialDistribution.constructor : lambda <=0");
    }
    
    // Ce constructeur n'a pas d'interet sauf celui d'etre compatible avec les autres distributions dans un cas general
    public ExponentialDistribution(double[] desParams) throws IllegalArgumentException {
		this();
		if (hasValidParam(desParams))
			setParams(desParams);
		else
			throw new IllegalArgumentException("ExponentialDistribution.constructor : lambda <=0");
    }
    
	public boolean hasValidParam(double[] desParams) {
		if (desParams[LAMBDA] >0)
			return true;
		else
			return false;
	}

    // Definitions de quelques methodes

    /**
     * Returns the value of the probability distribution function at x
     */
    public double pdf(double x) {
	if (x >= 0) {
	    return (params[LAMBDA] / Math.exp(params[LAMBDA] * x));
	}
	else { return 0; }
    }

    /**
     * Returns the value of the cumulative distribution function at x (=Fonction de repartition)
     */
    public double cdf(double x) {
        if (x > 0) {
            return (1.0d - Math.exp(-1.0d * params[LAMBDA] * x ));
        } 
	else { return 0; }
    }

    /**
     * Return the value of Inverse Function at x 
     * @param x with x>=0 and x<=1
     */
    public double inverseCdf(double x) throws IllegalArgumentException {
	if ( x>=0 && x<=1 ) {
	    return -1d/ params[LAMBDA]*Math.log( 1d - x);
	}
	else {
	    throw new IllegalArgumentException("ExponentialDistribution.inverseCdf : x must be >=0 and <=1");
	}
    }
    

    public double mean() {  return (1d/params[LAMBDA]); }
    
    public double var() { return Math.pow(1.0d/params[LAMBDA], 2.0d);  }

    public double rand() { return ((-1.0d/params[LAMBDA]) * Math.log(1-random.nextDouble())); }
 
	public String getEquation() {
		return new String(" f(x)= 1/lambda * exp(-x/lambda) pour x>0, lambda>0 where 1/lambda = mean time of a process");
	}

    /**
     * Methode: Main pour tester la classe
     */

    public static void main (String[] args) {
	ExponentialDistribution loiExp = new ExponentialDistribution (4d);
	//ExponentialDistribution  loiExp = new ExponentialDistribution (Double.longBitsToDouble(0x7ff0000000000000L));
	//loiExp.setParam(0,Double.longBitsToDouble(0x7ff0000000000000L));

	//System.out.println(Double.compare(0,Double.POSITIVE_INFINITY));
	
	for (int i=0;i<10000;i++) {
	    System.out.println(loiExp.rand());//+ " " + loiExp.getParam(LAMBDA));
	}
    } 

}
