/**
 * GenericDiscreteDistribution : Definition generique de loi discrete
 *
 * @author maria ladoue
 * @version 0.1, 01/09/02 
 */
package statistic.distribution;


import statistic.RandomGenerator;


/**
 * @author mladoue
 *
 */
public abstract class GenericDiscreteDistribution  extends GenericDistribution implements RandomGenerator {


 final  int KFACTOR = 1000;
 public boolean cdf;
 
    /**
     * Returns the value of the cumulative distribution function at x
     *    where x is an int
     */
    public abstract double cdf(int x);

    /**
     * Retourne la valeur de la pdf(x)
     *    where x is a double
     */
    public double pdf(double x) { return (double)this.pdf(x); }

    /**
     * Method cdf Returns the value of the cumulative distribution function at x
     *                      where x is a double
     *                     It's defined for compatibility problems
     * @param x
     * @return double
     */
    public double cdf(double x) { return (double)cdf(x); }

	/**
	 * Method randInt. Methode a definir dans chaque classe permettant d'obtenir un tirage pour une loi discrete
	 * @return int
	 * @throws OutOfMemoryError
	 */
  
   public int randInt()  throws OutOfMemoryError {
    	
    	setInitialisationStatus(true);
 		int k = 0;
 		
      	double FMAX = getFMAX();
      	double[] F = new double[2];	
		//F[0]=0d;F[1]=0d;
		
		 double u = random.nextDouble();
		  
         while( u > FMAX )  {
        		u = random.nextDouble();
          }
     //System.out.println("u: " + u + " cdf(1): "+cdf(k+1) + " cdf(2): "+(cdf(k+2) +cdf(k+1)));	
        while ( F[1] < u && k<getKMAX(KFACTOR)) {	 
                ++k;
                try { 
                	//System.out.println("sumCdf(k): " +getSumCdf(k));	
                   F[1] = getSumCdf(k);	//F[1] = F[0] + cdf(k);
                   //	F[0] = F[1];
                } 
                catch (ArrayIndexOutOfBoundsException e) {
                    System.out.println("GenericDiscreteDistribution.randInt : value of k is greater than the size of the table F[].");
               }               
        	   }	
        return k;
    } 
 
/**
 * Method randIntFast. Correct on 01/04/03
 * @return int
 * @throws OutOfMemoryError
 */
  public int randIntFast() throws OutOfMemoryError {
    	
 		int k = 0;
      	double[] F = new double[2];	
		
		double u = random.nextDouble();
	
        while ( F[1] < u ) {	 
        	if( k <5000) {  // avant le 01/04/03 k<1000
                k++;
                try { 
                F[1] = F[0] + cdf(k);
                F[0] = F[1];
                } 
                catch (ArrayIndexOutOfBoundsException e) {
                    System.out.println("GenericDiscreteDistribution.randIntFast : value of k is greater than the size of the table F[].");
                }
        	}
        	else {
        		// Si on arrive pas a trouve un k<1000, on retire un u et on recommence tout jusqu'a ce que se soit bon!!
        		//k=randInt();
        		//break;	
        		//System.out.println("GenericDiscreteDistribution.randIntFast => Error!");
        		k=0;
        		 u = random.nextDouble();
        		 F[0]=0d;
        		 F[1]=0d;
        	}             
        	   }	
        return k;
    }   
 
    /**
	 * Method getFMAX: For some Discrete Distribution we could have a pb to generate a randInt
	 * 							 because of the cdf value
	 * 							So when we generate random value we verify that
	 * 							 this value is < getFMAX()
	 * @return double
	 */
    public double getFMAX() {
    	  	double sumK = 0;
        	for (int i=1;i<=getKMAX(KFACTOR);i++) {
        			sumK += cdf(i);
        	}
        	return sumK;
    }
	
	/**
	 * Method getKMAX. randInt is between 1 and KMAX 
	 * @param x  we could have a more or less big KMAX (il changes the iteration number)
	 * @return int
	 */
    // On limite la valeur de k => KMAX
    public int getKMAX(int x) {	 return (int)((x+1)*mean()); }
    

	/**
 	* Method rand : to generate a random double(compatibility with GenericDistribution) value
 	* 
 	* @return a double value
 	*/
    public double rand() { return (double)randIntFast();  }
    
    public boolean getInitialisationStatus()  { return cdf;  }
    
    public void setInitialisationStatus(boolean cdf)  { this.cdf=cdf;   }
     
    public abstract double getSumCdf(int k);

// Meme methode que randInt mais avec un Vector...on ne l'utilise pas
/** public int randIntVector() {

        double u = random.nextDouble();
        int k = 0;
        int size = 5;
        Vector F = new Vector(size);
        F.add(0, new Double(0));

		for (int i=1;i<=getKMAX(KFACTOR);i++) {
        		  double Fi = (((Double)(F.elementAt(i - 1))).doubleValue()) + cdf(i);
                   F.insertElementAt(new Double(Fi), i);
        }	
	
	while (  u > (((Double)(F.elementAt(this.getKMAX(KFACTOR)-1))).doubleValue() ) ) {
	  		//System.out.println("GenericDiscreteDistribution.randInt : value of k is greater than:" + getKMAX(KFACTOR) ); 
        	
        	u = random.nextDouble();
	}
        while ((((Double)(F.elementAt(k))).doubleValue()) < u) {
                k++;
            } 

        return k;
    }
*/



}