/**
 * SigmaDistribution : Definition de la loi de sigma
 *
 * @author maria ladoue
 * @version 0.1, 01/09/02 
 */

package statistic.distribution;
  

import java.util.Vector;

import statistic.function.*;


public class SigmaDistribution extends GenericDiscreteDistribution  {//extends ServiceTimeDistribution

    /**    
     * loi de sigma : <BR> 
     *  P( sigma = k ) = (ro(k+1)-2*ro(k)+ro(k-1))/(1-ro(1)) <BR> 
     *     where ro  = autocorrelation function  <BR> 
     */

    public String name;
    public GenericFunction autoCorr;
    

 	 //Create a vector where we'll stock sumCdf(k) of rank k
     int size = 5;
     Vector F = new Vector(size);

	/**
	 * SigmaDistribution : default constructor
	 */
    public SigmaDistribution() {	
		name = new String("Sigma");
		this.autoCorr = new MarkovianFunction(0.2);
		this.cdf = false;
    }


	/**
	 * SigmaDistribution : Constructor with a GenericFunction as parameter 
	 * @param f	it's a GenericFunction
	 */
    public SigmaDistribution (GenericFunction f) {
    	this();	
		this.autoCorr = f;
    }
    
	/**
	 * SigmaDistribution : Constructor with a GenericFunction as parameter 
	 * @param f	it's a GenericFunction
	 * @param cdf	it's a boolean which permit to specify 
	 */
    public SigmaDistribution (GenericFunction f, boolean cdf) {	
	this(f);
	this.cdf = cdf;
	if(cdf == true) {
		initializeSumCdf(1,getKMAX(KFACTOR)) ;
	}
    }

	public boolean hasValidParam(double[] desParams) {
		return true;// Not of a real use here
	}
	
	/**
	 * Method cdf : To compute cumulative distribution function
	 * @see statistic.GenericDiscreteDistribution#cdf(int)
	 */
    public double cdf(int k) throws IllegalArgumentException {
	if (k >= 1) {
	    return ( (autoCorr.function(k+1) - 2d*autoCorr.function(k) + autoCorr.function(k-1)) /
		     (1d- autoCorr.function(1)) );
	}
	else {
	    throw new IllegalArgumentException("SigmaDistribution.cdf` : define for k>=1: " + k);
	}
    }
    
     public double getSumCdf(int k)   throws IllegalArgumentException {
     	if ( (getInitialisationStatus() == true) &&( k>=0) && ( k <= F.size()) )  {
     		return ((Double)(F.elementAt(k))).doubleValue();
     	}
     	else  {
 		throw new IllegalArgumentException("SigmaDistribution. getSumCdf : you have to SigmaDistribution.initializeSumCdf(a,b) and 0< k <" + (getKMAX(KFACTOR) +1)+ " , " + k  + " , " + cdf + "  "+ F.size());
 	}
    }
    
 	public void initializeSumCdf(int a,int b)  {
   	 		F.add(0, new Double(0));
   		 	double sumCdf = 0d;
   		 	
    		for (int l=a;l<=b;l++) {
				sumCdf += cdf(l);
				F.insertElementAt(new Double(sumCdf), l);
	    	}
    }
    
     public double getFMAX()  throws IllegalArgumentException {
     	if ( getInitialisationStatus() == true ) {
     		int KMAX = getKMAX(KFACTOR);
     		if (F.size() < KMAX) {
     			initializeSumCdf((F.size() +1), KMAX);
    		}
     		return getSumCdf(KMAX);
     	}
     	else  {
 			return super.getFMAX();
 		}
     }
    
    
	/**
	 * Method mean. 
	 * @see statistic.Distribution#mean()
	 */
    public double mean()  { return (1d / ( 1d - autoCorr.function(1) ));   }
    
    public double var() { return 0d; }
    
    
/**
 * Method getAutocorr : to be used by ChapeauSigmaDistribution
 * @return GenericFunction
 */
  public GenericFunction getAutocorr()  { return autoCorr;   }
    
 /**
 *  Method setAutocorr :
 * @param GenericFunction
 */
  public void setAutocorr(GenericFunction f)  { this.autoCorr = f;   } 
    
  public String getEquation() {
	  return new String("P( sigma = k ) = (ro(k+1)-2*ro(k)+ro(k-1))/(1-ro(1)) where ro  = autocorrelation function");
  }

    /**
     * Methode: Main pour tester la classe
     */

    public static void main (String[] args) {

	//new MarkovianFunction(0.2));
	//SigmaDistribution sd1 = new SigmaDistribution(new LRDFunction(0.0005));
	//System.out.println(sd1.rand());
	
	//double[] coeffFuncjpm = { 0.334, 0.566 };	
	//GenericFunction[] gftjpm = { new MarkovianFunction(0.021), new SRDFunction(0.269) };
	//GenericFunction gfjpm = new MixedFunction(gftjpm ,coeffFuncjpm);
	GenericFunction gfjpm = new  MarkovianFunction(0.2);
	SigmaDistribution sd2 = new SigmaDistribution(gfjpm);
	
	double[] coeffFuncbh = { 0.451, 0.549 };
	GenericFunction[] gftbh = { new MarkovianFunction(0.129), new SRDFunction(0.108) };
	GenericFunction gfbh = new MixedFunction(gftbh ,coeffFuncbh);
	SigmaDistribution sd3 = new SigmaDistribution(gfbh,true);
	
	System.out.println("Debut test");
	//System.out.println("mean: " + sd3.mean());
	
	//int sumK = 0, iteration=10000,KMAX=0,k;
	//for (int i=0;i<iteration;i++) {
		System.out.println( "cdf: " + sd2.cdf(500));
		System.out.println( "mean: " + sd2.mean());
		//System.out.println(new Date().toString());
		//for (int i=0;i<10;i++) {
		//System.out.println("Sigma Distribution: valeur aleatoire: " + sd3.randInt());
		//}
		//System.out.println(new Date().toString());
		
		//System.out.println(new Date().toString());
		//System.out.println("valeur aleatoire: " + sd3.randIntVector());
		//System.out.println(new Date().toString());
		
		  SigmaDistribution sd = new SigmaDistribution();
		  SigmaDistribution sd0 = new SigmaDistribution(new LRDFunction(0.3));
 		 System.out.println(sd.getAutocorr().toString());
		System.out.println(sd0.getAutocorr().toString());
	System.out.println("Fin test");
    }
    
    
  

}
