/**
 * MixedFunction : Definition des fonctions permettant de definir la fonction d'autocorrelation (ro) en mixant differents types de fonction
 *
 * @author maria ladoue
 * @version 0.1, 26/09/02 
 */

package statistic.function;


public class MixedFunction  extends GenericFunction {

 /**
  * Function type:<BR>
  *   f(k) = x<sub>1</sub> * f<sub>markovian</sub>(k) + x<sub>2</sub> * f<sub>srd</sub>(k) + x<sub>3</sub> * f<sub>lrd</sub>(k)<BR>
  */
protected GenericFunction[] functions;
protected double[] coeff;


public MixedFunction() {
	 name = new String("Mixed");
	 functions = new GenericFunction[2];
	 coeff = new double[2];
	 for (int i=0;i<2;i++) {
	 functions[i] = new SRDFunction();	
	 }
}

/**
 * Method MixedFunction.
 * @param functions
 * @param coeff
 * @throws IllegalArgumentException
 */
public MixedFunction(GenericFunction[] functions, double[] coeff) throws IllegalArgumentException{
    	this();
		if (functions.length != coeff.length) {
	    	throw new IllegalArgumentException("MixedFunction.constructor : For each function you have to specify a coefficient even if it's 1.");
		}
		else {
	    	this.functions = functions;
	    	this.coeff =  coeff;
		}
    }

public double function(int k) throws IllegalArgumentException {
	double ans = 0; 
	if (k>=0) {
	    for (int i=0;i<functions.length;i++) {
		ans += coeff[i] * functions[i].function(k);
	    }
	    return ans;
	}
	else {
	    throw new IllegalArgumentException("MixedFunction.function : define for k>=0.");
	}
    }

public String getEquation() {
	String ans = new String("Mixing of:");
	for (int i=0;i< functions.length;i++) {
		ans = ans.concat(" " + functions[i].getName());
	}
    	return ans;
  }

public double[] getWeight() { return coeff; }
public void setWeight(double[] weight) { this.coeff = weight; }
public GenericFunction[] getSpecificFunctions() { return functions; }
public GenericFunction getSpecificFunction(int funcNb) { return functions[funcNb]; }
public void setSpecificFunction(int funcNb,GenericFunction gcd) { functions[funcNb] = gcd; }

// To know with have a simple GenericFunction oe a true MixedFunction composed of several functions
public int getFunctionsNumber() {	return functions.length; }

/**
 * Method getNames.
 * @param functions
 * @return String[]
 */
public static String[] getNames(GenericFunction[] functions) {
	String[] names = new String[functions.length];
	for (int i=0;i<functions.length;i++) {
		names[i] = functions[i].getName();
	}
	return names;
}
 
/**
 * Renvoie et fixe les differentes valeurs de b
 */
    public double[] getFunctionsParameter() { 
    	double[] params = new double[functions.length];
    	for (int i=0;i<functions.length;i++) {
    	params[i] = functions[i].getParam();
    	}
   		return params;
    }
    
    public void setFunctionsParameter(double[] b) {
    	for (int i=0;i<functions.length;i++) {
    		functions[i].setParam(b[i]);
    	}
   }
    
    /**
     * Methode: Main pour tester la classe
     */

    public static void main (String[] args) {
	GenericFunction[] functions = { new SRDFunction(0.08), new MarkovianFunction(14.65) };
        double[] weigth =  {2,4};
	MixedFunction md1 = new MixedFunction(functions,weigth);
	System.out.println(md1.function(2));
    }
}

