/*
 * Bench scenario player.
 * Avec la premire machine connecte.
 * Scenario parametrables
 * Taille des messages paramtrable.
 * <br><u><b>Remarque importante</b></u> : l'envoi de messages correspond  un stockage dans la file d'envoi.
 * La barre de progression indique donc la progression du stockage des messages dans la file.
 * Ainsi, la partie du programme envoyant les messages est excute quasiment instantanment.
 */
 
package temp;

import javax.swing.*;
import java.util.*;
import java.net.*;
import java.io.*;
import java.text.*;

import utils.*;
import traffic.*;

public class BenchUDPScenarioPlayer extends Thread implements UDPSocketReader {

        protected GestionUDP gestionUDP;
        protected UDPServer server;
        protected JTextArea textAreaEtatApplication;
        protected JProgressBar progressBar;
         // Fichier scenario
    	protected File file;
    	protected FileReader fr;
       
        protected int longueurMessages;  // fixe dans certains cas comme l'audio
        protected int nbMessages;
        protected int cos;
        
         //Numro du datagramme envoye
         // le premier datagramme de donnees correspondra au Numero 2 car le 1 est reserve au mangement...
    	protected int noDatagrammeEnvoye=2; 

        /**
         * Initialise le Thread.
         * @param benchUDP La classe parente
         * @param serveur L'instance du serveur
         * @param textAreaEtatApplication La JTextArea pour afficher les messages concernant l'excution de l'application
         * @param progressBar La barre de progression de l'application
         * @throws FileNotFoundException
         */
        public BenchUDPScenarioPlayer(GestionUDP gestionUDP, UDPServer serveur, JTextArea textAreaEtatApplication, JProgressBar progressBar) throws FileNotFoundException{
                this.gestionUDP = gestionUDP;
                this.server = serveur;
                this.textAreaEtatApplication=textAreaEtatApplication;
                this.progressBar=progressBar;
                
                if (demanderParamtres()) { this.start(); }
        }

        /**
         * Implantation.
         */
        public synchronized void wakeUp(){
                notify();
        }

        /**
         * Appel  la mthode wait() de la classe Object.
         */
        protected synchronized void attendre(){
	    try{
		wait();
	    }
	    catch (InterruptedException e) {
		System.out.println(e.toString());
	    }
        }

        /**
         * Bench scenario player.
         * Les paquets envoyes vont etre forme comme suit:
         * 	<i>numeroPaquet</i> + " " + "random data" 
         * 	ou la taille de l'en-tete "numeroPaquet" est fixe et est fonction de la taille du fichier
         * 
         */
        public void run(){
	    if (server.nbConnection()==0){
			textAreaEtatApplication.append("No connected client.\n");
	    }
	    else{
			textAreaEtatApplication.append("Bench: Scenario Player: " + file.getName() + ".\n");
		
			
			SocketEcriture se = server.getThreadEcriture(0);
			SocketLecture sl = server.getThreadLecture(0);
			
			BufferedReader br = new BufferedReader(fr);
			String c;  // ligne courante du document a regarder
			StringTokenizer st;
			
			sl.setUDPSocketReader(this);
			
			nbMessages = FileLineCounter.countLines(file.getName());
			// packetNumberSize corresponds to the size of PacketNumber + one space: " "
			short packetNumberSize = (short)(MyNumberFormat.getPacketNumberHeaderSize(nbMessages) + 1);
			int messageSize = longueurMessages - packetNumberSize;
			String decF = "0";
			for (int k=1;k<packetNumberSize;k++) {
				decF = decF.concat("0");
			}
			
			DecimalFormat myPacketNumberSizeFormatter = new DecimalFormat(decF);
			
			se.messageAEnvoyer("1 scenarioplayer");   // premier message envoye
			se.messageAEnvoyer("1 "+ nbMessages); //+ "Number of messages to be send");
			//se.messageAEnvoyer("1 test");  // a enlever
			ChaineAleatoire ca; // chaine de caracteres aleatoires qui constituera les messages envoyes
			progressBar.setStringPainted(true);
			
			float f = nbMessages;
			
			f=100/f;
			int i = 1;
		
			String date = new Date().toString();
			textAreaEtatApplication.append("Start emission: " + date + ".\n");
		
			//envoi des messages au client
			try {
		  	while ( (c = br.readLine()) != null ) {
		  	
		  	st = new StringTokenizer(c);
			
			//definition of scenario file type
			int nbTokens = st.countTokens();
			
			String token1 = st.nextToken();
			String token2 = st.nextToken();
			
			// Emission du message
			if(nbTokens == 2) {
				ca = new ChaineAleatoire(messageSize);
				//System.out.println(ca);
				se.messageAEnvoyer(myPacketNumberSizeFormatter.format(noDatagrammeEnvoye) + " " + ca.gnrerChane());
			}
			
			if(nbTokens == 3) {
		 		try {
		 		String token3 = st.nextToken();
		 		 
		 		//Determine le debit applicatif => ne tient pas compte de l'overhead IP...
		 		ca = new ChaineAleatoire(Integer.valueOf(token3).intValue()-packetNumberSize);
				se.messageAEnvoyer(myPacketNumberSizeFormatter.format(noDatagrammeEnvoye) + " " + ca.gnrerChane());
  				}
		 	 	catch(NoSuchElementException e) {
		  		System.out.println(e);	
		  		}
			}
									
			// Attente entre 2 paquets
			try {
			    this.sleep(Long.valueOf(token1).longValue(),Integer.valueOf(token2).intValue());
			}
			catch(InterruptedException e) {
			    System.out.println(e);
			}
					  	
		    progressBar.setValue((int)(f*i)); // mise a jour de la barre de progres
		  
		    i++;
		    noDatagrammeEnvoye++;
		    
		} // end while

			textAreaEtatApplication.append("\nFin d'emission: " + new Date().toString() + ".\n");
			//noDatagrammeEnvoye=2;  // fin de l'emission...on "remet" a "zero" le compteur
			}  // end try
			catch(IOException e) {
				System.out.println("UDPServer:run : " + e.toString());
			}
		
			progressBar.setValue(0);
			progressBar.setStringPainted(false);
		
		textAreaEtatApplication.append(">Rception des rsultats du bench.\n");
		progressBar.setStringPainted(true);
		int[] messageArrivalTime = new int[nbMessages];
		String message;
		//rception des temps d'arrive des messages chez le client
		for (int j=0;j<nbMessages;j++){
			
		    progressBar.setValue((int)(f*j));
		    
		    while (sl.pasDeMessagesReu()){
		    	System.out.println("Debug attente");
				attendre();
		    }
		    message = sl.premierMessageReu();  // on retire un a un les messages dans la file d'attente
		    try {
				messageArrivalTime[j] = Integer.parseInt(message);
		    }
		    catch(NumberFormatException nfe) {
		    	System.out.println(nfe.toString());
		    }
		    catch (Exception e) {
				System.out.println(e.toString());
		    }
		}
		
		progressBar.setValue(0);
		progressBar.setStringPainted(false);
		
		resultsSave(messageArrivalTime);
		textAreaEtatApplication.append(">Fin de l'analyse.\n");
		
		
	    }
	   
        }  // end run()
 
 
    //Sauvegarde des rsultats dans un fichier sur le disque pour un traitement ultrieur.
    protected void resultsSave(int[] benchResults) {
    	
		SocketLecture sl=server.getThreadLecture(0);
		DatagramSocket socket=sl.getSocket();
		int tailleBufferReception = 0;
		int tailleBufferEmission = 0;
		try {
	    	tailleBufferReception=socket.getReceiveBufferSize();
	    	tailleBufferEmission=socket.getSendBufferSize();
		}
		catch (SocketException e) {
	    	System.out.println(e.toString());
		}
			
		BenchUDPScenarioPlayerS b = new BenchUDPScenarioPlayerS(benchResults,1000,tailleBufferReception,tailleBufferEmission,sl.getTailleBufferRceptionPaquet());
		textAreaEtatApplication.append(">>Les rsultats du bench ont t sauvegards dans le fichier : " + b.getNomFichier()+"\n");
        }
    
    /**
     * Demande  l'utilisateur les paramtres pour effectuer le bench
     * @return un boolen concernant la faon dont a t ferme la boite de dialogue par l'utilisateur.
     */
    protected boolean demanderParamtres() throws FileNotFoundException {
		Vector v1= new Vector();
		v1.add("Scenario file name: ");
		v1.add("Packet size: ");
		v1.add("Class of Service: ");
		//v1.add("Longueur des messages");
		Vector v2=new Vector();  // Valeur par defaut
		v2.add("MMPPNGeneFormat");
		v2.add("1000");
		v2.add("0");  // <=> pas de QOS 
		BoiteSaisies f = new BoiteSaisies(gestionUDP,"Scenario Player Parameter",v1,v2);
		Vector v3=f.getValeursChamps();
                try {
                
                if( setScenarioFile( (String)v3.get(0) ) ) {
					this.fr = new FileReader(file.getName());
                }
                else {
                	textAreaEtatApplication.append("File: " + (String)v3.get(0)   +" doesn't exist!");
                	return false; // si le fichier n'existe pas +> erreur sur la boite de saisie	
                }
		    	longueurMessages = Integer.parseInt((String)(v3.get(1)));
		    	cos = Integer.parseInt((String)(v3.get(2)));
                }
                catch (Exception e) {
		    		System.out.println(e.toString());
                }
                return f.getEtat();
    }
    
   public boolean setScenarioFile(String fileName) throws FileNotFoundException   {
        	if(FileTest.isExistent(fileName)) {
        		file = new File(fileName);
        		return true;
        	}
        	else {
        		System.out.println("File: " + fileName +" doesn't exist!" );
        	 	return false;
        	}
        }
    
    
}
