/**
 * GestionUDPS2S : Sauvegarde des rsultats du gestion ping-pong sur le disque dans le rpertoire courant.
 *                 Le tableau des rsultats contient pour chaque ping-pong effectu sa dure.
 */


package temp;
import java.io.*;
import java.util.*;


public class GestionUDPS2S {

    //Tableau contenant les rsultats du gestion.
    //Contient pour chaque ping-pong sa dure.
    protected int[] rsultatsGestion;
    protected int min,max;
    protected int longueurMessages;
    protected int nbPingPong;
    protected int tailleBufferReception, tailleBufferEmission;
    protected int tailleBufferRceptionPaquet;
    protected PrintWriter out;
    protected String nomFichier;

    /**
     * Initialise la sauvegarde des rsultats du gestion ping-pong sur le disque.
     * @param rsultatsGestion dure de chaque ping-pong
     * @param min dure minimale d'un ping-pong ou dure du ping-pong le plus rapide
     * @param max dure maximale d'un ping-pong ou dure du ping-pong le plus lent
     * @param longueurMessages longueur des messages utiliss pour effectuer le gestion
     * @param nbPingPong nombre de ping-pong du gestion
     * @param tailleBufferReception taille du buffer de rception du socket serveur
     * @param tailleBufferEmission taille du buffer d'mission du socket serveur
     * @param tailleBufferRceptionPaquet taille du buffer pour la rception de paquets
     */
    public GestionUDPS2S(int[] rsultatsGestion, int min, int max, int longueurMessages, int nbPingPong, int tailleBufferReception, int tailleBufferEmission, int tailleBufferRceptionPaquet) {
	this.rsultatsGestion=rsultatsGestion;
	this.min=min;
	this.max=max;
	this.longueurMessages=longueurMessages;
	this.nbPingPong=nbPingPong;
	this.tailleBufferReception=tailleBufferReception;
	this.tailleBufferEmission=tailleBufferEmission;
	this.tailleBufferRceptionPaquet=tailleBufferRceptionPaquet;
	sauvegarderRsultats();
    }
    
    protected void sauvegarderRsultats(){
	Date date=new Date();
	long time=date.getTime();
	nomFichier="ping-pong"+time+".txt";
	try{
	    out=new PrintWriter(new FileWriter(nomFichier));
	}
	catch (IOException e) {
	    System.out.println("GestionUDPS2S:sauvegarderRsultats => "+e.toString());
	}
	out.println("Rsultats du gestion ping-pong\r");
	out.println("-----------------------------------------------------------------------------------\r");
	out.println("Dure du ping-pong le plus rapide :\t"+min+"\r");
	out.println("Dure du ping-pong le plus lent :\t"+max+"\r");
	out.println("Longueur des messages utilise pour effectuer le gestion ping-pong :\t"+longueurMessages+"\r");
	out.println("Nombre de ping-pong effectus lors du gestion :\t"+nbPingPong+"\r");
	out.println("Taille du buffer de rception du socket du serveur :\t"+tailleBufferReception+"\r");
	out.println("Taille du buffer d'mission du socket du serveur :\t"+tailleBufferEmission+"\r");
	out.println("Taille du buffer pour la rception de paquets sur le serveur :\t"+tailleBufferRceptionPaquet+"\r");
	out.println("-----------------------------------------------------------------------------------\r");
	out.println("Prcisions :\r");
	out.println("Le protocole de la couche de Transport UDP assure que l'ordre d'arrive des messages et le mme que celui de l'mission.\r");
	out.println("Les dures sont donnes en millisecondes.\r");
	out.println("-----------------------------------------------------------------------------------\r");
	out.println("Dure pour chacun des ping-pong :\r");
	for (int i=0;i<rsultatsGestion.length;i++){
	    out.println(i+"\t"+rsultatsGestion[i]+"\r");
	}
	out.close();
    }
    
    /**
     * Renvoie le nom du fichier sous lequel ont t sauvegard les rsultats.
     * @return le nom du fichier
     */
    public String getNomFichier(){
	return nomFichier;
    }
 
    protected static void main(String args[]) {
	int[] i=new int[300];
	i[2]=20;i[10]=15;i[15]=20;i[25]=15;i[40]=25;i[45]=5;i[47]=2;i[49]=1;i[249]=5;
	GestionUDPS2S b=new GestionUDPS2S(i,2,249,20,1000,1000,1000,34);
    }
}
