/**
 * UDPClient : client UDP
 *
 * @author maria ladoue
 * @version 0.1, 04/07/02 
 */


package temp;
import java.io.*;
import java.net.*;

import javax.swing.*;   // ajouter pour la methode main uniquement

import traffic.*;
import utils.*;


/**
 * Cette classe permet de crer un client UDP.
 * A chaque connexion client-serveur, un DatagramSocket est cre.
 */

public class UDPClient {

    // Definition des variables

    // Numero du port du client(srcPort), numero du port(destPort) du serveur auquel se connecter,  
    public int srcPort, destPort;
    // Nom du serveur auquel se connecter
    public String host = null;
    // Pour recupere l'adresse locale
    public InetAddress localaddr;
    // Adresse IP du serveur auquel se connecter 
    protected InetAddress serverAddress = null;
    // Taille des buffers d'emission et de reception
    protected int sendBufferSize,receiveBufferSize;
    // Taille du buffer pour la reception des paquets
    protected int receivedPacketBufferSize;
    // L'instance de la classe qui va utiliser les mthodes de la classe SocketLecture
    protected UDPSocketReader udpSocketReader;
    // Classe de service dans laquelle sera envoye les paquets 
    public int cs;	
    //Le Thread permettant de lire sur le socket associ aux lectures
    protected SocketLecture socketLecture;
    //Le Thread permettant d'crire sur le socket associ aux critures
    protected SocketEcriture socketEcriture;
    //File de messages du client concernant l'tat de la connexion
    protected FileAttenteSynchronisee messagesClient;


    // Definition du constructeur

    /**
     * Constructeur : Etablit une connexion avec le serveur dont les attributs sont passes en parametres.
     *
     * @param host Le nom du serveur auquel se connecter
     * @param destPort Port du serveur auquel se connecter
     * @param srcPort Port du client
     * @param sendBufferSize Taille du buffer d'emission des datagrammes
     * @param receiveBufferSize Taille du buffer de reception des datagrammes
     * @param udpSocketReader Instance de la classe qui va utiliser les methodes de la classe SocketLecture 
     * @param receivedPacketBufferSize Taille du buffer pour la rception de paquets 
     * @param cs Classe de service
     */
    
    public UDPClient ( String unHote, int unPortDest, int unPortSrc, int tailleBufferReception, int tailleBufferEmission, UDPSocketReader reader, int tailleBufferRceptionPaquet, int uneCS) throws UnknownHostException,SocketException {

	this.destPort = unPortDest;
	this.host = unHote;
	this.sendBufferSize = tailleBufferEmission;
	this.cs = uneCS;
	this.srcPort = unPortSrc;
	this.receiveBufferSize = tailleBufferReception;
	this.receivedPacketBufferSize = tailleBufferRceptionPaquet;
	this.udpSocketReader = reader;
	messagesClient = new FileAttenteSynchronisee(); 

          try {
	      //DatagramSocket socket = new DatagramSocket(srcPort);
	      //socket.setSendBufferSize(sendBufferSize);
              //socket.setReceiveBufferSize(receiveBufferSize);
           ClasseService cs = new ClasseService(new DatagramSocket(srcPort),uneCS);
	      (cs.getDS()).setSendBufferSize(sendBufferSize);
	      (cs.getDS()).setReceiveBufferSize(receiveBufferSize);
	      
	      String message = "1 Hello server";
	      byte[] sendBuf = message.getBytes();
	      InetAddress adresseConnexion = InetAddress.getByName(host);
	      DatagramPacket packet = new DatagramPacket(sendBuf,sendBuf.length,adresseConnexion,destPort);
	      //socket.send(packet);
	      (cs.getDS()).send(packet);
	      
	      packet = new DatagramPacket(sendBuf,sendBuf.length);
	      //socket.receive(packet);
	      (cs.getDS()).receive(packet);
	      
	      int portConnection = packet.getPort();
	      messagesClient.ajouter("Connection on server " + "<" + adresseConnexion.getHostAddress() + " , " + portConnection + ">");
	      //socketEcriture = new SocketEcriture(socket,messagesClient,adresseConnexion,portConnection); 
	      //socketLecture = new SocketLecture(socket,socketEcriture,messagesClient,udpSocketReader,adresseConnexion,portConnection,receivedPacketBufferSize);
	      socketEcriture = new SocketEcriture(cs.getDS(),messagesClient,adresseConnexion,portConnection); 
	      socketLecture = new SocketLecture(cs.getDS(),socketEcriture,messagesClient,udpSocketReader,adresseConnexion,portConnection,receivedPacketBufferSize);
	  }
	  catch (IOException e) {
	      System.out.println("UDPClient:UDPClient : "+e.toString());
	  }
    }

       /**
        * Etablit une connexion avec le serveur dont les attributs sont passs en paramtres.
        * @param serverInetAddress L'adresse IP du serveur auquel se connecter
        * @param portServeur Le port du serveur auquel se connecter
        * @param portClient Le port du client
        * @param tailleBufferRception La taille du buffer de rception des datagrammes
        * @param tailleBufferEmission La taille du buffer d'mission des datagrammes
        * @param lecteur l'instance de la classe qui va utiliser les mthodes de la classe SocketLecture
        * @param tailleBufferRceptionPaquet Taille du buffer pour la rception de paquets (>=34 (23 (message fermeture connexion) + 11 (numrotation datagrammes)))
        */
        public UDPClient(InetAddress serverInetAddress, int portServeur, int portClient, int tailleBufferReception, int tailleBufferEmission, UDPSocketReader reader, boolean tcpNoDelay, int tailleBufferRceptionPaquet,int uneCS){
                this.serverAddress=serverInetAddress;
                this.destPort=portServeur;
                this.srcPort=portClient;
				this.cs = uneCS;
                this.receiveBufferSize=tailleBufferReception;
                this.sendBufferSize=tailleBufferEmission;
                this.udpSocketReader=reader;
                this.receivedPacketBufferSize=tailleBufferRceptionPaquet;
                messagesClient=new FileAttenteSynchronisee();
                try {
		   		 //DatagramSocket socket = new DatagramSocket(srcPort);
		    //socket.setSendBufferSize(tailleBufferEmission);
		    //socket.setReceiveBufferSize(tailleBufferReception);
		    ClasseService cs = new ClasseService(new DatagramSocket(srcPort),uneCS);
		    (cs.getDS()).setSendBufferSize(sendBufferSize);
		    (cs.getDS()).setReceiveBufferSize(receiveBufferSize);

		    String message="Hello server";
		    byte[] buf=message.getBytes();
		    DatagramPacket packet=new DatagramPacket(buf,buf.length,serverAddress,destPort);
		    //socket.send(packet);
		    (cs.getDS()).send(packet);

		    packet=new DatagramPacket(buf,buf.length);
		    //socket.receive(packet);
		    (cs.getDS()).receive(packet);

		    int portConnection=packet.getPort();
		    messagesClient.ajouter("Connexion au serveur " + "<" + serverAddress.getHostAddress() + " , " + portConnection + ">");
		    //socketEcriture=new SocketEcriture(socket,messagesClient,serverAddress,portConnection);
		    //socketLecture=new SocketLecture(socket,socketEcriture,messagesClient,udpSocketReader,serverAddress,portConnection,receivedPacketBufferSize);
		    socketEcriture = new SocketEcriture(cs.getDS(),messagesClient,serverAddress,portConnection); 
		    socketLecture = new SocketLecture(cs.getDS(),socketEcriture,messagesClient,udpSocketReader,serverAddress,portConnection,receivedPacketBufferSize);

                }
                catch (IOException e) {
		    System.out.println("UDPClient:UDPClient => "+e.toString());
                }
        }



    // Definition de quelques methodes
    
    public void setCS(DatagramSocket unDS,int uneCS) throws  SocketException {
		(new ClasseService()).setOption(0,new Integer(uneCS));
    }


    // Pour recuperer les differentes variables


    /**  public int getCS() throws  SocketException {
	return cs.getClasseService();
	}*/


    // Faire quelques tests: adresse multicast, adresse de loopback...
    public String natureAdresse(String host) throws SecurityException {
	try {
	    InetAddress ia = InetAddress.getByName(host);

	    if (ia.isLoopbackAddress()) {
		return "Loopback Address";
	    }
	    
	    if (ia.isMulticastAddress()) {
		return "Multicast Address";
	    }
	}
	catch (UnknownHostException e) {
	    System.out.println("Erreur dans l'adresse IP");
	    System.exit(1);
        }
	return "Non Specific Address";
    }
    
  
    /**
     * Renvoit le Thread associ  la lecture sur le socket de la connexion.
     * @return le Thread de lecture
     */
    public SocketLecture getThreadLecture() {
	return socketLecture;
    }
    
    /**
     * Renvoit le Thread associ  l'criture sur le socket de la connexion.
     * @return le Thread d'criture
     */
    public SocketEcriture getThreadEcriture() {
	return socketEcriture;
    }
    
    /**
     * Rcupre le premier message concernant l'tat de la connexion.
     * <br><b><u>ATTENTION</u></b>: il faut tester si la file est vide avant d'appeler cette mthode.
     * Si la file est vide, une exception FileVideException est leve.
     * @return une chane de caractres
     */
    public String getPremierMessageConnexion(){
	return messagesClient.retirer();
    }
    
    /**
     * Indique si il y a un message concernant l'tat de la connexion dans la file.
     * @return un boolen
     */
    public boolean fileMessageConnexionVide() {
	return messagesClient.vide();
    }
    
    /**
     * Methode: Main pour tester la classe
     */
    
    public static void main(String[] argv) {
	try{
	    GestionUDP gestionUDP = new GestionUDP();;
	    JTextArea textAreaEtatConnexion = new JTextArea();
	    JTextArea textAreaEtatApplication = new JTextArea();
	    JProgressBar progressBar =  new JProgressBar();

       	    UDPClient monPaquetGen = new UDPClient("kawa",1150,1160,256,256,new GestionUDPClient(gestionUDP,textAreaEtatConnexion,textAreaEtatApplication,progressBar),256,1);
	}
	catch(UnknownHostException e) {
	    System.out.println("The IP address of a host could not be determined.");  
	}
	catch(SocketException e) {
	    System.out.println("Traffic class could not be determined.");  
	}
	
    }

}
