package test.model;

import java.io.*;
import java.util.Date;
import java.util.Vector;

import multimedia.codec.AudioCodec;
import multimedia.codec.AudioCodecExample;
import multimedia.model.MMPPNModel;
import statistic.RandomGenerator;
import utils.*;
import utils.myPlotter.OneGraphPlotter;

/**
 * MMPPNTest
 * 
 * @author mladoue
 * @version 0.1, 28/10/02 
 */
public class MMPPNTest  extends GenericModelTest  implements RandomGenerator, AudioCodecExample {

 protected MMPPNModel model;

	public MMPPNTest(MMPPNModel model,int tpsSimul,String fichierReq,boolean graphic)  throws IOException{	
		this.model = model;
		this.tpsSimul = tpsSimul;
		resultFileName = fichierReq;
		//resultFile = new File(fichierReq);
		//this.fichierReq = FileTest.testExistent(resultFile.getName(),graphic);
	}
	
	/**
	 * Method getStateList. Create a file which presents states changement at one time
	 * @throws IOException
	 */
	public void getStateList() throws IOException {
	// Definition des variables
    double time = 0, s;
    int tpsSimulS = tpsSimul*60;  // on le passe en secondes
    int etat = model.getEtatDepart();
    double T1 = 0, p;
	
    // Creation du scenario a proprement parler: le temps entre l'emission de chaque paquet
    while ( time < tpsSimulS ) {
	
	T1 = model.calculT( (model.getParam(etat,"alpha") + model.getParam(etat,"beta")) );  // Donne la duree de l'etat actif
	p = random.nextDouble();  // Determination de l'etat suivant
	if ( p <= model.getParam(etat,"beta") / ( model.getParam(etat,"alpha") + model.getParam(etat,"beta") ) ) {
	    fichierReq.write(etat++); 
	}
	else { fichierReq.write(etat--); }
	
	time += T1;
    } //fin du while
    
	fichierReq.flush();
  	System.out.println("File " + resultFile.getName() + " created!");
	}
	
	public void traceStateEvolution() {
		// Definition des variables
		double time = 0, s;
		int tpsSimulS = tpsSimul*60;  // on le passe en secondes
		int etat = model.getEtatDepart();
		double T1 = 0, p;
		Vector stateList = new Vector();
		int cpt =0;
	
		while ( time < tpsSimulS ) {
	
		T1 = model.calculT( (model.getParam(etat,"alpha") + model.getParam(etat,"beta")) );  // Donne la duree de l'etat actif
		p = random.nextDouble();  // Determination de l'etat suivant
		if ( p <= model.getParam(etat,"beta") / ( model.getParam(etat,"alpha") + model.getParam(etat,"beta") ) ) {
			stateList.add(cpt, new Integer(etat++)); 
		}
		else { stateList.add(cpt, new Integer(etat--)); }
		cpt++;
		time += T1;
		} //fin du while
 		
 		int[] states = MyTableOperation.transformV2IT(stateList);   
 		
 		OneGraphPlotter rp = new OneGraphPlotter();
 		rp.plotIntTableData(states, tpsSimul, "States changement in the course of time", "Time","States number", true, false);
 		
	}
	
	
	/**
	 * Method getPacketNumberAtTime. Create a file which contains number of packet created per states
	 * @param duration
	 * @throws IOException
	 */
	public void getPacketNumberAtTime( double duration )  throws IOException {	
	 // Definition des variables
    double time = 0, s;
    int tpsSimulS = tpsSimul*60;  // on le passe en secondes
    int etat = model.getEtatDepart(), packetNb;
    double T1 = 0, T2 = 0, p, sOld=0d;
	
    // Creation du scenario a proprement parler: le temps entre l'emission de chaque paquet
    while ( time < duration ) {
	s = time;
	
	T1 = model.calculT( (model.getParam(etat,"alpha") + model.getParam(etat,"beta")) );  // Donne la duree de l'etat actif
	
	if (etat != 0) {   
		packetNb = 0;
	    while ( s < Math.min(time + T1 , tpsSimulS) ) {
		T2 = model.calculT( model.getParam(etat,"lambda") ); // Donne le temps d'inter-arrive dans l'etat courant
		if( (s + T2) < (time + T1) ) {   
		 	packetNb++;
		    s +=  T2;
		}		
		else { break; }
	    }  // end of while
	    fichierReq.write("Number of packets generated during states "+ etat +":  " + packetNb +  "\n");
	    
	}
	p = random.nextDouble();  // Determination de l'etat suivant
	if ( p <= model.getParam(etat,"beta") / ( model.getParam(etat,"alpha") + model.getParam(etat,"beta") ) ) {
	    etat++; 
	}
	else { etat--; }
	sOld = s;
	time += T1;
	    
    } //fin du while
    
	fichierReq.flush();
  	System.out.println("File " + resultFile.getName() + " created!");	
	}
	
	/**
	 * Methode: Main 
 	*/

    public static void main (String[] args) throws IOException {

	
	String s2 = "MMPPNGene";
	MMPPNModel mmpp20 = new MMPPNModel(20,1,3.12,0.78,new AudioCodec(1000,0.25));
	MMPPNTest mmpp20test = new MMPPNTest (mmpp20,20,s2,false);
	
	System.out.println("Debut: " + new Date().toString());
	//mmpp20test.getPacketNumberAtTime(20);
	mmpp20test.traceStateEvolution();
	System.out.println("Fin: " +new Date().toString());
    }
}
