/**
 * OnOffTest: Pour generer des fichiers permettant de tester la validite du modele OnOff
 *
 * @author maria ladoue
 * @version 0.1, 25/10/02 
 */

package test.model;

import java.io.*;
import java.util.Vector;

import statistic.distribution.*;
import statistic.*;
import utils.*;
import utils.myPlotter.PlotterProperties;
import utils.myPlotter.OneGraphPlotter;
import utils.myPlotter.TwoGraphPlotter;
import multimedia.codec.*;
import multimedia.model.OnOffModel;


public class OnOffTest extends GenericModelTest implements RandomGenerator,AudioCodecExample {

    protected OnOffModel model;
	protected final static int ON = 0;
	protected final static int OFF = 1;

 	public OnOffTest () { graphic = false; }

    /**
     * OnOffTest
     *     
     *@param model it must be an On-Off Model
     *@param tpsSimul it is the test duration in minutes
     *@param fichierReq it's a file where results will be enregistrated
     *@param graphic
     *@throws IOException
     */
    public OnOffTest (OnOffModel model,int tpsSimul,String fichierReq,boolean graphic) throws IOException {
		this.model = model;
		this.tpsSimul = tpsSimul;
		resultFile = new File(fichierReq);
		this.fichierReq = FileTest.testExistent(resultFile.getName(),graphic);
    }

 
 	public double[][] computePeriodOnDuration() {
 		
 		Vector periodOnDuration = new Vector();
		Vector periodOffDuration = new Vector();
		
		double time = 0, s, T1 = 0, T2 = 0, sOld=0d;
		int tpsSimulS = tpsSimul*60;  // on le passe en secondes
		int etat = model.getEtatDepart(), nbPacket = 0, nboctet = 0, cptOn = 0, cptOff =0;
		double tpsIP = (model.getCodec()).getTpsIP(); // info recuperer du codec

		while (time < tpsSimulS) {
	    
			s=time;		
			if (etat == 0) {
				T1 = model.calculT(etat);
				periodOnDuration.add(cptOn, new Double(T1));
				time += T1;
				cptOn++;
				etat = 1;
			} // fin du if    
			else {
				T2 = model.calculT(etat);
				periodOffDuration.add(cptOff, new Double(T2));
				time +=T2;
				cptOff++;
				etat = 0;
			} // fin du else
		}
		
		int onPeriodNumber = periodOnDuration.size();
		int offPeriodNumber = periodOffDuration.size();
	
		if(onPeriodNumber != offPeriodNumber) {
			int diff;
			if( (diff = (onPeriodNumber - offPeriodNumber)) > 0 ) {
				for (int i=0;i<diff;i++) {
					periodOffDuration.add(cptOff, new Double(0));
					cptOff++;
				}
			}
			else { 
				for (int i=0;i<diff;i++) {
					periodOnDuration.add(cptOn, new Double(0));
					cptOn++;
				}
			}
		}		
		
		double[] onPeriod = MyTableOperation.transformV2DT(periodOnDuration);
		double[] offPeriod = MyTableOperation.transformV2DT(periodOffDuration);
		
		double[][] ans = new double[2][onPeriodNumber];
			for (int i=0;i<onPeriodNumber;i++ ) {
				ans[0][i] = onPeriod[i];
				ans[1][i] = offPeriod[i];
		}
		return ans;
	
	}
 	
 	public void tracePeriodTimeEvolution() {
 		double[][] periods = computePeriodOnDuration();
 		double[] periodOn = MyTableOperation.transformDTT2DT( periods,0);
		double[] periodOff = MyTableOperation.transformDTT2DT( periods,1);
		
 		TwoGraphPlotter trp = new TwoGraphPlotter();
		trp.plotTwoDoubleTableData("Period On/Off Evolution in time", periodOn, periodOff, getTestDuration(), "Period On Evolution", "Period Off Evolution");
 		
 	}
 	
 	
 	public double[] rateEvolutionComputation() {
		double time = 0, s, T1 = 0, T2 = 0, sOld=0d;
		int tpsSimulS = tpsSimul*60;  // on le passe en secondes
		int etat = model.getEtatDepart(), nbPacket= 0, nbBytes=0,cpt=0;
		int size = model.getCodec().getSize();
		double tpsIP = (model.getCodec()).getTpsIP(); // info recuperer du codec
		Vector debit = new Vector();
		
		while (time < tpsSimulS) {
			s=time;		
			if (etat == 0) {
			T1 = model.calculT(etat);
			while( s < Math.min(time + T1, tpsSimulS) ) {
						nbPacket++; // Pour calculer le debit moyen...
						s += tpsIP;
						debit.add(cpt, new Double( ( nbBytes + nbPacket*size ) / s*8 /1000) );
						cpt++;
			} // fin du deuxieme while
				
			time += T1;
			etat = 1;
			} // fin du if    
			else {
				T2 = model.calculT(etat);
				time +=T2;
				etat = 0;
			} // fin du else
		}
		return MyTableOperation.transformV2DT(debit);
 	}

/**
 *  Several Methods which permit to plot some informations
 */

public void traceRateEvolution() {
	
	double[] rates = rateEvolutionComputation();
	OneGraphPlotter rp = new OneGraphPlotter();
	rp.plotDoubleTableData(rates, tpsSimul, true, false);
	PlotterProperties.setPlotProperties(rp.getSpecificPlot(0), 700, 600, "Througput (in kb/s)", 0, (int)Math.round(TestParameters.getMaximumDoubleValue(rates) + 1), 0, tpsSimul, "Time (in mn)", "Throughput (in kb/s)");
	rp.getSpecificPlot(0).setImpulses(false);
}

/**
 *  Several Methods which permit to create result file
 */
  
/**
 * Method genererPeriodFile. Permet de generer un fichier ou ne s'affiche que les periodes On ou que les periodes OFF
 * @param period ON ou OFF au choix
 * @throws IOException
 */
public void createPeriodFile( int period ) throws IOException {

	// Definition des variables
	double time = 0, s, T1 = 0, T2 = 0, sOld=0d;
	int tpsSimulS =  tpsSimul*60;  // on le passe en secondes
	int etat = model.getEtatDepart(), nbPacket = 0, nboctet = 0;
	
	double tpsIP = (model.getCodec()).getTpsIP(); // info recuperer du codec

	while (time < tpsSimulS) {
	    
	    s=time;		
	    if (etat == 0) {
		T1=model.calculT(etat);
		
		//System.out.println("Ton duration: " + T1);	 //DEBUG
		if (period == ON) { fichierReq.write(T1 + "\n"); }
		time += model.calculT(etat);
		etat = 1;
	    } // fin du if
	    
	    else {
		T2=model.calculT(etat);
		
		//System.out.println("Toff duration: " + T2); //DEBUG
		if (period == OFF) { fichierReq.write(T2 + "\n"); }
		time += model.calculT(etat);
		etat = 0;
	    } // fin du else
	    
	}  // fin du while
	fichierReq.flush();

	System.out.println("File " + resultFile.getName() + " created!");
}


/**
 * Method genererDistributionPacketFile. Definit afin d'etudier la repartion des paquets durant Ton
 * 	Cree un fichier indiquant le tps inter-paquet des paquets crees pendant une periode ON
 * 	Et en fin de period, on ajoute la phrase suivante:
 * 		Number of packet generated during this ON period:  <nbPacket>
 * @throws IOException
 */
 public void createDistributionPacketFile() throws IOException {
 	// Definition des variables
	double time = 0,s;
	int tpsSimulS =  tpsSimul*60;  // on le passe en secondes
	int etat = model.getEtatDepart();
	double T1 = 0, T2 = 0, sOld=0d;
	int nbPacket= 0, nboctet=0;

	double tpsIP =(model.getCodec()).getTpsIP(); // info recuperer du codec

	while (time < tpsSimulS) {
	    
	    s=time;		
	    if (etat==0) {
		T1=model.calculT(etat);
		nbPacket = 0;
		while( s < Math.min(time + model.calculT(etat),tpsSimulS) ) {	
		    fichierReq.write((s-sOld)*1000 + "\n");  // inter-packet time in s => in ms
		    nbPacket++; // Pour calculer le nb de paquet generer par Ton
		    sOld=s;
		    s += tpsIP;
		} // fin du deuxieme while
		fichierReq.write("Number of packet generated during this ON period: "+  nbPacket +" \n");  // inter-packet time in s => in ms
		time += model.calculT(etat);
		etat = 1;
	    } // fin du if
	    
	    else {
		T2=model.calculT(etat);
		time += model.calculT(etat);
		etat = 0;
	    } // fin du else
	    
	}  // fin du while

	fichierReq.flush();
	
	System.out.println("File " + resultFile.getName() + " created!");
 	
 }
    
/**
 * Methode: Main
 */

    public static void main (String[] args) throws IOException {

	String s1 ="OnOffGene";
	String s2 = "OnOffGeneText";

	OnOffModel oom1 = new OnOffModel(1,new ExponentialDistribution(2.8571429),new ExponentialDistribution(1.5384615),G711);//new AudioCodec(136,0.012));
	OnOffTest oog1 = new OnOffTest(oom1,200,s1,false);
	//OnOffTest oog2 = new OnOffTest(oom1,90,s2,false);
	//oog1.tracePeriodTimeEvolution();
	oog1.traceRateEvolution();

    }

}
