package traffic;

/**
 * SimpleUDPClient : Thread correspondant a un client UDP qui pourra etre lance en mode texte via RunUDPClient
 * 
 * @author mladoue
 * @version 0.1, 06/12/02 
 */

import java.io.*;
import java.net.*;

import utils.LocalUDPPortsScanner;

public class SimpleUDPClient extends Thread {

protected int srcPort;
protected int destPort;
protected String host;
protected DatagramSocket socketUDP;     
protected InetAddress serverAddress = null;
protected boolean isIpv4;

/**
 * Method SimpleUDPClient.
 * @param destPort	port du server auquel on veut se connecter
 * @param srcPort		port depuis lequel on se connecte et sur lequel le server enverra ses reponses 
 * @param host			nom du server
 */
public SimpleUDPClient( int destPort, int srcPort, String host, boolean isIpv4) throws IllegalArgumentException, NumberFormatException {
	if ( srcPort > 1024 && destPort > 1024 ) {
		this.srcPort = srcPort;
		this.destPort = destPort;
	}
	else {
	 	System.out.println("Error : Destination port: " + destPort + "and source port: " + srcPort + " must be greater than 1024!");	
	}
	this.host = host;
	this.isIpv4 = isIpv4;
	this.start();	
}

  public void run() {
  	
      DatagramPacket message;
    
      try {
	  		serverAddress = InetAddress.getByName(host);
		} 
      catch (UnknownHostException e) {
	 	 System.out.println("Error on serverAddress");
	 	 System.exit (1);
        }
      // Ouvrir un socket UDP      
      try {
      		if ( LocalUDPPortsScanner.isSpecificPortFree(srcPort) ) {
	 		 socketUDP = new DatagramSocket(srcPort); // ouverture s'un DatagramSocket sur un port donne
      		}
      		else {
      			System.out.println("The port: " + srcPort + " is already in use. We try to find another one....");
      			boolean free= false;
      			while(!free) {
      				srcPort++;
      				if(	LocalUDPPortsScanner.isSpecificPortFree(srcPort)) {
      					free = true;	
      				}
      			}
      			socketUDP = new DatagramSocket(srcPort);
      			System.out.println("We choose port number: "  + srcPort + ".");
      		}
		} 
      catch  (IOException e) {
	  			System.out.println("Error on DatagramSocket : DatagramSocket creation");
	  			e.printStackTrace();
	  		return;
        }
        
      // Envoyer un message a destination du port du server
      String data = "Hello server";
	  byte[] sendBuf = data.getBytes();
	  
      message = new DatagramPacket(sendBuf, sendBuf.length, serverAddress, destPort);
      
      try { 
	  socketUDP.send(message);
	   System.out.println("Send to: " + serverAddress + " on port: " + destPort);
		} 
      catch  (IOException e) {
	  	System.out.println("Emission failed ...");
	  	e.printStackTrace();
	  	return;
       }
        
     
      // Attendre la reponse emise par le serveur
      message = new DatagramPacket(sendBuf, sendBuf.length);
      try {
	  socketUDP.receive(message);
		} 
      catch  (IOException e) {
	  System.out.println("Reception failed...");
	  e.printStackTrace();
	  return;
        }
      
      // Visualisation du message recu    
      String received = new String(message.getData());
      System.out.println("Received: "+ received +  " on port: " + srcPort);
     
     
     try {
	InetAddress client = InetAddress.getLocalHost();
	System.out.println("Client created on " + client.getHostName() + ": destination/source port: " + destPort +"/"+ srcPort +", server name: " + host +".");
	}
	catch (UnknownHostException uhe) { uhe.printStackTrace(); }
	
     // Prepare to received all the datagrams (ie. the multimedia stream)
     int compt=0;
     while (true) {
		try {
				// To leave the wile (true) if there is no more datagrams.
			socketUDP.setSoTimeout(5000);
			socketUDP.receive(message);
			compt++;
			if ( compt != 0 && compt % 100 ==0)
				System.out.println("NBPAQUETS:"+compt);
		}
		catch (SocketTimeoutException se) {
			System.out.println("NBPAQUETS:"+compt);
			System.out.println("Transmission terminated: "+compt+" datagrams were received");
			return;
		}
		catch  (IOException e) {
			System.out.println("Reception failed...");
			e.printStackTrace();
			return;
		}
	}
    }  // end run
}






    
    
  


