package traffic;


/**
 * SocketLecture :Cette classe permet d'associer un Thread  un DatagramSocket pour la lecture de datagrammes.
 *                <br><u><b>ATTENTION</b></u>: les messages "demandefermeclosed13579" et "fermeclosed02468" sont rservs.
 *                <br>L'intrt de cette classe est d'viter l'appel bloquant aux fonctions de rception sur les sockets.
 *                 Le Thread scanne l'arrive de messages sur le socket et les stocke dans une file.
 *                 Pour cela, le Thread associ  l'envoi des datagrammes les numrote.
 *
 * @author 
 * @version 0.1, 04/07/02 
 */


import java.net.*;
import java.io.*;

import utils.*;

 
public class SocketLecture extends Thread {

    //Le DatagramSocket par lequel on reoit des datagrammes
    protected DatagramSocket socket;
    //La file de messages reus
    protected FileAttenteSynchronisee file;
    //Boolen indiquant si le Thread est termin
    protected boolean termine;
    //Le SocketEcriture associ au socket
    protected SocketEcriture socketEcriture;
    //File de messages concernant la connexion
    protected FileAttenteSynchronisee messages;
    //L'instance de la classe qui va utiliser les mthodes de la classe SocketLecture
    protected UDPSocketReader udpSocketReader;
    //Adresse IP de la machine distante
    protected InetAddress adresseConnexion;
    //Port de la machine distante
    protected int portConnexion;
    //Taille du buffer pour la rception de paquets; voir DatagramPacket(byte[]buf, intlength)
    protected int tailleBufferRceptionPaquet;
    

    
    /**
     * Cration du buffer de lecture des datagrammes reus par le socket.
     * @param socket Le DatagramSocket sur lequel recevoir des datagrammes
     * @param socketEcriture Le SocketEcriture associ au socket
     * @param messages file de messages concernant la connexion
     * @param lecteur l'instance de la classe qui va utiliser les mthodes de la classe SocketLecture
     * @param adresseConnexion Adresse IP de la machine distante
     * @param portConnexion Port de la machine distante
     * @param tailleBufferRceptionPaquet Taille du buffer pour la rception de paquets (>=34 (23 (message fermeture connexion) + 11 (numrotation datagrammes)))
     * ; voir DatagramPacket(byte[]buf, intlength)
     */
    
        public SocketLecture(DatagramSocket socket, SocketEcriture socketEcriture, FileAttenteSynchronisee messages,UDPSocketReader reader, InetAddress adresseConnexion, int portConnexion, int tailleBufferRceptionPaquet){
	    	this.socket = socket;
	    	this.socketEcriture = socketEcriture;
	    	this.messages = messages;
	    	this.udpSocketReader = reader;
	    	this.adresseConnexion = adresseConnexion;
	    	this.portConnexion = portConnexion;
	    	this.tailleBufferRceptionPaquet = tailleBufferRceptionPaquet;
	    	file = new FileAttenteSynchronisee();
	    	termine = false;
	    	this.start();
        }
    
    /**
     * Implantation de la mthode run de la classe Thread.
     * Rception des datagrammes sur le socket
     */
    public void run(){
	String message="";
	if (udpSocketReader==null)
	    throw new UDPSocketReaderIndefiniException("Le lecteur du SocketLecture est indefini.");

	while (!(message.equals("closed")) && !(message.equals("askclosed"))){  
	    message=recevoirMessage();
	    if (!(message.equals("closed")) && !(message.equals("askclosed"))){
		file.ajouter(message);
		udpSocketReader.wakeUp();
	    }
	}
	termine=true;
	if (message.equals("askclosed")){
	    socketEcriture.fermerSocket();
	}
	if (message.equals("closed")){
	    messages.ajouter("Fermeture de la connexion avec <" + adresseConnexion.getHostAddress() + " , " + portConnexion + ">");
	    socket.close();
	}
	//On rveille le lecteur car ce dernier peut tre en train d'attendre un message.
	//Ce dernier doit alors dtecter que le Thread est termin et que le socket va tre ferm.
	udpSocketReader.wakeUp();
    }
    
    /**
     * Reoit un datagramme et vrifie qu'UDP n'a pas fait d'erreur (bon acheminement, ordre d'arrive).
     * Renvoit le message contenu dans les donnes du datagramme.
     * Les donnes d'un datagramme sont composes de : numro du datagramme + espace + message
     * @return le message
     */
    protected synchronized String recevoirMessage(){
	
	String donnes = recevoirDatagramme();
	
	String message = lireMessage(donnes);
	return message;
    }
    
    /**
     * Reoit un datagramme et renvoit les donnes (datagramme moins son entte) qu'il contient.
     * @return les donnes contenues dans le datagramme.
     */
    protected synchronized String recevoirDatagramme(){
	String donnes="";
	try {
	    byte[] buf = new byte[tailleBufferRceptionPaquet];
	    DatagramPacket packet;
	    packet = new DatagramPacket(buf,buf.length);
	    socket.receive(packet);
	    String tmp = new String(packet.getData());
	    donnes = tmp.substring(0,packet.getLength());
	    
	}
	catch (IOException e) {
	    System.out.println("SocketLecture.recevoirDatagramme => "+e.toString());
	}
	catch (StringIndexOutOfBoundsException sioobe) {
		//System.out.println(donnes.charAt(0));
	    System.out.println("SocketLecture.recevoirDatagramme => "+ sioobe.toString());
	}
	return donnes;
    }
    
    /**
     * Renvoit le numro du datagramme contenu en dbut des donnes issues du datagramme
     */

    protected  int lireNoDatagramme(String donnes) throws IndexOutOfBoundsException {
	int no=0;
	int pos=0;
	while (donnes.charAt(pos)!=' ') {
	    no =  no*10 + donnes.charAt(pos)-48;  // on passe le chiffre lu precedement en dizaine
	    pos++;
	}
	return no;
    }

    /**
     * Renvoit le message contenu dans les donnes (datagramme moins son entte) du datagramme
     * Les donnes sont constitues du numro du datagramme, d'un espace et enfin du message
     */
    int noMessage = 1;
    protected String lireMessage(String data)  {
		int pos=0;
		try {
		//System.out.println(data + " " + data.length() + " "  +  noMessage );
		while (data.charAt(pos)!=' ') {
	    pos++;
		}
	}
	catch (StringIndexOutOfBoundsException e) {
		System.out.println(data + " " + data.length() + " "  +  noMessage );
		e.printStackTrace();
	}
	pos++;
	noMessage++;
	return data.substring(pos,data.length());
	
    }
    
    /**
     * Indique si il reste des messages non lus dans la file.
     * @return un boolen
     */
    public boolean pasDeMessagesReu(){
		return file.vide();
    }
    
    /**
     * Renvoit le premier message lu de la file de messages.
     * @return le message sous forme de chane de caractres
     */
    public String premierMessageReu(){
   	//ring ans;
	return file.retirer();
    }
    
    /**
     * Renvoit le socket par lequel on reoit des datagrammes
     * @return le socket
     */
    public DatagramSocket getSocket(){
	return socket;
    }
    
    /**
     * Mthode permettant de savoir si le Thread est termin.
     * @return un boolen
     */
    public boolean estTermin(){
	return termine;
    }
    
    /**
     * Permet de dterminer l'instance de la classe qui va utiliser les mthodes de la classe SocketLecture.
     * <br><b><u>ATTENTION</u></b>: cette mthode doit imprativement tre appele aprs l'instanciation de   
     * la classe, sinon une exception LecteurIndefiniException sera leve.
     * @param lecteur l'instance
     */
    public void setUDPSocketReader(UDPSocketReader reader){
	this.udpSocketReader = reader;
    }
    
    /**
     * Permet de rcuprer la taille du buffer pour la rception de paquets.
     * <br><b>DatagramPacket(byte[]buf, intlength)</b>
     * @return un entier correspondant  la taille
     */
    public int getTailleBufferRceptionPaquet(){
		return tailleBufferRceptionPaquet;
    }
    
    /**
 	* Methode: Main pour tester la classe
 	*/

    public static void main (String[] args) throws IOException {
    ChaineAleatoire ca = new ChaineAleatoire(1000-6);
    String random = ca.gnrerChane();
    String s = "00001" + " " + random;
   //System.out.println(lireNoDatagramme(s));
    //System.out.println( s );
   //System.out.println( lireMessage(s) );
    }
    
    
}
