package traffic;

/**
 * UDPServer : 	Cette classe permet de crer un serveur UDP => reception de paquet UDP + reponse
 * 					Ce serveur peut grer un nombre illimit de clients.
 * 					A chaque connexion client-serveur, un DatagramSocket est cre.
 *
 * @author maria ladoue
 * @version 0.1, 04/07/02 
 */


import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;   // ajouter pour la methode main

import utils.*;
import temp.GestionUDP;
import temp.GestionUDPServeur;

public class UDPServer extends Thread {

    // Definition des variables

    // Port sur lequel le serveur attend les connexions
    protected int srcPort;

    // socketUDP est le socket d'ecoute du serveur
    protected DatagramSocket socketUDP;

    // Booleen indiquant si il faut ou non attendre des connexions -> arreter/fermer le DatagramSocket
    protected boolean waitConnection;

    /**
     * Les Threads de lecture et criture associs au socket de la connexion client-serveur
     * sont situs au mme emplacement dans les vecteurs listeThreadLecture et listeThreadEcriture
     */

    // Vecteur listant les Threads permettant de lire sur les sockets associs aux lectures
    protected Vector listeThreadLecture;

    //Vecteur listant les Threads permettant d'crire sur les sockets associs aux critures
    protected Vector listeThreadEcriture;

    //Nombre de connexions = nb de sockets ouvertes
    protected int nbConnection;

    // Taille des buffers d'emission et de recpetion
    protected int sendBufferSize,receiveBufferSize;

    //File de messages du serveur concernant l'tat de la connexion
    protected  FileAttenteSynchronisee messagesServeur;

    //L'instance de la classe qui va utiliser les mthodes de la classe SocketLecture
    protected UDPSocketReader udpSocketReader;

    // Taille du buffer pour la reception des paquets
    protected int receivedPacketBufferSize;

    // Fichier scenario
    protected File f;
    protected FileReader fr;
 

    /**
     * Constructeur -> creation du ServerSocket
     *
     * @param srcPort Port sur lequel le serveur va attendre des connexions
     * @param receiveBufferSize Taille du buffer de rception des datagrammes
     * @param senbdBufferSize Taille du buffer d'mission des datagrammes
     * @param udpSocketReader Instance de la classe qui va utiliser les methodes de SocketLecture
     * @param receivedPacketBufferSize Taille du buffer pour la rception de paquets
     */

 //* @param fName the name of the file which will be played

    public UDPServer (int srcPort,int receiveBufferSize,int sendBufferSize, UDPSocketReader reader, int receivedPacketBufferSize)  {

	this.srcPort = srcPort;
	this.receiveBufferSize = receiveBufferSize;
	this.sendBufferSize = sendBufferSize;
	this.udpSocketReader = reader;
	this.receivedPacketBufferSize = receivedPacketBufferSize;
	//setScenarioFile(fName);
	//this.fr = new FileReader(f.getName());

	listeThreadLecture = new Vector();
	listeThreadEcriture = new Vector();
	nbConnection = 0;

	messagesServeur = new FileAttenteSynchronisee();  

	try {
	    socketUDP = new DatagramSocket(srcPort);  
	    //messagesServeur.ajouter("Cration du DatagramSocket sur lequel attendre les connexions des clients <" + socketUDP.getInetAddress().getHostAddress() + " , " + socketUDP.getLocalPort() + ">");
 		messagesServeur.ajouter("Cration du DatagramSocket sur lequel attendre les connexions des clients <" + socketUDP.getLocalAddress().getCanonicalHostName() + " , " + socketUDP.getLocalPort() + ">");
	}

	
	catch( NullPointerException npe) {
		npe.printStackTrace();
	}
	
	catch(IOException e) {
	    System.out.println("UDPServer: UDPServer =>" + e.toString());
	}
	
	catch(ArrayIndexOutOfBoundsException aioobe) {
	    System.out.println("UDPServer: UDPServer =>" + aioobe.toString());
	    aioobe.printStackTrace();
	}


	waitConnection = true;
	this.start();
    }

 
    /**
     * Methode run() de la classe Thread.<br>
     * Attend la connexion des clients. A chaque arrive d'un nouveau client,
     * elle lui dedie un DatagramSocket (i.e un port de la machine) auquel il pourra se "connecter".
     */

    public void run() {

	// Definition des variables

	boolean start = false;

	int destPort;  // Port de destination = port du client
	int newPort = srcPort;
	DatagramPacket packet = null;
	byte[] tampon = new byte[256];  // taille [par defaut a l'etablissement de la connexion
	DatagramSocket newDS; // A chaque nouveau client un DatagramSocket est cree
	InetAddress clientAddress;
	SocketEcriture socketEcriture;

	//BufferedReader br = new BufferedReader(fr);
	//String c;  // ligne courante du document a regarder
	//StringTokenizer st;
   

	try  {
		messagesServeur.ajouter("Client connection wait...");
	    while ( waitConnection || !start ) {
	  
		start = true;
		// Attendre un message emis par le client
		packet  = new DatagramPacket(tampon, tampon.length);
		socketUDP.receive(packet);
		//System.out.println("Message client: " + packet.getData().toString());
		
		if (waitConnection) {
			
		    // On recupere l'addresse IP et le port
		    clientAddress = packet.getAddress();
		    destPort = packet.getPort();
		
		    messagesServeur.ajouter("New connection with client " + "<" + clientAddress.getHostAddress() + " , " + destPort  + ">");
		    newPort++;

		    newDS = new DatagramSocket(newPort);
		    newDS.setReceiveBufferSize(receiveBufferSize);
		    newDS.setSendBufferSize(sendBufferSize);
		    
		    // Envoyer la reponse vers le client
		    packet = new DatagramPacket( tampon, tampon.length, clientAddress, destPort);
		    //System.out.println(packet.getData());
		    newDS.send(packet);
		    
		    socketEcriture = new SocketEcriture( newDS, messagesServeur, clientAddress, destPort);
		    listeThreadEcriture.add(socketEcriture);
		    listeThreadLecture.add(new SocketLecture(newDS,socketEcriture,messagesServeur,udpSocketReader,clientAddress,destPort,receivedPacketBufferSize));
			
		    nbConnection++;
		    	
	/**
			// Envoyer la reponse vers le client
		    //packet = new DatagramPacket( tampon, tampon.length, clientAddress, destPort);
		    //newDS.send(packet);
		   
		    String date = new Date().toString();
		    System.out.println("Debut d'emission: " + date);
		     messagesServeur.ajouter("Debut d'emission: " + date);
		     
		    while ( (c = br.readLine()) != null ) {
		  	//messagesServeur.ajouter("Emission de paquet");
			//newDS.send(packet);
			st = new StringTokenizer(c);
			
			String token1 = st.nextToken();
			String token2 = st.nextToken();
		
			
			// Emission du paquet
			if(nbTokens == 2) {
				byte[] tampon2 = new byte[1000];
				DatagramPacket message2 = new DatagramPacket (tampon2 , tampon2.length, clientAddress, destPort);
				newDS.send(message2);
				
			}
			
			if(nbTokens == 3) {
		 		try {
		 		String token3 = st.nextToken();
		  		byte[] tampon3      = new byte[Integer.valueOf(token3).intValue()];
		  		final DatagramPacket message3 = new DatagramPacket (tampon3, tampon3.length , clientAddress, destPort);
		 		 newDS.send(message3);
  			}
		  catch(NoSuchElementException e) {
		  System.out.println(e);	
		  }
			}		
			// Attente entre 2 paquets
			try {
			    this.sleep(Long.valueOf(token1).longValue(),Integer.valueOf(token2).intValue());
			}
			catch(InterruptedException e) {
			    System.out.println(e);
			}
			
		    } // end while
	
		    br.close();
		    date =new Date().toString();
		     messagesServeur.ajouter("Fin d'emission: " + date);
		    System.out.println("Fin d'emission: " + date);
		   
		} // fin du if
	    }  // fin du while
*/
	}  //end if
    }  // end while
	    socketUDP.close();
	    messagesServeur.ajouter("Fermeture du DatagramSocket pour la connexion des clients");
	    
	} // fin du try
	
	catch (UnknownHostException e) {
	    System.out.println("UDPServer:run : Don't know about host:" + packet.getAddress());
	    System.exit(1);
	} 
	catch (IOException e) {
	    System.out.println("UDPServer:run : " + e.toString() + " port : " +  packet.getPort());
	}
	
    } // fin de run

    /**
     * Methode pour arreter le serveur UDP: stopServer
     */
    
    public void fermetureServeur() {
	waitConnection = false;
	// il faut se connecter au serveur pour qu'il n'attende plus de connexion

	try {
	    String message="1 Hello server";
	    byte[] sendBuf = message.getBytes();
	    DatagramPacket packet = new DatagramPacket(sendBuf,sendBuf.length,InetAddress.getByName("localhost"),srcPort);
	    socketUDP.send(packet);
	}	
	catch (UnknownHostException e) {
	    System.out.println("The IP address of the local host could not be determined.");
	}
	catch (IOException e) {
	    System.out.println("UDPServer: => fermetureServeur() " + e.toString());
	}
    }

     /**
      * Renvoit le Thread associ  la lecture sur le socket d'une connexion.
      * @param n le numro de la connexion
      * @return le Thread de lecture
      */
    public SocketLecture getThreadLecture(int n){
	majVecteursThread();
	return (SocketLecture)(listeThreadLecture.get(n));
    }
    
    /**
     * Renvoit le Thread associ  l'criture sur le socket d'une connexion.
     * @param n le numro de la connexion
     * @return le Thread d'criture
     */
    public SocketEcriture getThreadEcriture(int n){
	majVecteursThread();
	return (SocketEcriture)(listeThreadEcriture.get(n));
    }
    
    // Renvoie le nombre de connections actives
    public int nbConnection() {
		return nbConnection;
    }
    
    //Examine pour chaque connexion si elle est termine et procde aux MAJ
   
    protected void majVecteursThread() {
	SocketEcriture se;
	SocketLecture sl;
	//On procde  une MAJ des vecteurs en examinant chaque connection
	for (int i=0;i<nbConnection;i++){
	    se=(SocketEcriture)(listeThreadEcriture.get(i));
	    sl=(SocketLecture)(listeThreadLecture.get(i));
	    //Test si:
	    //      les Threads de lecture et criture sont termins
	    //      la file des messages reus du Thread de lecture est vide
                        if (se.estTermin() && sl.estTermin() && sl.pasDeMessagesReu()){
			   			 listeThreadEcriture.removeElementAt(i);
			   			 listeThreadLecture.removeElementAt(i);
			    		nbConnection--;
                        }
	}
    }

    /**
     * Rcupre le premier message concernant l'tat de la connexion.
     * <br><b><u>ATTENTION</u></b>: il faut tester si la file est vide avant d'appeler cette mthode.
     * Si la file est vide, une exception FileVideException est leve.
     * @return une chane de caractres
     */

    public String getPremierMessageConnexion(){ return messagesServeur.retirer();  }

    /**
     * Indique si il y a un message concernant l'tat de la connexion dans la file.
     * @return un boolen
     */

    public boolean fileMessageConnexionVide(){ return messagesServeur.vide();  }
        
        public void setScenarioFile(String fileName) throws FileNotFoundException   {
        	if(FileTest.isExistent(fileName)) {
        		f = new File(fileName);
        	}
        	else {
        	System.out.println("File: " + fileName +" doesn't exist!" );	
        	}
        }

	public void putApplicationMessage(String message) {
		messagesServeur.ajouter("\n"+ message);
	}
	
  

    /**
     * Methode: Main pour tester la classe
     */

    public static void main (String[] args) throws IOException {
	GestionUDP gu = new GestionUDP();
	JTextArea textAreaEtatConnexion =  new JTextArea();
	JTextArea textAreaEtatApplication =  new JTextArea();
	JProgressBar progressBar =  new JProgressBar();
	
    GestionUDPServeur gus = new GestionUDPServeur(gu,textAreaEtatConnexion,textAreaEtatApplication,progressBar);
	System.out.println(gus.toString());
	UDPServer pc = new UDPServer(1120,1000,1000,gus,34);
    }
}
