package traffic.distributed;

/**
 * HostInformationsClient : Doit etre lancee sur la machine cliente
 * 
 * @author mladoue
 * @version 0.1, 12/02/03 
 */

import java.rmi.*;
import java.util.SortedSet;
import java.io.*;
import java.net.*;


public class HostInformationsClient {

	String serverName="", serverIPAddress="", clientName;
	int clientPort=0, serverPort=0, clientCos=0, scenarioFileNumber;
	boolean clientAddressType = true;
	String[] clientsList, scenarioFileList;
        int[] clientsPortList;
	SortedSet scenarioFiles;
	
	HostInformations infos;
	
	protected int type;
	protected  static int CLIENT = 0;
	protected  static int SERVER = 1;
	protected boolean rmiServerAvailable = false;
	

/**
 * Constructor HostInformationsClient.
 * 
 * @param hostName the name of the host where the server run
 * @param rmiPort  it's the port of the rmiregistrery server which permit to find the HostInformationsServer object
 * @param serviceName
 */
 public HostInformationsClient(String hostName, int rmiPort,String serviceName, int type) {
     this.type = type;
     
     try {	
         infos = (HostInformations)Naming.lookup("rmi://" + hostName + ":" + rmiPort  + "/" + serviceName );

	 InetAddress client = InetAddress.getLocalHost();
         
         if(type==CLIENT) {
             clientName = client.getHostName();
             String[] tmp = (clientName.replace('.',' ')).split(" ",3);
             clientName = tmp[0];
             clientPort = infos.getUDPClientPort(clientName);
             serverName = infos.getServerNameForClient(clientName, clientPort);
             clientCos =  infos.getClientCOS(clientName, clientPort);
             clientAddressType = infos.isClientIPv4(clientName, clientPort);

                 // Recup serverPort...
             serverPort = infos.getServerPortForClient(clientName, clientPort);
         }
         else { // We are a server... no clientName and serverName is hostname
             serverName = client.getHostName();
             String[] tmp = (serverName.replace('.',' ')).split(" ",3);
             serverName = tmp[0];
             serverPort = infos.getUDPServerPort(serverName);
         }

         serverIPAddress = infos.getServerIPAddress(serverName, serverPort);
         
         scenarioFileNumber = infos.getScenarioFileNumber(serverName, serverPort);
         clientsList= infos.getClientsList(serverName, serverPort);
         clientsPortList= infos.getClientsPortList(serverName, serverPort);
         scenarioFileList = infos.getScenarioFilesList(serverName, serverPort);
         scenarioFiles = infos.getScenarioFiles(serverName, serverPort);
         
         if (type==SERVER) {
                 // First, check if our clients are using Ipv4/Ipv6
             if (clientsList.length != 0 && clientsPortList.length !=0) {
                 clientAddressType=infos.isClientIPv4(clientsList[0], clientsPortList[0]);
             }
             else {
                 System.err.println("Grrr, check why a server has an empty list of clients");
             }
                 // Get the scenario file for this server.
             Object [] filesName = scenarioFiles.toArray();
             for(int i=0; i <filesName.length; i++) {
                 System.out.println("Ouvervure du fichier : "+(String)filesName[i]);
                 byte[] fileData = infos.transferScenario((String)filesName[i]);
                     // Remove path of filesNames[i] string if needed.
                 String[] tmp = ((String)filesName[i]).split("/");
                 System.out.println("===> filename="+tmp[tmp.length-1]);
                     // Write the file in tmp directory of server Machine.
                 File file = new File("/tmp/" + tmp[tmp.length-1]);
                 System.out.println(file.getAbsolutePath());
                 BufferedOutputStream output = new BufferedOutputStream( new FileOutputStream( file.getAbsolutePath() ));
                 output.write(fileData,0,fileData.length);
                 output.flush();
                 output.close();
             }

             for(int i=0; i <scenarioFileList.length; i++) {
                 String[] tmp = scenarioFileList[i].split("/");
                 scenarioFileList[i]="/tmp/"+tmp[tmp.length-1];
             }

             System.out.println("Files transfer succeded!");
         }
         
     }
     catch (java.net.UnknownHostException uhe) { uhe.printStackTrace(); }
     
     catch (MalformedURLException mue) { System.err.println("//" + hostName + ":" + rmiPort + "/" + serviceName + ": URL RMI invalid"); }
     
     catch (RemoteException re) { System.err.println("L'objet distant a leve l'exception : " + re); }
     
     catch (NotBoundException nbe) { System.err.println("L'objet distant demande n'a pas ete trouve sur le serveur distant!"); }

     catch (IOException ioe) { ioe.printStackTrace(); }
 }
	
	
	public String getServerName() { return serverName; }
	public int getUDPServerPort() { return serverPort; }
	public String getServerIPAddress() { return serverIPAddress; }
	public int getScenarioFileNumber() { return scenarioFileNumber; }
	public String[] getClientsList() { return clientsList; }
	public int[] getClientsPortList() { return clientsPortList; }
	public String[] getScenarioFilesList() { return scenarioFileList; }
	public SortedSet getScenarioFiles() { return scenarioFiles; }
	public int getUDPClientPort() { return clientPort; }
	public int getClientCOS() { return clientCos; }
	public boolean isClientIPv4() { return clientAddressType; }
 		
 }


