package traffic.distributed;

/**
 * HostInformationsImpl : Ce code sera lance sur la/les machines clientes (=> en demande de docs multimedia)
 * 						Il lancera un client UDP en attente sur un port specifique d'un document multimedia
 * 
 * @author maria ladoue
 * @version 0.1, 12/02/03 
 */

import java.io.*;
import java.rmi.*;
import java.rmi.server.*;
import java.util.SortedSet;

// For MessageManager class.
import utils.*;


public class HostInformationsImpl implements HostInformations {
	
    HostInformationsXMLFile hostInfoFile;
    String[] filesName;
    boolean graphic;


    public HostInformationsImpl() throws RemoteException, IOException {
        initialize("hostinfo.xml");
    }

    public HostInformationsImpl(String xmlhostfile) throws RemoteException, IOException {
        initialize(xmlhostfile);
    }

    public HostInformationsImpl(boolean graphic) throws RemoteException, IOException {
        this.graphic = graphic;
        initialize("hostinfo.xml");
    }
    
    protected void initialize(String xmlhostfile) throws RemoteException, IOException {
            // Load the information file
        hostInfoFile = new HostInformationsXMLFile(new String(xmlhostfile));
        
/*            // Populate the array of scenarii.
        filesName = new String[hostInfoFile.getScenarioFileNumber(hostInfoFile.serverName())];
        for (int i=0; i< hostInfoFile.getScenarioFileNumber(); i++)
        {
            String tmp = (String)getScenarioFiles().first();
            filesName[i] = tmp;
            getScenarioFiles().remove(tmp);
        }
*/      
            // Expose the interface to other (Remote) objects
        UnicastRemoteObject.exportObject(this);
    }

    public String[] getServerList() throws RemoteException {
        return  hostInfoFile.getServerList();
    }
    
    public String[] getJavaCmdServerList() throws RemoteException {
        return  hostInfoFile.getJavaCmdServerList();
    }
    
    public int[] getServerPortList() throws RemoteException {
        return  hostInfoFile.getServerPortList();
    }
    
    public String getServerNameForClient(String clientName, int port) throws IOException, RemoteException {
		return hostInfoFile.getServerNameForClient(clientName, port);
    }

    public int getServerPortForClient(String clientName, int port) throws IOException, RemoteException {
		return hostInfoFile.getServerPortForClient(clientName, port);
    }

    public int getUDPServerPort(String serverName)  throws IOException, RemoteException , NumberFormatException {
		return  hostInfoFile.getUDPServerPort(serverName);
    }

    public String getServerIPAddress(String serverName, int port) throws RemoteException {
		return hostInfoFile.getServerIPAddress(serverName, port);
    }

    public int getUDPClientPort(String clientName) throws RemoteException, NumberFormatException, IOException {
		return  hostInfoFile.getUDPClientPort(clientName);
    }

    public int getClientCOS(String clientName, int port) throws RemoteException, NumberFormatException, IOException {
		return  hostInfoFile.getClientCOS(clientName, port);
    }

    public boolean isClientIPv4(String clientName, int port) throws RemoteException, IOException {
		return hostInfoFile.isClientIPv4(clientName, port);
    }

    public String[] getScenarioFilesList(String serverName, int port)  throws RemoteException {
		return hostInfoFile.getScenarioFilesList(serverName, port); 	
    }

    public int getScenarioFileNumber(String serverName, int port) throws RemoteException {
		return hostInfoFile.getScenarioFileNumber(serverName, port);	
    }

    public SortedSet getScenarioFiles(String serverName, int port) throws RemoteException {
		return hostInfoFile.getScenarioFiles(serverName, port);
    }

    public String[] getClientsList(String serverName, int port)  throws RemoteException {
		return hostInfoFile.getClientsList(serverName, port);	
    }

    public int[] getClientsPortList(String serverName, int port)  throws RemoteException {
		return hostInfoFile.getClientsPortList(serverName, port);	
    }


    public String[] getClientsList()  throws RemoteException {
		return hostInfoFile.getClientsList();	
    }

    public String[] getJavaCmdClientsList()  throws RemoteException {
		return hostInfoFile.getJavaCmdClientsList();	
    }

    public int[] getClientsPortList()  throws RemoteException {
		return hostInfoFile.getClientsPortList();	
    }

    public String getAccessTypeForClient(String clientName, int port) throws RemoteException {
        return hostInfoFile.getAccessTypeForClient(clientName, port);
    }

    public String getAccessTypeForServer(String serverName, int port) throws RemoteException {
        return hostInfoFile.getAccessTypeForServer(serverName, port);
    }

/**
 * Permet de transferer un fichier dont le nom est passe en parametre
 *
 * @see traffic.generation.ScenarioInformations#transferScenarios(String[])
 */
    public byte[] transferScenario(String fileName) throws RemoteException, java.io.IOException {	    	
        try {
            File file = new File(fileName);
            byte buffer[] = new byte[ (int)file.length() ];
            BufferedInputStream input = new BufferedInputStream( new FileInputStream(fileName) );
            
            input.read(buffer,0,buffer.length);
            input.close();
            
            MessageManager.informationMessage( fileName + " transfer completed!", graphic);
            
            return buffer;
        }
        catch(Exception e) {
            System.out.println("HostInformationsImpl.transferScenarios: " + e.getMessage() );
            e.printStackTrace();
            return null;
        }
    }
    
//    public String[] getScenarioFileNames() throws RemoteException { return filesName; }
    
}
