package traffic.distributed;

/**
 * HostInformationsXMLFile: Use XML to implemente host information file reading
 *                          Extends the parent class to manage several servers.
 *
 * @author dsagnol
 * @version 0.2, 10/06/03
 */

import java.io.File;
import org.w3c.dom.Document;
import org.w3c.dom.*;


import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;


/*
 * Constraint: The name of the client must be uniq... A same client can not be used
 * to contact more than ONE server
 */
public class HostInformationsXMLFile extends HostInformationsFile {

    // Specify which method to use (rsh, ssh, etc.) to start a new java prog on a remote host.
    protected String[] serversAccessType;
    protected String[] clientsAccessType;

    protected String[] serversJavaCmd;
    protected String[] clientsJavaCmd;
    
    public HostInformationsXMLFile(String hostinfofile) throws java.io.IOException {
        super(hostinfofile);
    }

    public void readFile() {
        try {
                // Load XML file and parse it.
            DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
            Document doc = docBuilder.parse (new File(hostinfofile));
            
             // normalize text representation
            doc.getDocumentElement ().normalize ();
            
            // Quick check that the file starts with Configuration tag.
            if (doc.getDocumentElement().getNodeName() != "Configuration") {
                System.out.println ("Wrong XML file, check it !");
                System.exit(-1);
            }
            else {
                System.out.println("Parsing Configuration file for "+ doc.getDocumentElement().getAttribute("name")); 
            }
            NodeList listOfServers = doc.getElementsByTagName("Server");
            NodeList listOfAllClients = doc.getElementsByTagName("Client");
            System.out.println("Find: " + listOfServers.getLength()+" server(s)");
            System.out.println("Find: " + listOfAllClients.getLength()+" client(s)");
            
                // Instanciate Server arrays.
            serverNames = new String[listOfServers.getLength()];
            ipAddress = new String[listOfServers.getLength()];
            serverPort = new int[listOfServers.getLength()];
            serversAccessType = new String[listOfServers.getLength()];
            serversJavaCmd = new String[listOfServers.getLength()];
            alreadyServedServer=new boolean[listOfServers.getLength()];
            
                // Instanciate Client arrays.
            machinesNb = listOfAllClients.getLength();
            clientsName = new String[machinesNb];
            scFileName = new String[machinesNb];
            clientsPort = new int[machinesNb];
            clientsCos = new int[machinesNb];
            clientsIpv4 = new boolean[machinesNb];
            clientsAccessType = new String[machinesNb];
            clientsJavaCmd = new String[machinesNb];
            serverToContact = new int[machinesNb];
            alreadyServedClient = new boolean[machinesNb];

                // At the end, must be equal to machinesNb !
            int incNumberClient=0;
                // Get information for the servers
            for(int s=0; s<listOfServers.getLength() ; s++){
                Node currentServerNode = listOfServers.item(s);
                if (currentServerNode.getNodeType() == Node.ELEMENT_NODE) {
                    NamedNodeMap slistAttr = currentServerNode.getAttributes();
                        // Server Name
                    Node name = slistAttr.getNamedItem( "name");
                    System.out.print(name.getNodeValue()+" ");
                    serverNames[s] = name.getNodeValue();
                        // Server IP Adresse
                    Node ipadr = slistAttr.getNamedItem( "ipadr");
                    ipAddress[s] = ipadr.getNodeValue();
                    System.out.print(ipAddress[s]+" ");
                        // Server Port
                    Node port = slistAttr.getNamedItem( "port");
                    System.out.print(port.getNodeValue()+ " ");
                    serverPort[s]=Integer.parseInt(port.getNodeValue());
                        // Remote invocation type
                    Node actype = slistAttr.getNamedItem( "accesstype");
                    System.out.print(actype.getNodeValue()+" ");
                    serversAccessType[s]=actype.getNodeValue();
                        // Java Cmd
                    Node javacmd = slistAttr.getNamedItem( "javacmd");
                    System.out.println(javacmd.getNodeValue());
                    serversJavaCmd[s]=javacmd.getNodeValue();

                        // intialization.
                    alreadyServedServer[s]=false;
                        // Client/Server IP Protocol
                    Node ipprot = slistAttr.getNamedItem( "ipprot");
                    
                        // Now, read the clients that had to connect to this server.
                        // Take care here that not all child node are "Client" node
                    NodeList listOfClients = currentServerNode.getChildNodes();

                        // We Have to check that all child node are "client" nodes
                    for (int i=0; i <  listOfClients.getLength() ; i++) {
                        Node currentClientNode = listOfClients.item(i);
                        if (currentClientNode.getNodeType() == Node.ELEMENT_NODE && currentClientNode.getNodeName()=="Client") {
                            NamedNodeMap clistAttr = currentClientNode.getAttributes();
                		// Client Name
                            Node cname = clistAttr.getNamedItem( "name");
                            System.out.print("  "+cname.getNodeValue()+" ");
                            clientsName[incNumberClient]=cname.getNodeValue();
                		// Client Port
                            Node cport = clistAttr.getNamedItem( "port");
                            System.out.print(cport.getNodeValue()+ " ");
                            clientsPort[incNumberClient]=Integer.parseInt(cport.getNodeValue());
                                // Client IP protocol
                            System.out.print(ipprot.getNodeValue()+ " ");
                            if (ipprot.getNodeValue().equalsIgnoreCase("ipv4"))
                                clientsIpv4[incNumberClient]=true;
                            else
                                clientsIpv4[incNumberClient]=false;
                                // Client scenario file
                            Node scfile = clistAttr.getNamedItem( "scfile");
                            System.out.print(scfile.getNodeValue()+ " ");
                            scFileName[incNumberClient]=scfile.getNodeValue();
                                // Client COS
                            Node cos = clistAttr.getNamedItem( "cos");
                            System.out.print(cos.getNodeValue()+ " ");
                            clientsCos[incNumberClient]=Integer.parseInt(cos.getNodeValue());
                                // Remote invocation type
                            Node cactype = clistAttr.getNamedItem( "accesstype");
                            System.out.print(cactype.getNodeValue()+" ");
                            clientsAccessType[incNumberClient]=cactype.getNodeValue();
                                // Java Cmd
                            Node cjavacmd = clistAttr.getNamedItem( "javacmd");
                            System.out.println(cjavacmd.getNodeValue());
                            clientsJavaCmd[incNumberClient]=cjavacmd.getNodeValue();

                                // Finally, add a reference to its server.
                            serverToContact[incNumberClient]=s;

                                // intialization.
                            alreadyServedClient[incNumberClient]=false;
                            
                            incNumberClient++;
                        }
                    }
                }
            }
            if (incNumberClient != machinesNb) {
                System.out.println("Please check xml file, it seems that there is a problem with client number");
            }
        }
        catch (SAXParseException err) {
            System.out.println ("** Parsing error" + ", line " + err.getLineNumber () + ", uri " + err.getSystemId ());
            System.out.println(" " + err.getMessage ());
        }
        catch (SAXException e) {
            Exception x = e.getException ();
            ((x == null) ? e : x).printStackTrace ();
        }
        catch (Throwable t) {
            t.printStackTrace ();
        }
    }

    public String getAccessTypeForServer(String serverName, int port) {
        int k=-1;
        for(int i=0; i < serversAccessType.length; i++) {
            if (serverNames[i].equals(serverName) && serverPort[i]==port) {k=i;}
        }
        if (k == -1) { throw new IllegalArgumentException("HostInformationsXMLFile.getAccessTypeForServer : Cannot get Server access type information for server "+serverName);}
        
        return serversAccessType[k];
    }

    public String getAccessTypeForClient(String clientName, int port) {
        int k=-1;
        for(int i=0; i < clientsAccessType.length; i++) {
            if (clientsName[i].equals(clientName) && clientsPort[i]==port) {k=i;}
        }
        if (k == -1) { throw new IllegalArgumentException("HostInformationsXMLFile.getAccessTypeForClient  : Cannot get client access type information for server "+clientName);}

        return clientsAccessType[k];
    }

    public String[] getJavaCmdClientsList() {
        return clientsJavaCmd;
    }

    public String[] getJavaCmdServerList() {  return serversJavaCmd;  }
    

    public static void main(String[] arg) {
        try {
            HostInformationsXMLFile test = new HostInformationsXMLFile("hostinfo.xml");
        }
        catch (java.io.IOException ioe) {
            System.out.println(ioe);
        }
    }
}
