package utils.myPlotter;

import java.awt.BorderLayout;
import java.awt.Container;
import java.io.IOException;
import javax.swing.*;

import multimedia.VideoDefinition;
import multimedia.VideoExample;
import multimedia.codec.VideoCodec;
import multimedia.model.MGInfModel;

import statistic.distribution.*;
import statistic.function.*;
import ptolemy.plot.*;


/**
 * MGInfAnalysePlotter
 * 
 * @author mladoue
 * @version 0.1, 17/04/03
 */

public class MGInfAnalysePlotter extends JFrame implements VideoExample {

	protected Plot downPlot, upPlot;
	protected Container contentPane;
	protected MixedFunction autoCorr;
	protected double[] weight;
	protected GenericContinuousDistribution distributionTest;
	protected int analyseDuration;
	protected VideoCodec codec;


public MGInfAnalysePlotter(String plotTitle, MGInfModel mginfModel, int testDuration, VideoCodec codec ) {
	
	this.analyseDuration = testDuration;
	this.autoCorr = (MixedFunction)mginfModel.getAutoCorr();
	this.weight = ((MixedFunction)autoCorr).getWeight();
	this.distributionTest = mginfModel.getDistributionTest();
	this.codec = codec;
	
	//	Instantiate the two plots.
	 downPlot = new Plot();
	 upPlot = new Plot();
	
	 // Set the size of the toplevel window.
	 setSize(750, 650);
	 this.setTitle("Allegro Plotter");
	
	PlotterProperties.setPlotProperties(upPlot, 350, 300, "CDF de la taille des GOP", 0,1, 0, testDuration,"time","Period duration (in ms)");
	PlotterProperties.setPlotProperties(downPlot, 350, 300, "Fonction d'auto-correlation" , 0, 1, 0, testDuration,"time","Period duration (in ms)");
	 
	plotInformations(40000,1000);
	
	contentPane = this.getContentPane();
	contentPane.setLayout(new BorderLayout());
	
	contentPane.add(upPlot,BorderLayout.NORTH);
	contentPane.add(downPlot,BorderLayout.SOUTH);
	
	show();
	
}

public MGInfAnalysePlotter(String plotTitle, VideoDefinition vd) {
	
	this.analyseDuration = vd.getTpsSimul();
	this.autoCorr = (MixedFunction)vd.getModel().getAutoCorr();
	this.weight = ((MixedFunction)autoCorr).getWeight();
	this.distributionTest = vd.getModel().getDistributionTest();
	codec = vd.getCodec();
	
	//	Instantiate the two plots.
	 downPlot = new Plot();
	 upPlot = new Plot();
	
	 // Set the size of the toplevel window.
	setSize(750, 650);
	this.setTitle("Allegro Plotter");
	
	int movieSize = getDocumentSize();
	//System.out.println("Document size: " + movieSize/1000000 + " MB.");  //DEBUG
	
	
	PlotterProperties.setPlotProperties(upPlot, 350, 300, "CDF de la taille des GOP", 0, 1, 0, movieSize/1000,"Size (in bytes)","cdf");
	PlotterProperties.setPlotProperties(downPlot, 350, 300, "Fonction d'auto-correlation" , 0, 1, 0, 1000,"No correlation","");
	PlotterProperties.setXTick(downPlot, 1000d);
	PlotterProperties.setYTick(downPlot, 1d);
	
	plotInformations(movieSize/1000,1000);
	
	contentPane = this.getContentPane();
	contentPane.setLayout(new BorderLayout());
	
	contentPane.add(upPlot,BorderLayout.NORTH);
	contentPane.add(downPlot,BorderLayout.SOUTH);
	
	show();
	
}



public void plotInformations(int movieSize, int s) throws IllegalArgumentException{

	for (int i = 0; i<movieSize ; i++) {
		upPlot.addPoint(0, (double)i, distributionTest.cdf( (double)i ), true);
	}
	for (int i=0;i<s;i++) {	
		downPlot.addPoint(0, (double)i, autoCorr.function(i), true);
	}
	
}


public int getDocumentSize() {
	int N = (int)(analyseDuration*60 * (((VideoCodec)codec).getFramesNumber()) / (((VideoCodec)codec).getCompression().getGOP().getLength()));
	double slotDuration = (double)analyseDuration/(double)N;  // in minutes 
	int movieSize = (int)((distributionTest.mean())/slotDuration)*analyseDuration;
	return movieSize;
}

public static void main(String args[]) throws IOException {
	GenericFunction[] gf = {new SRDFunction(0.04)};
	double[] weight = { 1d } ;
	MixedFunction mf = new MixedFunction(gf,weight);
	MGInfModel mginf = new MGInfModel(mf,2d,new GammaDistribution(5d,1d/10d),new ExponentialDistribution(0.2d),new ExponentialDistribution(0.3d));
	
	//MGInfAnalysePlotter twoPlotExample = new MGInfAnalysePlotter("OnOff_30Format.scf",mginf,60);
	MGInfAnalysePlotter twoPlotExample = new MGInfAnalysePlotter("Jurrassik Park Movie Analyse", JurassicParkLow);
}



}
